/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.alignment;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;

import org.synteny.Protein;

import process.StreamReaderThread;

public class MuscleMultipleSequenceAligner implements MultipleSequenceAligner{

	private Collection<Protein> _proteins;
	private String _textAlignment;
	private String _htmlAlignment;
	private String _musclePath;

	public MuscleMultipleSequenceAligner(String musclePath){
		_proteins = new ArrayList<Protein>();
		_musclePath = musclePath;
		_textAlignment = new String();
		_htmlAlignment = new String();
	}

	public void alignSequences() {
		Runtime r = Runtime.getRuntime();
		Process p;
		try {
			p = r.exec(_musclePath+" -maxiters 10 -html -quiet");
			System.out.println(_musclePath+" -maxiters 10 -quiet -html -in - -out -");

			PrintWriter pw = new PrintWriter(p.getOutputStream());
			for(Protein prot : _proteins){
				pw.println(prot.toFasta());
				System.out.println(prot.toFasta());
			}
			pw.println();
			System.out.println();
			pw.flush();
			pw.close();
			System.out.println("Fin Input muscle");

			String line= "";
			StreamReaderThread errThread = new StreamReaderThread(p.getErrorStream());
			StreamReaderThread outThread = new StreamReaderThread(p.getInputStream());

			errThread.start();
			outThread.start();
			
			System.out.println(p.waitFor());
			errThread.join();
			outThread.join();
			
			_textAlignment = outThread.getResult();
			_htmlAlignment = outThread.getResult();
			System.err.print(errThread.getResult());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public String getHtmlAlignement() {
		return _htmlAlignment;
	}

	public Collection getProtein() {
		return _proteins;
	}

	public String getTextAlignement() {
		return _textAlignment;
	}

	public void setProtein(Collection protein) {
		_proteins.addAll(protein);
	}

}
