/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny;

import java.util.ArrayList;
import java.util.Collection;

public class BlocImpl implements Bloc{

	Comparison _comparison;
	Collection<Paralog> _paralogs;
	
	public BlocImpl(Comparison c){
		_comparison = c;
		_paralogs = new ArrayList<Paralog>();
	}
	
	public Comparison getComparison() {
		return _comparison;
	}

	public Collection getParalog() {
		return _paralogs;
	}

	public void setComparison(Comparison comparison) {
		_comparison = comparison;
	}

	public void setParalog(Collection paralog) {
		_paralogs = paralog;
	}

	public void addParalog(Paralog paralog) {
		_paralogs.add(paralog);
		paralog.addBloc(this);
	}

	public int getNbParalogs() {
		return _paralogs.size();
	}

	public int getMaximumSize() {
		int maxPos = 0;
		for(Object o : getParalog()){
			Paralog p = (Paralog)o;
			int s = p.getOrtholog().size();
			if(s>maxPos)
				maxPos = s;
		}
		return (maxPos);
	}
	
	public int getGenomeStart(){
		int minPos = getComparison().getRefGenome().getSize();
		for(Object o : getParalog()){
			Paralog p = (Paralog)o;
			Ortholog minO = (Ortholog)(p.getOrtholog().get(0));
			int pos1 = minO.getRefProtein().getGenomePosition();
			if(pos1<minPos){
				minPos = pos1;
			}
		}
		return (minPos);
	}
	
	public void delete(){
		Comparison _comparison;
		for(Object o : _paralogs){
			Paralog p = (Paralog)o;
			p.delete();
		}
		_paralogs = null;
	}

	public Protein getFirstReferenceProtein() {
		Protein prot = null;
		for(Object o : getParalog()){
			Paralog p = (Paralog)o;
			for(Object o2 : p.getOrtholog()){
				Ortholog ort = (Ortholog)o2;
				if(prot==null || ort.getRefProtein().getGenomePosition()<prot.getGenomePosition()){
					prot=ort.getRefProtein();
				}
			}
		}
		return prot;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Bloc o){
		Bloc bloc = (Bloc) o;
		return (getFirstReferenceProtein().getGenomePosition() - bloc.getFirstReferenceProtein().getGenomePosition());
	}
}


