/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


public class ComparisonImpl implements Comparison {

	Genome _compGenome;
	Genome _refGenome;
	List<Bloc> _blocs;
	Synteny _synteny;

	public ComparisonImpl(Genome compGenome,Genome refGenome) {
		_compGenome = compGenome;
		_refGenome = refGenome;
		_blocs = new ArrayList<Bloc>();
	}

	public List getBloc() {
		return _blocs;
	}

	public Genome getCompGenome() {
		return _compGenome;
	}

	public Genome getRefGenome() {
		return _refGenome;
	}

	public Synteny getSynteny() {
		// TODO Auto-generated method stub
		return null;
	}

	public void setBloc(List bloc) {
		_blocs = bloc;
	}

	public void setCompGenome(Genome compGenome) {
		_compGenome = compGenome;
	}

	public void setRefGenome(Genome refGenome) {
		_refGenome = refGenome;

	}

	public void setSynteny(Synteny synteny) {
		_synteny = synteny;
	}

	public void addParalog(Paralog paralog) {
		boolean added = false;
		for(Bloc b : _blocs){
			Paralog toAdd = null;
			Iterator<Paralog> ite = b.getParalog().iterator();
			while(ite.hasNext() && !added){
				Paralog p = ite.next();
				if(p.intersect(paralog)){
					toAdd = paralog;
					added = true;
				}
			}
			if(toAdd!=null){
				b.addParalog(toAdd);
			}
		}
		if(!added){
			Bloc b = new BlocImpl(this);
			b.addParalog(paralog);
			_blocs.add(b);
		}
	}

	public int getNbBlocs() {
		return _blocs.size();
	}

	public int compareTo(Comparison arg0) {
		double similarity1;
		double similarity2;
		
		similarity1 = getCompGenome().taxonomicSimilarity(getRefGenome());
		similarity2 = arg0.getCompGenome().taxonomicSimilarity(arg0.getRefGenome());
		
		if(similarity1>similarity2) return -1;
		else if(similarity1==similarity2) return 0;
		else return 1;
	}

	public Map getBlocSizeDistribution() {
		HashMap<Integer, Integer> result = new HashMap<Integer, Integer>();
		for(Object o1 : getBloc()){
			Bloc b = (Bloc)o1;
			for(Object o2 : b.getParalog()){
				int sizeParal;
				Paralog p = (Paralog)o2;
				sizeParal = p.getOrtholog().size();
				if(!result.containsKey(sizeParal))
					result.put(sizeParal, 0);
				result.put(sizeParal,result.get(sizeParal)+1);
			}
		}
		return result;
	}
	
	public void delete(){
		_refGenome = null;
		for(Object o : _blocs){
			Bloc b = (Bloc)o;
			b.delete();
		}
		_blocs = null;
		_synteny = null;
		_compGenome.clearProtList();
		_compGenome = null;
	}
}
