/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.synteny.factory.HttpSpeciesTreeFactory;

public class GenomeImpl implements Genome {

	SpeciesTree _spTree;
	List<Protein> _proteinSet;
	// Pour retrouver les protéines plus vites!
	Map<Integer,Protein>  _proteinIdMap;
	
	Comparison _compComparison;
	Collection<Comparison> _refComparisons;
	
	//List<Protein> _proteinSet;
	
	public GenomeImpl(SpeciesTree spt){
		_proteinIdMap = new HashMap<Integer, Protein>();
		_refComparisons = new ArrayList<Comparison>();
		_spTree = spt;
		_proteinSet = new ArrayList<Protein>();
	}
	
	public Comparison getCompComparison() {
		return _compComparison;
	}

	public List getProtein() {
		return _proteinSet;
	}

	public Collection getRefComparison() {
		return _refComparisons;
	}

	public SpeciesTree getSpeciesTree() {
		return _spTree;
	}

	public void setCompComparison(Comparison compComparison) {
		_compComparison = compComparison;
	}

	public void setProtein(List protein) {
		// TODO Auto-generated method stub

	}

	public void setRefComparison(Collection refComparison) {
		_refComparisons = refComparison;
	}

	public void setSpeciesTree(SpeciesTree speciesTree) {
		_spTree = speciesTree;
	}

	public String getSpeciesAbbrv() {
		return _spTree.getSpAbbrv();
	}

	public String getSpeciesName() {
		return _spTree.getName();
	}

	public void addProtein(Protein protein) {
		//int position = getInsertPosition(protein);
		_proteinSet.add(protein);
		//_proteinSet.add(position,protein);
		_proteinIdMap.put(protein.getIdentifier(),protein);
	}

	public int getSize() {
		return _spTree.getSize();
	}

	public void setSize(int size) {
		_spTree.setSize(size);
	}
	
	public String toString(){
		return getSpeciesName();
	}

	public int compareTo(Genome o) {
		return (getSpeciesName().compareTo(o.getSpeciesName()));
	}

	public double taxonomicSimilarity(Genome genome) {
		List<SpeciesTree> path1 = _spTree.getTaxonomy();
		List<SpeciesTree> path2 = genome.getSpeciesTree().getTaxonomy();
		int tailleMin;
		int tailleMax;
		if(path1.size()<path2.size()){
			tailleMin = path1.size();
			tailleMax = path2.size();
		}else{
			tailleMax = path1.size();
			tailleMin = path2.size();
		}
		
		double similarity = 0.0;
		for(int i=0;i<tailleMin;i++){
			if(path1.get(i).equals(path2.get(i))){
				similarity+=1.0;
			}
		}
		return (similarity/tailleMax);
	}

	public void addRefComparison(Comparison comparison) {
		_refComparisons.add(comparison);
	}

	public boolean containsProtein(int pid) {
		return _proteinIdMap.containsKey(pid);
	}

	public Protein getProtein(int pid) {
		return _proteinIdMap.get(pid);
	}

	public int getGenomePositionOfProtein(ProteinImpl proteinImpl) {
		return getProtein().indexOf(proteinImpl);
	}
	
	public static void main(String[] args) {
		HttpSpeciesTreeFactory hstf = new HttpSpeciesTreeFactory();
		SpeciesTree st = hstf.newSpeciesTreeInstance();
		GenomeImpl gi = new GenomeImpl(st);
		
		Protein p1 = new ProteinImpl(2);
		p1.setGenomeStartStop(100, 1000);
		Protein p2 = new ProteinImpl(3);
		p2.setGenomeStartStop(1500, 1700);
		Protein p3 = new ProteinImpl(4);
		p3.setGenomeStartStop(2000, 2100);
		Protein p4 = new ProteinImpl(5);
		p4.setGenomeStartStop(2200, 2100);
		Protein p5 = new ProteinImpl(6);
		p5.setGenomeStartStop(222000, 2100);
		Protein p6 = new ProteinImpl(7);
		p6.setGenomeStartStop(222001, 2100);
		
		gi.addProtein(p1);
		gi.addProtein(p3);
		gi.addProtein(p4);
		gi.addProtein(p2);
		gi.addProtein(p6);
		gi.addProtein(p5);
		Collections.sort(gi.getProtein());
	}
	
	public void clearProtList(){
		_proteinSet.clear();
		_proteinIdMap.clear();
	}
}
