/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;


public class ParalogImpl implements Paralog {

	int _identifier;
	Collection<Bloc> _blocs;
	List<Ortholog> _orthologs;
	
	public ParalogImpl(int paraId){
		_identifier = paraId;
		_blocs = new ArrayList<Bloc>();
		_orthologs = new ArrayList<Ortholog>();
	}
	
	public Collection getBloc() {
		return _blocs; 
	}

	public List getOrtholog() {
		return _orthologs;
	}

	public void setBloc(Collection bloc) {
		_blocs = bloc;
	}

	public void setOrtholog(List ortholog) {
		_orthologs = ortholog;
	}

	public int getIdentifier() {
		return _identifier;
	}
	
	public boolean equals(Object obj) {
		if (obj instanceof ParalogImpl) {
			ParalogImpl new_name = (ParalogImpl) obj;
			return new_name.getIdentifier()==getIdentifier();
		}
		return false;
	}

	public void addOrtholog(Ortholog ortho) {
		Iterator<Ortholog> iterator = _orthologs.iterator();
		int place = 0;
		boolean trouve = false;
		while(iterator.hasNext() && !trouve){
			Ortholog orth = iterator.next();
			if(orth.getRefProtein().getGenomeStart()>ortho.getRefProtein().getGenomeStart()){
				trouve = true;
				_orthologs.add(place, ortho);
			}
			place++;
		}
		if(!trouve){
			_orthologs.add(ortho);
		}
	}

	public boolean intersect(Paralog paralog) {
		for(Object o1 : paralog.getOrtholog()){
			for(Object o2 : getOrtholog()){
				if(((Ortholog)o1).getRefProtein().getIdentifier()==((Ortholog)o2).getRefProtein().getIdentifier()){
					return true;
				}
			}
		}
		return false;
	}
	
	public void delete(){
		_blocs = null;
		for(Object o : _orthologs){
			Ortholog or = (Ortholog)o;
			or.delete();
		}
		_orthologs = null;
	}
	
	public void addBloc(Bloc b){
		_blocs.add(b);
	}
}
