/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import org.synteny.search.ProteinInfoSearch;


public class ProteinImpl implements Protein {

	int _proteinId;
	boolean _strand;
	String _name;
	Genome _genome;
	int _start;
	int _stop;
	String _chromosome;
	String _function;
	String _sequence;
	Collection<Genome> _genomesMatchs;
	ProteinInfoSearch _search;
	Collection<Ortholog> _ortholog;

	public ProteinImpl(int proteinId){
		_genomesMatchs = new HashSet<Genome>();
		_proteinId = proteinId;
		_start = -1;
		_stop =-1;
		_chromosome = null;
		_function = null;
		_ortholog = new HashSet<Ortholog>();
	}

	public Genome getGenome() {
		return _genome;
	}

	public void setGenome(Genome genome) {
		_genome = genome;

	}

	public int getIdentifier() {
		return _proteinId;
	}

	public String getFunction() {
		if(_function==null)
			setInfos();
		return _function;
	}

	public boolean equals(Object o){
		if (o instanceof ProteinImpl) {
			ProteinImpl new_name = (ProteinImpl) o;
			return new_name.getIdentifier()==getIdentifier();
		}
		return false;
	}

	public boolean getStrand() {
		return _strand;
	}

	public void setStrand(boolean strand) {
		_strand = strand;
	}

	public String getName() {
		if(_name==null)
			setInfos();
		return _name;
	}

	public void setName(String name) {
		_name = name;
	}

	public int getGenomePosition() {
		return getGenome().getGenomePositionOfProtein(this);
	}

	public int getGenomeStart() {
		if(_start==-1)
			setInfos();
		return _start;
	}

	public int getGenoneStop() {
		if(_stop==-1)
			setInfos();
		return _stop;
	}

	public void setGenomeStartStop(int start, int stop) {
		_start = start;
		_stop = stop;
	}

	public int compareTo(Protein o) {
		if(o.getChromosome()==null || getChromosome()==null || o.getChromosome().equals(getChromosome())){
			return getGenomeStart()-o.getGenomeStart();
		}
		else{
			return getChromosome().compareTo(o.getChromosome());
		}
	}

	public String getChromosome() {
		return _chromosome;
	}

	public void setChromosome(String chromosome) {
		_chromosome = chromosome;

	}
	
	public String getSequence(){
		if(_sequence==null)
			setInfos();
		return _sequence;
	}

	/**
	 * @param p Proteine
	 * @return TRUE if the protein in argument has an ortholog in a synteny bloc in the comparison
	 */
	public int matchInOtherComparisonNumber(List<Genome> genomeList) {
		int i=0;
		for(Genome g : genomeList){
			if(_genomesMatchs.contains(g)){
				i++;
			}
		}
		return i;
	}

	public void addMatchInComparison(Genome g){
		if(!_genomesMatchs.contains(g))
			_genomesMatchs.add(g);
	}
	
	public void delete(){
		_name = null;
		_genome = null;
		_chromosome = null;
		_genomesMatchs = null;
		_search = null;
	}

	public void setFunction(String function) {
		_function = function;
	}

	public ProteinInfoSearch getProteinInfoSearch() {
		return _search;
	}

	public void setProteinInfoSearch(ProteinInfoSearch proteinInfoSearch) {
		_search = proteinInfoSearch;
	}
	
	private void setInfos(){
		Map m = _search.search(this);
		
		if(_function==null){
			_function = (String)(m.get("function"));
		}
		if(_chromosome==null){
			_chromosome = (String)(m.get("chromosome"));
		}
		if(_name==null){
			_name = (String)(m.get("name"));
		}if(_sequence==null){
			_sequence = (String)m.get("sequence");
		}
		if(_start==-1 || _stop==-1){
			_start = Integer.parseInt((String)(m.get("start")));
			_stop = Integer.parseInt((String)(m.get("stop")));
		}
	}
	
	public String toString(){
		return getName();
	}
	
	public Collection<Ortholog> getOrthologs(){
		return _ortholog;
	}
	
	public void addOrtholog(Ortholog ortho){
		_ortholog.add(ortho);
	}
	
	public String toFasta(){
		int i=0;
		String sequence = getSequence();
		StringBuffer sb = new StringBuffer();
		sb.append(">"+getIdentifier()+"_"+getGenome().getSpeciesAbbrv()+" "+getGenome().getSpeciesName()+" "+getName()+" - "+"\n");
		while(i<(sequence.length()-60)){
			sb.append(sequence.substring(i, i+60));
			sb.append("\n");
			i+=60;
		}
		sb.append(sequence.substring(i,sequence.length()));
		sb.append("\n");
		return sb.toString();
	}
	
	public boolean functionMatch(String pattern){
		return (getFunction().toLowerCase().contains(pattern.toLowerCase()));
	}
	
	public boolean geneNameMatch(String pattern){
		return (getName().toLowerCase().contains(pattern.toLowerCase()));
	}
}
