/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.swing.tree.TreeNode;


public interface SpeciesTree extends TreeNode{

	/**
	 * @return  Returns the genome.
	 * @uml.property  name="genome"
	 * @uml.associationEnd  inverse="speciesTree:org.synteny.Genome"
	 * @uml.association  name="hasForTaxonomy"
	 */
	public Genome getGenome();

	/**
	 * Setter of the property <tt>genome</tt>
	 * @param genome  The genome to set.
	 * @uml.property  name="genome"
	 */
	public void setGenome(Genome genome);

	/**
	 * @return  Returns the subSpeciesTree.
	 * @uml.property  name="subSpeciesTree"
	 * @uml.associationEnd  multiplicity="(0 -1)" inverse="parentSpeciesTree:org.synteny.SpeciesTree"
	 */
	public List getSubSpeciesTree();

	/**
	 * Setter of the property <tt>subSpeciesTree</tt>
	 * @param subSpeciesTree  The subSpeciesTree to set.
	 * @uml.property  name="subSpeciesTree"
	 */
	public void setSubSpeciesTree(List subSpeciesTree);

	/**
	 * @return  Returns the parentSpeciesTree.
	 * @uml.property  name="parentSpeciesTree"
	 * @uml.associationEnd  inverse="subSpeciesTree:org.synteny.SpeciesTree"
	 */
	public SpeciesTree getParentSpeciesTree();

	/**
	 * Setter of the property <tt>parentSpeciesTree</tt>
	 * @param parentSpeciesTree  The parentSpeciesTree to set.
	 * @uml.property  name="parentSpeciesTree"
	 */
	public void setParentSpeciesTree(SpeciesTree parentSpeciesTree);

	/**
	 * @return  Returns the rootSpeciesTree.
	 * @uml.property  name="rootSpeciesTree"
	 * @uml.associationEnd  inverse="speciesTree1:org.synteny.SpeciesTree"
	 */
	public SpeciesTree getRootSpeciesTree();

	/**
	 * Setter of the property <tt>rootSpeciesTree</tt>
	 * @param rootSpeciesTree  The rootSpeciesTree to set.
	 * @uml.property  name="rootSpeciesTree"
	 */
	public void setRootSpeciesTree(SpeciesTree rootSpeciesTree);


	/**
	 */
	public abstract void addSpecies(List<String> taxonomy , String sp_abbrv, String species_name, int nbProt);


	/**
	 */
	public abstract String getName();

	public abstract void setName(String name);

	public abstract String getSpAbbrv();

	public abstract void setAbbrv(String abbrv);

	public abstract Map<String, SpeciesTree> getAssocAbbreviationTree();


	/**
	 */
	public abstract Genome getGenome(String speciesabbrv);


	/**
	 */
	public abstract int getSize();


	/**
	 */
	public abstract void setSize(int size);


	/**
	 * Methods which returns all the genomes which are sons of the current node
	 */
	public abstract Collection<Genome> getGenomes();


	/**
	 */
	public abstract List getTaxonomy();
	
	/**
	 * @return The total number of leaves descending from this node 
	 */
	public int getLeafCount();
}
