/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.tree.TreeNode;

import sun.security.action.GetLongAction;


public class SpeciesTreeImpl implements SpeciesTree,Enumeration{

	List<SpeciesTree> _subSpeciesTrees;
	String _name;
	String _sp_abbrv;
	Genome _genome;
	SpeciesTree _root;
	SpeciesTree _parent;
	Map<String,SpeciesTree> _assocAbbrvTree;
	int _nbProt;
	Iterator _iterator;

	public SpeciesTreeImpl(String name){
		_root = null;
		_parent = null;
		_subSpeciesTrees = new ArrayList<SpeciesTree>();
		_name = name;
		_sp_abbrv = null;
	}

	public SpeciesTreeImpl(SpeciesTree root,String name,SpeciesTree parent){
		_root = root;
		_subSpeciesTrees = new ArrayList<SpeciesTree>();
		_name = name;
		_parent = parent;
		_sp_abbrv = null;
	}

	public Genome getGenome() {
		if(_subSpeciesTrees.size()==0 && _genome==null){
			_genome = new GenomeImpl(this);
			_genome.setSpeciesTree(this);
		}
		return _genome;
	}

	public SpeciesTree getParentSpeciesTree() {
		return _parent;
	}

	public SpeciesTree getRootSpeciesTree() {
		return _root;
	}

	public List getSubSpeciesTree() {
		return _subSpeciesTrees;
	}

	public void setGenome(Genome genome) {
		_genome = genome;
	}

	public void setParentSpeciesTree(SpeciesTree parentSpeciesTree) {
		_parent = parentSpeciesTree;
	}

	public void setRootSpeciesTree(SpeciesTree rootSpeciesTree) {
		_root = rootSpeciesTree;
	}

	public void setSubSpeciesTree(List subSpeciesTree) {
		_subSpeciesTrees = subSpeciesTree;
	}

	/* (non-Javadoc)
	 * @see org.synteny.SpeciesTree#addSpecies(java.util.List, java.lang.String, java.lang.String)
	 */
	public void addSpecies(List taxonomy, String sp_abbrv, String species_name, int nbProt) {
		if(taxonomy.size()==0){
			_name = species_name;
			_sp_abbrv = sp_abbrv;
		}else{
			if(taxonomy.get(0).equals(_name)){
				if(taxonomy.size()>2){
					boolean found = false;
					Iterator<SpeciesTree> it = _subSpeciesTrees.iterator();
					while(it.hasNext() && !found){
						SpeciesTree st = it.next();
						if(st.getName().equals(taxonomy.get(1))){
							taxonomy.remove(0);
							st.addSpecies(taxonomy, sp_abbrv, species_name,nbProt);
							found = true;
							break;
						}
					}
					if(!found){
						taxonomy.remove(0);
						SpeciesTree sti = new SpeciesTreeImpl(_root,(String)taxonomy.get(0),this);
						if(_subSpeciesTrees.contains(sti)){
							sti = _subSpeciesTrees.get(_subSpeciesTrees.indexOf(sti));
						}else{
							_subSpeciesTrees.add(sti);
						}
						sti.addSpecies(taxonomy, sp_abbrv, species_name,nbProt);
					}
				} else{
					taxonomy.remove(0);
					SpeciesTree sti = new SpeciesTreeImpl(_root,species_name,this);
					sti.setAbbrv(sp_abbrv);
					sti.setSize(nbProt);
					if(!_subSpeciesTrees.contains(sti))
						_subSpeciesTrees.add(sti);
				}
			}
		}
	}

	public String getName() {
		return _name;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		return _name+" ("+getLeafCount()+")";
	}

	public String getSpAbbrv() {
		return _sp_abbrv;
	}

	public void setAbbrv(String abbrv) {
		_sp_abbrv = abbrv;
	}

	public void setName(String name) {
		_name = name;
	}

	public Map<String, SpeciesTree> getAssocAbbreviationTree() {
		Map<String, SpeciesTree> map;
		if(_root == null){
			if(_assocAbbrvTree==null)
				_assocAbbrvTree = new HashMap<String, SpeciesTree>();
			map = _assocAbbrvTree;
		}else{
			map = new HashMap<String, SpeciesTree>();
		}

		if(map.size()==0){
			if(_subSpeciesTrees.size()==0){
				HashMap<String, SpeciesTree> m = new HashMap<String, SpeciesTree>();
				m.put(getSpAbbrv(), this);
				return m;
			}else{
				for(SpeciesTree spt : _subSpeciesTrees){
					Map<String, SpeciesTree> m = spt.getAssocAbbreviationTree();
					for(String key : m.keySet()){
						map.put(key, m.get(key));
					}
				}
			}
		}
		return map;
	}

	/* (non-Javadoc)
	 * @see org.synteny.SpeciesTree#getGenome(java.lang.String)
	 */
	public Genome getGenome(String speciesabbrv) {
		return getAssocAbbreviationTree().get(speciesabbrv).getGenome();
	}


	public int getSize() {
		return _nbProt;
	}

	public void setSize(int size) {
		_nbProt = size;
	}


	public Enumeration children() {
		_iterator = _subSpeciesTrees.iterator();
		return this;
	}

	public boolean getAllowsChildren() {
		return true;
	}

	public TreeNode getChildAt(int childIndex) {
		return _subSpeciesTrees.get(childIndex);
	}

	public int getChildCount() {
		return _subSpeciesTrees.size();
	}

	public int getIndex(TreeNode node) {
		return _subSpeciesTrees.indexOf(node);
	}

	public TreeNode getParent() {
		return _parent;
	}

	public boolean isLeaf() {
		return (_sp_abbrv!=null);
	}

	public boolean hasMoreElements() {
		return _iterator.hasNext();
	}

	public Object nextElement() {
		return _iterator.next();
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object o){
		if (o instanceof SpeciesTree) {
			SpeciesTree new_name = (SpeciesTree) o;
			return new_name.getName().equals(getName());
		}
		return false;
	}

	public int hashCode() {
		return getName().hashCode();
	}


	public Collection<Genome> getGenomes() {
		Collection<Genome> coll = new ArrayList<Genome>();
		if(isLeaf()){
			coll.add(getGenome());
		}else{
			for(SpeciesTree st : _subSpeciesTrees){
				coll.addAll(st.getGenomes());
			}
		}
		return coll;
	}

	public List getTaxonomy() {
		List<SpeciesTree> taxo = new ArrayList<SpeciesTree>();
		SpeciesTree current = this;
		while(!current.equals(getRootSpeciesTree())){
			taxo.add(0, current);
			current = current.getParentSpeciesTree();
		}
		return taxo;
	}
	
	/* (non-Javadoc)
	 * @see org.synteny.SpeciesTree#getLeafCount()
	 */
	public int getLeafCount(){
		if(isLeaf())
			return 1;
		else{
			int count = 0;
			
			for( Object o : getSubSpeciesTree()){
				SpeciesTree s = (SpeciesTree)o;
				count+= s.getLeafCount();
			}
			return count;
		}
	}
}