/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny;

import java.util.List;

import org.synteny.factory.ComparisonFactoryListener;
import org.synteny.factory.ComparisonsFactory;
import org.synteny.factory.ProteinInfoSearchFactory;
import org.synteny.factory.SpeciesTreeFactory;
import org.synteny.ihm.SyntenyListener;

import java.util.Collection;
import org.synteny.task.Task;


public interface Synteny extends ComparisonFactoryListener{

	/**
	 * @return            Returns the comparison.
	 * @uml.property   name="comparison"
	 * @uml.associationEnd   multiplicity="(0 -1)" ordering="true" container="java.util.List" inverse="synteny:org.synteny.Comparison"
	 * @uml.association   name="hasComparisons"
	 */
	public List getComparison();

	/**
	 * Setter of the property <tt>comparison</tt>
	 * @param comparison  The comparison to set.
	 * @uml.property  name="comparison"
	 */
	public void setComparison(List comparison);

	public void addComparison(Comparison comp);

	public Comparison getComparison(Genome g);

	public void setReferenceSpecies(Genome g,Collection<Genome> compGenomes);


	/**
	 * @return        Returns the comparisonsFactory.
	 * @uml.property   name="comparisonsFactory"
	 * @uml.associationEnd   inverse="synteny:org.synteny.factory.ComparisonsFactory"
	 * @uml.association   name="hasCompFactory"
	 */
	public ComparisonsFactory getComparisonsFactory();

	/**
	 * Setter of the property <tt>comparisonsFactory</tt>
	 * @param comparisonsFactory  The comparisonsFactory to set.
	 * @uml.property  name="comparisonsFactory"
	 */
	public void setComparisonsFactory(ComparisonsFactory comparisonsFactory);

	/**
	 * @return        Returns the speciesTreeFactory.
	 * @uml.property   name="speciesTreeFactory"
	 * @uml.associationEnd   inverse="synteny:org.synteny.factory.SpeciesTreeFactory"
	 * @uml.association   name="hasSpTreeFactory"
	 */
	public SpeciesTreeFactory getSpeciesTreeFactory();

	/**
	 * Setter of the property <tt>speciesTreeFactory</tt>
	 * @param speciesTreeFactory  The speciesTreeFactory to set.
	 * @uml.property  name="speciesTreeFactory"
	 */
	public void setSpeciesTreeFactory(SpeciesTreeFactory speciesTreeFactory);

	/**
	 * @return        Returns the speciesTree.
	 * @uml.property   name="speciesTree"
	 * @uml.associationEnd   inverse="synteny:org.synteny.SpeciesTree"
	 */
	public SpeciesTree getSpeciesTree();

	/**
	 * Setter of the property <tt>speciesTree</tt>
	 * @param speciesTree  The speciesTree to set.
	 * @uml.property  name="speciesTree"
	 */
	public void setSpeciesTree(SpeciesTree speciesTree);

	/**
	 * @return        Returns the syntenyListener.
	 * @uml.property   name="syntenyListener"
	 * @uml.associationEnd   multiplicity="(0 -1)" inverse="synteny:org.synteny.ihm.SyntenyListener"
	 * @uml.association   name="hasListener"
	 */
	public Collection getSyntenyListener();

	/**
	 * Setter of the property <tt>syntenyListener</tt>
	 * @param syntenyListener  The syntenyListener to set.
	 * @uml.property  name="syntenyListener"
	 */
	public void setSyntenyListener(Collection syntenyListener);


	/**
	 */
	public abstract void addSyntenyListener(SyntenyListener listener);

	public abstract void removeSyntenyListener(SyntenyListener listener);
	
	public abstract Genome getReferenceSpecies();
}
