/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny;

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.synteny.factory.ComparisonsFactory;
import org.synteny.factory.GenomeFactory;
import org.synteny.factory.SpeciesTreeFactory;
import org.synteny.ihm.SyntenyListener;


public class SyntenyImpl implements Synteny{
	ComparisonsFactory _compFactory;
	SpeciesTreeFactory _stf;
	GenomeFactory _genomeFactory;
	Genome _reference;
	List<Comparison> _comparisons;
	Map<Genome, Comparison> _mapGenomeComparison;

	SpeciesTree _speciesTree;
	Collection<SyntenyListener> _syntenyListeners;
	
	public SyntenyImpl(SpeciesTreeFactory sf, ComparisonsFactory cf, GenomeFactory genomeFactory){
		_stf = sf;
		_syntenyListeners = new ArrayList<SyntenyListener>();
		_speciesTree = _stf.newSpeciesTreeInstance();
		_genomeFactory = genomeFactory;
		_compFactory = cf;
		cf.addComparisonFactoryListener(this);
		_comparisons = new ArrayList<Comparison>();
		_mapGenomeComparison = new HashMap<Genome, Comparison>();
	}
	
	public List getComparison() {
		return _comparisons;
	}

	public void setComparison(List comparison) {
		_comparisons.clear();
		_mapGenomeComparison.clear();
		for(Object c : comparison){
			addComparison((Comparison)c);
		}
	}

	public void addComparison(Comparison comp) {
		_comparisons.add(comp);
		_mapGenomeComparison.put(comp.getCompGenome(), comp);
	}

	public Comparison getComparison(Genome g) {
		return _mapGenomeComparison.get(g);
	}

	public void setReferenceSpecies(Genome g,Collection<Genome> compGenomes) {
		try {
			delete();
			_reference = _genomeFactory.newGenomeInstance(g.getSpeciesTree());
			for(Object o: _reference.getProtein()){
				Protein p = (Protein)o;
				System.out.print(p.getGenomeStart()+"--");
			}
			_speciesTree.getAssocAbbreviationTree().get(g.getSpeciesAbbrv()).setGenome(_reference);
			List<Comparison> listTemp = getComparisonsFactory().newComparisonsList(_speciesTree,_reference,compGenomes);
			Collections.sort(listTemp);
			setComparison(listTemp);
			refSpeciesChanged();
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void refSpeciesChanged() {
		for(SyntenyListener sl : _syntenyListeners){
			sl.referenceSpeciesChanged();
		}
	}

	public ComparisonsFactory getComparisonsFactory() {
		return _compFactory;
	}

	public SpeciesTreeFactory getSpeciesTreeFactory() {
		return _stf;
	}

	public void setComparisonsFactory(ComparisonsFactory comparisonsFactory) {
		_compFactory = comparisonsFactory;
	}

	public void setSpeciesTreeFactory(SpeciesTreeFactory speciesTreeFactory) {
		_stf = speciesTreeFactory;		
	}

	public SpeciesTree getSpeciesTree() {
		return _speciesTree;
	}

	public void setSpeciesTree(SpeciesTree speciesTree) {
		_speciesTree = speciesTree;
	}

	public void addSyntenyListener(SyntenyListener listener) {
		_syntenyListeners.add(listener);		
	}
	
	public void removeSyntenyListener(SyntenyListener listener){
		if(_syntenyListeners.contains(listener))
			_syntenyListeners.remove(listener);
	}

	public Collection getSyntenyListener() {
		return  _syntenyListeners;
	}

	public void setSyntenyListener(Collection syntenyListener) {
		_syntenyListeners = syntenyListener;
	}
	
	public Genome getReferenceSpecies(){
		return _reference;
	}

	public void genomeAnalysed(String speciesName,int nbGenomes) {
		for(SyntenyListener sl : _syntenyListeners){
			sl.speciesBeingAnalysed(speciesName,nbGenomes);
		}
	}

	private void delete() {
		for(Comparison c : _comparisons){
			c.delete();
		}
		if(_reference!=null)
			_reference.getRefComparison().clear();
		_comparisons.clear();
		_mapGenomeComparison.clear();
	}
}
