/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 */

package org.synteny.factory;

import java.sql.Statement;
import java.util.Collections;
import java.util.Vector;

import org.synteny.Genome;
import org.synteny.GenomeImpl;
import org.synteny.Protein;
import org.synteny.ProteinImpl;
import org.synteny.SpeciesTree;
import org.synteny.ihm.preferences.Settings;
import org.synteny.search.ProteinInfoSearch;


public class DatabaseGenomeFactory implements GenomeFactory {

	SpeciesTree _sptree;
	ProteinInfoSearchFactory _infosFactory;
	
	public DatabaseGenomeFactory(){
		databaseManager = new DatabaseManagerImpl();
	}
	
	public Genome newGenomeInstance(SpeciesTree sptree) throws Exception {
			_sptree = sptree;
			_infosFactory = new DatabaseProteinInfoSearchFactory();
			Genome g = getGenome();
			return g;
	}
	
	private Genome getGenome() throws Exception{
		ProteinInfoSearch searcher = _infosFactory.createProteinSearch(); 
		Genome genome = new GenomeImpl(_sptree);
		try
		{
			Statement st;
			DatabaseQueryResult qr = new DatabaseQueryResult();
			String query = Settings.getInstance().getGenomeQuery().replace("%spabrv", _sptree.getSpAbbrv());
			qr = getDatabaseManager().sendQuery(query);
			
			for(Vector line : (Vector<Vector>)(qr.getDataVector()))
			{
					int pid = (Integer)line.elementAt(0);
					String geneName = (String)line.elementAt(1);
					boolean strand = line.elementAt(2).equals("+");
					int start = (Integer)line.elementAt(3);
					String chromosome = (String)line.elementAt(4);
					String function = (String)line.elementAt(5);
					
					Protein p = new ProteinImpl(pid);
					p.setName(geneName);
					p.setGenome(genome);
					p.setGenomeStartStop(start, -1);
					p.setStrand(strand);
					p.setChromosome(chromosome);
					p.setFunction(function);
					p.setProteinInfoSearch(searcher);
					genome.addProtein(p);
			}
			Collections.sort(genome.getProtein());
			for(Object o : genome.getProtein()){
				Protein p = (Protein)o;
				System.out.println(p.getGenomeStart()+" "+p.getChromosome());
			}
			return genome;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new Exception("Unable to read input stream");
		}
	}

	
	public ProteinInfoSearchFactory getProteinInfoSearchFactory() {
		return _infosFactory;
	}

	public void setProteinInfoSearchFactory(
			ProteinInfoSearchFactory proteinInfoSearchFactory) {
		_infosFactory = proteinInfoSearchFactory;
	}

	/**
	 * @uml.property  name="databaseManager"
	 * @uml.associationEnd  inverse="databaseGenomeFactory:org.synteny.factory.DatabaseManager"
	 */
	private DatabaseManager databaseManager;

	/**
	 * Getter of the property <tt>databaseManager</tt>
	 * @return  Returns the databaseManager.
	 * @uml.property  name="databaseManager"
	 */
	public DatabaseManager getDatabaseManager() {
		return databaseManager;
	}

	/**
	 * Setter of the property <tt>databaseManager</tt>
	 * @param databaseManager  The databaseManager to set.
	 * @uml.property  name="databaseManager"
	 */
	public void setDatabaseManager(DatabaseManager databaseManager) {
		this.databaseManager = databaseManager;
	}

}
