/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.factory;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.synteny.SpeciesTree;
import org.synteny.SpeciesTreeImpl;
import org.synteny.ihm.preferences.Settings;


public class DatabaseSpeciesTreeFactory implements SpeciesTreeFactory {

	public DatabaseSpeciesTreeFactory(){
		databaseManager = new DatabaseManagerImpl();
	}

	public SpeciesTree newSpeciesTreeInstance() {
		SpeciesTree st = fillSpeciesTree(); 
		return st;
	}

	private SpeciesTree fillSpeciesTree(){
		SpeciesTree st_root = new SpeciesTreeImpl("root");
		st_root.setRootSpeciesTree(st_root);
		try {
			for(Vector line : (Vector<Vector>)(getResponseLines().getDataVector())){
				String sp_abbrv = (String)(line.elementAt(0));
				List<String> taxo = new ArrayList<String>();
				taxo.add("root");
				for(String s : ((String)line.elementAt(1)).replace(" ","").split(";")){
					taxo.add(s);
				}
				String species_name = (String)line.elementAt(2);
				long nbProt = (Long)line.elementAt(3);
				st_root.addSpecies(taxo, sp_abbrv, species_name, (int)nbProt);
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return st_root;
	}

	public DatabaseQueryResult getResponseLines() throws Exception
	{
		return getDatabaseManager().sendQuery(Settings.getInstance().getSpeciesTreeQuery());
	}

	/**
	 * @uml.property  name="databaseManager"
	 * @uml.associationEnd  inverse="databaseSpeciesTreeFactory:org.synteny.factory.DatabaseManager"
	 */
	private DatabaseManager databaseManager;

	/**
	 * Getter of the property <tt>databaseManager</tt>
	 * @return  Returns the databaseManager.
	 * @uml.property  name="databaseManager"
	 */
	public DatabaseManager getDatabaseManager() {
		return databaseManager;
	}

	/**
	 * Setter of the property <tt>databaseManager</tt>
	 * @param databaseManager  The databaseManager to set.
	 * @uml.property  name="databaseManager"
	 */
	public void setDatabaseManager(DatabaseManager databaseManager) {
		this.databaseManager = databaseManager;
	}
}
