/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.factory;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.synteny.Comparison;
import org.synteny.ComparisonImpl;
import org.synteny.Genome;
import org.synteny.Ortholog;
import org.synteny.OrthologImpl;
import org.synteny.Paralog;
import org.synteny.ParalogImpl;
import org.synteny.Protein;
import org.synteny.ProteinImpl;
import org.synteny.SpeciesTree;

public class FileComparisonFactory implements ComparisonsFactory {
	
	File _compFile;
	SpeciesTree _speciesTree;
	Genome _refGenome = null;
	Collection<ComparisonFactoryListener> _listeners;
	ProteinInfoSearchFactory _infosFactory;
	
	public FileComparisonFactory(String file){
		_compFile = new File(file);
		_listeners = new ArrayList<ComparisonFactoryListener>();
		_infosFactory = new FileProteinInfosSearchFactory();
	}
	
	public List newComparisonsList(SpeciesTree st, Genome refGenome) {
		_speciesTree = st;
		// TODO Auto-generated method stub
		return getComparisons();
	}
	
	private List<Comparison> getComparisons(){
		Map<String,Comparison> mapAbbrvComp = new HashMap<String, Comparison>();
		List<Paralog> existingPara = new ArrayList<Paralog>();
		List<Comparison> existingComp = new ArrayList<Comparison>();
		Paralog previousParalog=null;
		try {
			BufferedReader bf = new BufferedReader(new InputStreamReader(new FileInputStream(_compFile)));
			String line;
			bf.readLine();bf.readLine();
			while((line=bf.readLine())!=null){
				String[] tab = line.split("\\t");
				int blocid = Integer.parseInt(tab[0]);
				String compSpeciesAbbrv = tab[2];
				String refSpeciesAbbrv = tab[1];
				int pidSp1 = Integer.parseInt(tab[3]);
				int pidSp2 = Integer.parseInt(tab[8]);
				String geneNameSp1 = tab[7];
				String geneNameSp2 = tab[12];
				boolean strandSp1 = (tab[4].equals("+"));
				boolean strandSp2 = (tab[9].equals("+"));
				int startSp1 = Integer.parseInt(tab[5]);
				int stopSp1  = Integer.parseInt(tab[6]);
				Genome compGenome = _speciesTree.getGenome(compSpeciesAbbrv);
				Comparison comparison;
				
				if(_refGenome==null){
					_refGenome = _speciesTree.getGenome(refSpeciesAbbrv);
				}
				
				if(mapAbbrvComp.containsKey(compSpeciesAbbrv)){
					comparison = mapAbbrvComp.get(compSpeciesAbbrv);
				}else{
					for(ComparisonFactoryListener cfl : _listeners)
						cfl.genomeAnalysed(compGenome.getSpeciesName(),107);
					comparison = new ComparisonImpl(compGenome,_refGenome);
					compGenome.setCompComparison(comparison);
					_refGenome.addRefComparison(comparison);
					existingComp.add(comparison);
					mapAbbrvComp.put(compSpeciesAbbrv,comparison);
				}
				//System.out.println(mapAbbrvComp.keySet().size()+"--"+existingPara.size());
				
				Protein p1 = _refGenome.getProtein(pidSp1);
				Protein p2 = compGenome.getProtein(pidSp2);

				if(p1==null){
					p1 = new ProteinImpl(pidSp1);
					p1.setStrand(strandSp1);
					p1.setGenome(_refGenome);
					p1.setName(geneNameSp1);
					p1.setGenomeStartStop(startSp1, stopSp1);
					_refGenome.addProtein(p1);
				}
			
				if(p2==null){
					p2 = new ProteinImpl(pidSp2);
					compGenome.addProtein(p2);
					p2.setStrand(strandSp2);
					p2.setGenome(compGenome);
					p2.setName(geneNameSp2);
				}
				
				Paralog paraCourant = new ParalogImpl(blocid);
				if(existingPara.contains(paraCourant)){
					paraCourant = existingPara.get(existingPara.indexOf(paraCourant));
				}else{
					existingPara.add(paraCourant);
					if(previousParalog!=null){
						comparison.addParalog(previousParalog);
					}
					previousParalog = paraCourant;
				}

				Ortholog orthoCourant = new OrthologImpl(blocid,paraCourant);
				orthoCourant.setCompProtein(p2);
				orthoCourant.setRefProtein(p1);
				p2.addOrtholog(orthoCourant);
				p1.addOrtholog(orthoCourant);
				paraCourant.addOrtholog(orthoCourant);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return existingComp;
	}

	public void addComparisonFactoryListener(
			ComparisonFactoryListener comparisonFactoryListener) {
		_listeners.add(comparisonFactoryListener);
	}

	public Collection getComparisonFactoryListener() {
		// TODO Auto-generated method stub
		return _listeners;
	}

	public void setComparisonFactoryListener(
			Collection comparisonFactoryListener) {
		_listeners = comparisonFactoryListener;
	}

	public ProteinInfoSearchFactory getProteinInfoSearchFactory() {
		// TODO Auto-generated method stub
		return _infosFactory;
	}

	public void setProteinInfoSearchFactory(
			ProteinInfoSearchFactory proteinInfoSearchFactory) {
		_infosFactory = proteinInfoSearchFactory;
	}
}
