/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.factory;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Collections;

import org.synteny.Genome;
import org.synteny.GenomeImpl;
import org.synteny.Protein;
import org.synteny.ProteinImpl;
import org.synteny.SpeciesTree;


public class FileGenomeFactory implements GenomeFactory {

	SpeciesTree _spTree;
	ProteinInfoSearchFactory _infosFactory;

	public Genome newGenomeInstance(SpeciesTree spTree) throws Exception  {
		// TODO Auto-generated method stub
		_spTree = spTree;
		return getGenome();
	}

	private Genome getGenome() throws Exception{
		String line;
		Genome genome = new GenomeImpl(_spTree);
		try
		{
			BufferedReader s = new BufferedReader(new InputStreamReader(new FileInputStream(_spTree.getSpAbbrv()+"_genome.txt")));
			if( s.readLine().contains("!synteview data!")){
				s.readLine();
			}
			line = s.readLine();
			while (line != null)
			{
				if(!line.substring(0,1).equals("!")){
					String[] split =line.split("\t");
					int pid = Integer.parseInt(split[0]);
					String geneName = split[1];
					boolean strand = split[2].equals("+");
					int start = Integer.parseInt(split[3]);
					String chromosome = split[4];
					Protein p = new ProteinImpl(pid);
					p.setName(geneName);
					p.setGenome(genome);
					p.setGenomeStartStop(start, -1);
					p.setStrand(strand);
					p.setChromosome(chromosome);
					genome.addProtein(p);
				}
				line = s.readLine();
			}
			s.close();
			Collections.sort(genome.getProtein());
			return genome;
		}
		catch(Exception e)
		{
			throw new Exception("Unable to read input stream");
		}


	}

	public ProteinInfoSearchFactory getProteinInfoSearchFactory() {
		return _infosFactory;
	}

	public void setProteinInfoSearchFactory(
			ProteinInfoSearchFactory proteinInfoSearchFactory) {
		_infosFactory = proteinInfoSearchFactory;
		
	}
}
