/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.factory;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import org.synteny.SpeciesTree;
import org.synteny.SpeciesTreeImpl;

public class FileSpeciesTreeFactory implements SpeciesTreeFactory {

	File _speciesFile;

	public FileSpeciesTreeFactory(String file){
		_speciesFile = new File(file);
	}

	public SpeciesTree newSpeciesTreeInstance() {
		return fillSpeciesTree();
	}

	private SpeciesTree fillSpeciesTree(){
		SpeciesTree st_root = new SpeciesTreeImpl("root");
		st_root.setRootSpeciesTree(st_root);
		try {
			FileInputStream fis = new FileInputStream(_speciesFile);
			BufferedReader dis = new BufferedReader(new InputStreamReader(fis));
			String line;
			dis.readLine();dis.readLine();
			while((line = dis.readLine())!=null){
				System.out.println(line);
				String[] tab = line.split("\\t");
				String sp_abbrv = tab[0];
				List<String> taxo = new ArrayList<String>();
				taxo.add("root");
				for(String s : tab[1].replace(" ","").split(";")){
					taxo.add(s);
				}
				String species_name = tab[2];
				int nbProt = Integer.parseInt(tab[3]);
				st_root.addSpecies(taxo, sp_abbrv, species_name, nbProt);
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return st_root;
	}
}
