/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.factory;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.synteny.Comparison;
import org.synteny.ComparisonImpl;
import org.synteny.Genome;
import org.synteny.Ortholog;
import org.synteny.OrthologImpl;
import org.synteny.Paralog;
import org.synteny.ParalogImpl;
import org.synteny.Protein;
import org.synteny.ProteinImpl;
import org.synteny.SpeciesTree;
import org.synteny.ihm.preferences.Settings;
import org.synteny.search.ProteinInfoSearch;


public class HttpComparisonsFactory implements ComparisonsFactory {

	URL _blocsUrl;
	HttpURLConnection _server;
	SpeciesTree _speciesTree;
	Genome _refGenome;
	Collection<ComparisonFactoryListener> _listeners;
	ProteinInfoSearchFactory _infosFactory;

	public HttpComparisonsFactory(){
		_listeners = new ArrayList<ComparisonFactoryListener>();
		_infosFactory = new HttpProteinInfoSearchFactory();
	}

	public List newComparisonsList(SpeciesTree sp, Genome refGenome,Collection<Genome> genomesComp) {
		_speciesTree = sp;
		try {
			_blocsUrl = new URL(Settings.getInstance().getSyntenyURL().replace("%species", refGenome.getSpeciesAbbrv()).replace("%comp", phpSerialize(genomesComp)));
			System.out.println(_blocsUrl);
			_refGenome = _speciesTree.getGenome(refGenome.getSpeciesAbbrv());
			connect();
		} catch (Exception e) {
			e.printStackTrace();
		}
		List<Comparison> lc = getComparisons(); 
		disconnect();
		return lc;
	}

	private List<Comparison> getComparisons(){
		List<Comparison> existingComp = new ArrayList<Comparison>();
		Paralog previousParalog=null;
		Comparison previousComparison = null;
		ProteinInfoSearch searcher = _infosFactory.createProteinSearch();
		int currentBlocId = -1;
		Paralog paraCourant = null;
		Comparison comparison = null;
		Genome compGenome = null;
		Genome previousCompGenome = null;
		
		try {
			for(String line : getResponseLines()){
				String[] tab = line.split("\\t");
				int blocid = Integer.parseInt(tab[0]);
				String compSpeciesAbbrv = tab[2];
				int pidSp1 = Integer.parseInt(tab[3]);
				int pidSp2 = Integer.parseInt(tab[8]);
				String geneNameSp1 = tab[7];
				String geneNameSp2 = tab[12];
				boolean strandSp1 = (tab[4].equals("+"));
				boolean strandSp2 = (tab[9].equals("+"));
				int startSp1 = Integer.parseInt(tab[5]);
				int stopSp1  = Integer.parseInt(tab[6]);

				if(comparison==null || !(previousCompGenome.getSpeciesAbbrv().equals(compSpeciesAbbrv))){
					compGenome = _speciesTree.getGenome(compSpeciesAbbrv);
					previousCompGenome = compGenome;
					for(ComparisonFactoryListener cfl : _listeners){
						cfl.genomeAnalysed(compGenome.getSpeciesName(),108);
					}
					comparison = new ComparisonImpl(compGenome,_refGenome);
					compGenome.setCompComparison(comparison);
					_refGenome.addRefComparison(comparison);
					existingComp.add(comparison);
				}

				Protein p1 = _refGenome.getProtein(pidSp1);
				Protein p2 = compGenome.getProtein(pidSp2);

				if(p1==null){
					p1 = new ProteinImpl(pidSp1);
					p1.setStrand(strandSp1);
					p1.setGenome(_refGenome);
					p1.setName(geneNameSp1);
					p1.setGenomeStartStop(startSp1, stopSp1);
					p1.setProteinInfoSearch(searcher);
					_refGenome.addProtein(p1);
				}
				if(p2==null){
					p2 = new ProteinImpl(pidSp2);
					p2.setStrand(strandSp2);
					p2.setGenome(compGenome);
					p2.setName(geneNameSp2);
					p2.setProteinInfoSearch(searcher);
					compGenome.addProtein(p2);
				}
				p1.addMatchInComparison(compGenome);
				if(paraCourant==null || blocid != currentBlocId){
					paraCourant = new ParalogImpl(blocid);
					currentBlocId = blocid;
					if(previousParalog!=null){
						//previousParalog
						previousComparison.addParalog(previousParalog);
					}
					previousParalog = paraCourant;
					previousComparison = comparison;
				}
				Ortholog orthoCourant = new OrthologImpl(blocid,paraCourant);
				orthoCourant.setCompProtein(p2);
				orthoCourant.setRefProtein(p1);
				p2.addOrtholog(orthoCourant);
				p1.addOrtholog(orthoCourant);
				paraCourant.addOrtholog(orthoCourant);	
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return existingComp;
	}

	/**
	 * @param method: String object for client method (POST, GET,...)
	 */
	private void connect() throws Exception
	{
		System.out.println("Récupération des données de BLOCS");
		try
		{
			_server = (HttpURLConnection)_blocsUrl.openConnection();
			_server.setDoInput(true);
			_server.setDoOutput(true);
			_server.setRequestMethod("GET");
			_server.setRequestProperty("Content-type",
			"application/x-www-form-urlencoded");
			_server.connect();
		}
		catch (Exception e)
		{
			throw new Exception("Connection failed");
		}
	}

	public void disconnect()
	{
		_server.disconnect();
	}

	public Collection<String> getResponseLines() throws Exception
	{
		Collection<String> result = new ArrayList<String>();
		String line;
		for(Object o : getComparisonFactoryListener()){
			ComparisonFactoryListener cfl = (ComparisonFactoryListener)o;
			cfl.genomeAnalysed("Retrieving synteny data",108);
		}

		try
		{
			BufferedReader s = new BufferedReader(
					new InputStreamReader(
							_server.getInputStream()));
			if( s.readLine().contains("!synteview data!")){
				s.readLine();
			}
			line = s.readLine();
			while (line != null)
			{
				if(!line.substring(0,1).equals("!")){
					result.add(line);
				}
				line = s.readLine();
			}
			s.close();
		}
		catch(Exception e)
		{
			throw new Exception("Unable to read input stream");
		}
		return result;
	}

	public Collection getComparisonFactoryListener() {
		return _listeners;
	}

	public void setComparisonFactoryListener(
			Collection comparisonFactoryListener) {
		_listeners = comparisonFactoryListener;
	}

	public void addComparisonFactoryListener(
			ComparisonFactoryListener comparisonFactoryListener) {
		_listeners.add(comparisonFactoryListener);

	}

	public ProteinInfoSearchFactory getProteinInfoSearchFactory() {
		return _infosFactory;
	}

	public void setProteinInfoSearchFactory(
			ProteinInfoSearchFactory proteinInfoSearchFactory) {
		_infosFactory = proteinInfoSearchFactory;
		
	}
	
	public String phpSerialize(Collection<Genome> collGenomes){
		String tab = "";
		Iterator<Genome> it = collGenomes.iterator();
		while(it.hasNext()){
			tab+=it.next().getSpeciesAbbrv();
			if(it.hasNext())
				tab+="_";
		}
		return tab;
	}
}
