/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.factory;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Collections;

import org.synteny.Genome;
import org.synteny.GenomeImpl;
import org.synteny.Protein;
import org.synteny.ProteinImpl;
import org.synteny.SpeciesTree;
import org.synteny.ihm.preferences.Settings;
import org.synteny.search.ProteinInfoSearch;


public class HttpGenomeFactory implements GenomeFactory {

	URL _genomeURL;
	HttpURLConnection _server;
	SpeciesTree _sptree;
	ProteinInfoSearchFactory _infosFactory;
	
	public HttpGenomeFactory(){
	}
	
	public Genome newGenomeInstance(SpeciesTree sptree) throws Exception {
			_sptree = sptree;
			_genomeURL = new URL(Settings.getInstance().getGenomeURL().replace("%species", _sptree.getSpAbbrv()));
			_infosFactory = new HttpProteinInfoSearchFactory();
			connect();
			Genome g = getGenome();
			disconnect();
			return g;
	}
	
	private Genome getGenome() throws Exception{
		String line;
		ProteinInfoSearch searcher = _infosFactory.createProteinSearch(); 
		Genome genome = new GenomeImpl(_sptree);
		try
		{
			BufferedReader s = new BufferedReader(
					new InputStreamReader(
							_server.getInputStream()));
			if( s.readLine().contains("!synteview data!")){
				s.readLine();
			}
			line = s.readLine();
			while (line != null)
			{
				if(!line.substring(0,1).equals("!")){
					String[] split =line.split("\t");
					int pid = Integer.parseInt(split[0]);
					String geneName = split[1];
					boolean strand = split[2].equals("+");
					int start = Integer.parseInt(split[3]);
					String chromosome = split[4];
					String function = split[5];
					
					Protein p = new ProteinImpl(pid);
					p.setName(geneName);
					p.setGenome(genome);
					p.setGenomeStartStop(start, -1);
					p.setStrand(strand);
					p.setChromosome(chromosome);
					p.setFunction(function);
					p.setProteinInfoSearch(searcher);
					genome.addProtein(p);
				}
				line = s.readLine();
			}
			s.close();
			Collections.sort(genome.getProtein());
			for(Object o : genome.getProtein()){
				Protein p = (Protein)o;
				System.out.println(p.getGenomeStart()+" "+p.getChromosome());
			}
			return genome;
		}
		catch(Exception e)
		{
			throw new Exception("Unable to read input stream");
		}
	}

	
	private void connect() throws Exception
	{
		System.out.println("Récupération des données du Genome "+_sptree.getSpAbbrv());
		try
		{
			_server = (HttpURLConnection) _genomeURL.openConnection();
			_server.setDoInput(true);
			_server.setDoOutput(true);
			_server.setRequestMethod("GET");
			_server.setRequestProperty("Content-type",
			"application/x-www-form-urlencoded");
			_server.connect();
		}
		catch (Exception e)
		{
			throw new Exception("Connection failed");
		}
	}

	public void disconnect()
	{
		_server.disconnect();
	}

	public ProteinInfoSearchFactory getProteinInfoSearchFactory() {
		return _infosFactory;
	}

	public void setProteinInfoSearchFactory(
			ProteinInfoSearchFactory proteinInfoSearchFactory) {
		_infosFactory = proteinInfoSearchFactory;
		
	}
}
