/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.factory;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.synteny.SpeciesTree;
import org.synteny.SpeciesTreeImpl;
import org.synteny.ihm.preferences.Settings;


public class HttpSpeciesTreeFactory implements SpeciesTreeFactory {

	URL _treeURL;
	HttpURLConnection _server;

	public HttpSpeciesTreeFactory(){
		try {
			_treeURL = new URL(Settings.getInstance().getSpeciesTreeURL());
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public SpeciesTree newSpeciesTreeInstance() {
		try {
			connect();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		SpeciesTree st = fillSpeciesTree(); 
		disconnect();
		return st;
	}

	private SpeciesTree fillSpeciesTree(){
		SpeciesTree st_root = new SpeciesTreeImpl("root");
		st_root.setRootSpeciesTree(st_root);
		
		try {
			for(String line : getResponseLines()){
				String[] tab = line.split("\\t");
				String sp_abbrv = tab[0];
				List<String> taxo = new ArrayList<String>();
				taxo.add("root");
				for(String s : tab[1].replace(" ","").split(";")){
					taxo.add(s);
				}
				String species_name = tab[2];
				int nbProt = Integer.parseInt(tab[3]);
				st_root.addSpecies(taxo, sp_abbrv, species_name, nbProt);
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return st_root;
	}
	
	/**
	 * @param method: String object for client method (POST, GET,...)
	 */
	private void connect() throws Exception
	{
		System.out.println("Récupération des données d'espèces");
		try
		{
			_server = (HttpURLConnection)_treeURL.openConnection();
			_server.setDoInput(true);
			_server.setDoOutput(true);
			_server.setRequestMethod("GET");
			_server.setRequestProperty("Content-type",
			"application/x-www-form-urlencoded");
			_server.connect();
		}
		catch (Exception e)
		{
			throw new Exception("Connection failed");
		}
	}

	public void disconnect()
	{
		_server.disconnect();
	}

	public Collection<String> getResponseLines() throws Exception
	{
		Collection<String> result = new ArrayList<String>();
		String line;

	
		try
		{
			BufferedReader s = new BufferedReader(
					new InputStreamReader(
							_server.getInputStream()));
			if( s.readLine().contains("!synteview data!")){
				s.readLine();
			}
			line = s.readLine();
			while (line != null)
			{
				if(!line.substring(0,1).equals("!")){
					result.add(line);
				}
				line = s.readLine();
			}
			s.close();
		}
		catch(Exception e)
		{
			throw new Exception("Unable to read input stream");
		}
		return result;
	}
	
	public static void main(String[] args) {
		SpeciesTreeFactory stf = new HttpSpeciesTreeFactory();
		SpeciesTree st = stf.newSpeciesTreeInstance();
	}
}
