/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.factory;

import org.synteny.Synteny;

public interface SyntenyFactory {


	/**
	 */
	public abstract Synteny newSyntenyInstance();

	/**
	 * @return      Returns the speciesTreeFactory.
	 * @uml.property   name="speciesTreeFactory"
	 * @uml.associationEnd   inverse="syntenyFactory:org.synteny.factory.SpeciesTreeFactory"
	 * @uml.association   name="hasSPTreeFactory"
	 */
	public SpeciesTreeFactory getSpeciesTreeFactory();

	/**
	 * Setter of the property <tt>speciesTreeFactory</tt>
	 * @param speciesTreeFactory  The speciesTreeFactory to set.
	 * @uml.property  name="speciesTreeFactory"
	 */
	public void setSpeciesTreeFactory(SpeciesTreeFactory speciesTreeFactory);

	/**
	 * @return      Returns the comparisonsFactory.
	 * @uml.property   name="comparisonsFactory"
	 * @uml.associationEnd   inverse="syntenyFactory:org.synteny.factory.ComparisonsFactory"
	 * @uml.association   name="hasCompFactory"
	 */
	public ComparisonsFactory getComparisonsFactory();

	/**
	 * Setter of the property <tt>comparisonsFactory</tt>
	 * @param comparisonsFactory  The comparisonsFactory to set.
	 * @uml.property  name="comparisonsFactory"
	 */
	public void setComparisonsFactory(ComparisonsFactory comparisonsFactory);

	/**
	 * @return      Returns the genomeFactory.
	 * @uml.property   name="genomeFactory"
	 * @uml.associationEnd   inverse="syntenyFactory:org.synteny.factory.GenomeFactory"
	 * @uml.association   name="hasGenomeFactory"
	 */
	public GenomeFactory getGenomeFactory();

	/**
	 * Setter of the property <tt>genomeFactory</tt>
	 * @param genomeFactory  The genomeFactory to set.
	 * @uml.property  name="genomeFactory"
	 */
	public void setGenomeFactory(GenomeFactory genomeFactory);

	/**
	 * @return      Returns the proteinSearchFactory.
	 * @uml.property   name="proteinSearchFactory"
	 * @uml.associationEnd   inverse="syntenyFactory:org.synteny.factory.ProteinInfoSearchFactory"
	 */
	public ProteinInfoSearchFactory getProteinSearchFactory();

	/**
	 * Setter of the property <tt>proteinSearchFactory</tt>
	 * @param proteinSearchFactory  The proteinSearchFactory to set.
	 * @uml.property  name="proteinSearchFactory"
	 */
	public void setProteinSearchFactory(
			ProteinInfoSearchFactory proteinSearchFactory);


}
