/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class BlocSizeAdjustmentWidget extends JPanel implements ChangeListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = -1195489622986221282L;

	JSlider _blocWidth;
	JSlider _blocHeight;
	JButton _searchButton;
	JLabel _widthLabel;
	JLabel _heightLabel;
	
	public BlocSizeAdjustmentWidget(){
		super();
		setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
		
		add(getWidthLabel());
		add(getBlocWidthSlider());
		
		add(getHeightLabel());
		add(getBLocHeightSlider());
		add(getSearchButton()); 
	}
	
	public int getBlocWith(){
		return getBlocWidthSlider().getValue();
	}
	
	public int getBlocHeight(){
		return getBLocHeightSlider().getValue();
	}
	
	public JSlider getBlocWidthSlider(){
		if(_blocWidth == null){
			_blocWidth = new JSlider(2,30,2);
			_blocWidth.setOrientation(JSlider.HORIZONTAL);
			_blocWidth.setName("width");
			_blocWidth.addChangeListener(this);
		}
		return _blocWidth;
	}
	
	public JSlider getBLocHeightSlider(){
		if(_blocHeight == null){
			_blocHeight = new JSlider(1,30,1);
			_blocHeight.setOrientation(JSlider.HORIZONTAL);
			_blocHeight.setName("height");
			_blocHeight.addChangeListener(this);
		}
		return _blocHeight;
	}
	
	public JButton getSearchButton(){
		if(_searchButton == null){
			_searchButton = new JButton("searchBloc");
			_searchButton.setAlignmentX(JButton.CENTER_ALIGNMENT);
		}
		return _searchButton;
	}

	public void stateChanged(ChangeEvent arg0) {
		JSlider js = ((JSlider)(arg0.getSource()));
		if(js.getName().equals("width")){
			getWidthLabel().setText("Gene Nb ("+js.getValue()+")");
		}else{
			getHeightLabel().setText("Species Nb ("+js.getValue()+")");
		}
	}
	
	public JLabel getWidthLabel(){
		if(_widthLabel==null){
			_widthLabel = new JLabel("Gene Nb (2)");
		}
		return _widthLabel;
	}
	
	public JLabel getHeightLabel(){
		if(_heightLabel == null){
			_heightLabel = new JLabel("Species Nb (1)");
		}
		return _heightLabel;
	}
	
	public void addActionListener(ActionListener al){
		getSearchButton().addActionListener(al);
	}
	
	public void setNbGenomes(int nbGenomes){
		getBLocHeightSlider().setMaximum(nbGenomes);
	}
}
