/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JPanel;

import org.synteny.Bloc;
import org.synteny.Comparison;
import org.synteny.ihm.layout.ComparisonLayout;


public class ComparisonPanel extends JPanel{

	public static int COMPARISON_HEIGHT = 30;
	public static int COMPARISON_TOP_MARGIN = 2;
	public static int COMPARISON_BOTTOM_MARGIN = 2;
	/**
	 * 
	 */
	private static final long serialVersionUID = 4991107854215056246L;
	/**
	 * @uml.property  name="comparison"
	 * @uml.associationEnd  inverse="comparisonPanel:org.synteny.Comparison"
	 * @uml.association  name="hasComparison"
	 */
	private Comparison comparison;
	
	/**
	 * @uml.property   name="blocPanel"
	 * @uml.associationEnd   multiplicity="(0 -1)" inverse="comparisonPanel:org.synteny.ihm.SimpleBlocPanel"
	 * @uml.association   name="hasBlocPanels"
	 */
	private Collection blocPanel;


	public ComparisonPanel(Comparison c){
		comparison = c;
		blocPanel = new ArrayList<SimpleBlocPanel>();
		setLayout(new ComparisonLayout());
		for(Object o : c.getBloc()){
			Bloc b = (Bloc) o;
			SimpleBlocPanel bp = new SimpleBlocPanel(b);
			blocPanel.add(bp);
			bp.setComparisonPanel(this);
			add(bp);
		}
		setBackground(Color.white);
		setOpaque(true);
		//setPreferredSize(new Dimension(getComparison().getRefGenome().getSize()*(ProteinPanel.PROTEIN_WIDTH+ProteinPanel.PROTEIN_MARGIN),));
	}
	
	/**
	 * Getter of the property <tt>comparison</tt>
	 * @return  Returns the comparison.
	 * @uml.property  name="comparison"
	 */
	public Comparison getComparison() {
		return comparison;
	}

	/**
	 * Setter of the property <tt>comparison</tt>
	 * @param comparison  The comparison to set.
	 * @uml.property  name="comparison"
	 */
	public void setComparison(Comparison comparison) {
		this.comparison = comparison;
	}

	/**
	 * @uml.property  name="syntenyPanel"
	 * @uml.associationEnd  inverse="comparisonPanel:org.synteny.ihm.SyntenyPanel"
	 * @uml.association  name="hasComparisonPanel"
	 */
	private SyntenyPanel syntenyPanel;

	/**
	 * Getter of the property <tt>syntenyPanel</tt>
	 * @return  Returns the syntenyPanel.
	 * @uml.property  name="syntenyPanel"
	 */
	public SyntenyPanel getSyntenyPanel() {
		return syntenyPanel;
	}

	/**
	 * Setter of the property <tt>syntenyPanel</tt>
	 * @param syntenyPanel  The syntenyPanel to set.
	 * @uml.property  name="syntenyPanel"
	 */
	public void setSyntenyPanel(SyntenyPanel syntenyPanel) {
		this.syntenyPanel = syntenyPanel;
		for(Object o: getBlocPanel()){
			SimpleBlocPanel sb = (SimpleBlocPanel)o;
			for(Object o2: sb.getParalogPanel().getOrthologPanel()){
				OrthologPanel or = (OrthologPanel)o2;
				or.addSyntenyListener(syntenyPanel);
			}
		}
	}

	/**
	 * Getter of the property <tt>blocPanel</tt>
	 * @return  Returns the blocPanel.
	 * @uml.property  name="blocPanel"
	 */
	public Collection getBlocPanel() {
		return blocPanel;
	}

	/**
	 * Setter of the property <tt>blocPanel</tt>
	 * @param blocPanel  The blocPanel to set.
	 * @uml.property  name="blocPanel"
	 */
	public void setBlocPanel(Collection blocPanel) {
		this.blocPanel = blocPanel;
	}
	
	public void paint(Graphics g){
		super.paint(g);
		g.drawLine(0, 0, getWidth(), 0);
		g.drawLine(0, getHeight()-1, getWidth(), getHeight()-1);

	}

}
