/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JToolBar;

import org.synteny.Paralog;
import org.synteny.factory.IconFactory;
import org.synteny.factory.ParalogDialogMouseManager;


public class ComplexBlocDialog extends JDialog {

	/**
	 * 
	 */
	private static final long serialVersionUID = -728104589527675438L;
	/**
	 * @uml.property  name="paralogPanel"
	 * @uml.associationEnd  multiplicity="(0 -1)" inverse="complexBlocDialog:org.synteny.ihm.ParalogPanel"
	 */
	private Collection paralogPanel;

	
	public ComplexBlocDialog(SimpleBlocPanel b){
		super();
		blocPanel = b;
		setModal(true);
		setUndecorated(true);
		setLocationRelativeTo(b);
		initialize();
		pack();
		setResizable(false);
	}
	
	private void initialize(){
		setLayout(new BorderLayout());
		paralogPanel = new ArrayList<ParalogPanel>();
		JPanel jpPara = new JPanel();
		jpPara.setLayout(new GridLayout(blocPanel.getBloc().getNbParalogs()+1,1));
		for(Object o : blocPanel.getBloc().getParalog()){
			Paralog p = (Paralog)o;
			ParalogPanel pp = new ParalogPanel(getBlocPanel(),p);
			paralogPanel.add(pp);
			jpPara.add(pp);
		}
		
		JToolBar jtb = new JToolBar();
		jtb.setOrientation(JToolBar.HORIZONTAL);
		jtb.setLayout(new FlowLayout(FlowLayout.RIGHT));
		jtb.setMargin(new Insets(0,0,0,0));
		jtb.setFloatable(false);
		JButton jb = new JButton();
		jb.setIcon(IconFactory.getInstance().getCloseIcon());
		jb.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent e) {
				dispose();
			}

		});
		jtb.add(jb);
		jb.setPreferredSize(new Dimension(20,20));
		ParalogDialogMouseManager mm = new ParalogDialogMouseManager(this);
		jtb.addMouseMotionListener(mm);
		jtb.addMouseListener(mm);
		add(jtb,BorderLayout.NORTH);
		add(jpPara,BorderLayout.CENTER);
	}

	/**
	 * @uml.property   name="blocPanel"
	 * @uml.associationEnd   inverse="paralogDialog:org.synteny.ihm.SimpleBlocPanel"
	 * @uml.association   name="hasParalogDialog"
	 */
	private SimpleBlocPanel blocPanel;

	/**
	 * Getter of the property <tt>blocPanel</tt>
	 * @return  Returns the blocPanel.
	 * @uml.property  name="blocPanel"
	 */
	public SimpleBlocPanel getBlocPanel() {
		return blocPanel;
	}

	/**
	 * Setter of the property <tt>blocPanel</tt>
	 * @param blocPanel  The blocPanel to set.
	 * @uml.property  name="blocPanel"
	 */
	public void setBlocPanel(SimpleBlocPanel blocPanel) {
		this.blocPanel = blocPanel;
	}

	/**
	 * Getter of the property <tt>paralogPanel</tt>
	 * @return  Returns the paralogPanel.
	 * @uml.property  name="paralogPanel"
	 */
	public Collection getParalogPanel() {
		return paralogPanel;
	}

	/**
	 * Setter of the property <tt>paralogPanel</tt>
	 * @param paralogPanel  The paralogPanel to set.
	 * @uml.property  name="paralogPanel"
	 */
	public void setParalogPanel(Collection paralogPanel) {
		this.paralogPanel = paralogPanel;
	}

}
