/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JPanel;

import org.synteny.Genome;
import org.synteny.Protein;


public class ConservationRatePanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2811272013793208352L;

	/**
	 * @uml.property  name="syntenyPanel"
	 * @uml.associationEnd  inverse="conservationRatePanel:org.synteny.ihm.SyntenyPanel"
	 */
	private SyntenyPanel syntenyPanel;

	private Dimension _dimension;

	private Map<Integer,Integer> _rates;
	private List<Integer> _positions;
	private Color[] _histoColors;
	private HashMap<Color, HashMap<Color, GradientPaint>> _gradients;
	
	
	public ConservationRatePanel(SyntenyPanel sp){
		super();
		initialize(sp);
	}

	private void initialize(SyntenyPanel sp){
		syntenyPanel = sp;
		_rates = new HashMap<Integer,Integer>();
		_positions = new ArrayList<Integer>();
		_dimension = new Dimension();
		_histoColors = new Color[100];
		for(int i=0;i<100;i++){
			_histoColors[i] = new Color((int)(255-(2.55*i)),(int)(255-(2.55*i)),255);
		}
		_gradients = new HashMap<Color, HashMap<Color,GradientPaint>>();
		setBackground(Color.white);
		computeRates();
	}
	
	public void computeRates(){
		_rates.clear();
		_positions.clear();
		int i=0;
		Genome refGenome = getSyntenyPanel().getSynteny().getReferenceSpecies();
		if(refGenome != null){
			int width = getSyntenyPanel().getSynteny().getReferenceSpecies().getSize()*(ProteinPanel.PROTEIN_WIDTH+ProteinPanel.PROTEIN_MARGIN);
			int height = 50;
			setPreferredSize(new Dimension(width,height));
			for(Object o : refGenome.getProtein()){
				i++;
				Protein p = (Protein)o;
				/* nombre de comparaisons du ième gène */
				int maxHauteur = getSyntenyPanel().getVisibleComparedGenomes().size();
				int hauteur;
				if(maxHauteur==0){
					hauteur = 0;
				}else{
					hauteur = (p.matchInOtherComparisonNumber(getSyntenyPanel().getVisibleComparedGenomes())*(height-1))/maxHauteur+1;
				}
				int position = i*(ProteinPanel.PROTEIN_WIDTH+ProteinPanel.PROTEIN_MARGIN)+ProteinPanel.PROTEIN_MARGIN*2;
				_rates.put(position,hauteur);
			}
			_positions.addAll(_rates.keySet());
			Collections.sort(_positions);
		}
		showPanel();
	}

	/**
	 * Getter of the property <tt>syntenyPanel</tt>
	 * @return  Returns the syntenyPanel.
	 * @uml.property  name="syntenyPanel"
	 */
	public SyntenyPanel getSyntenyPanel() {
		return syntenyPanel;
	}

	/**
	 * Setter of the property <tt>syntenyPanel</tt>
	 * @param syntenyPanel  The syntenyPanel to set.
	 * @uml.property  name="syntenyPanel"
	 */
	public void setSyntenyPanel(SyntenyPanel syntenyPanel) {
		this.syntenyPanel = syntenyPanel;
	}

	public void hidePanel(){
		_dimension.setSize(_dimension.getWidth(), 0);
		setPreferredSize(_dimension);	
		updateUI();
	}

	public void showPanel(){
		int width = 0;
		if(getSyntenyPanel().getSynteny().getReferenceSpecies()!=null){
			width = getSyntenyPanel().getSynteny().getReferenceSpecies().getSize()*(ProteinPanel.PROTEIN_WIDTH+ProteinPanel.PROTEIN_MARGIN);
		}
		_dimension.setSize(width,50);
		setPreferredSize(_dimension);
		updateUI();
	}

	public void paint(Graphics g){
		super.paint(g);
		int prevHigh = 0;
		for(int i=0;i<_positions.size();i++){
			Integer position = _positions.get(i);
			int hauteur = _rates.get(position);
			g.setColor(_histoColors[hauteur]);
			
			if(!_histoColors[prevHigh].equals(_histoColors[hauteur])){
				((Graphics2D)g).setPaint(getGradientPaint(_histoColors[prevHigh],_histoColors[hauteur],position,hauteur));
			}
			g.fillRect(position,getHeight()-hauteur,(ProteinPanel.PROTEIN_WIDTH+ProteinPanel.PROTEIN_MARGIN)/2, hauteur);
			g.setColor(_histoColors[hauteur]);
			g.fillRect(position+(ProteinPanel.PROTEIN_WIDTH+ProteinPanel.PROTEIN_MARGIN)/2, getHeight()-hauteur, (ProteinPanel.PROTEIN_WIDTH+ProteinPanel.PROTEIN_MARGIN)/2+1, hauteur);
			
			g.setColor(Color.black);
			g.drawLine(position, getHeight()-prevHigh, position, getHeight()-hauteur);
			g.drawLine(position, getHeight()-hauteur, position+(ProteinPanel.PROTEIN_WIDTH+ProteinPanel.PROTEIN_MARGIN), getHeight()-hauteur);
			prevHigh = hauteur;
		}
		
		g.setColor(Color.black);
		g.drawLine(20, 1, getWidth(), 1);
		g.drawString("100%", 0, 10);
		g.drawLine(30, getHeight()-1, getWidth(), getHeight()-1);
		g.drawLine(30, getHeight()/2, getWidth(), getHeight()/2);
		g.drawString("50%", 0, getHeight()/2+10);
	}
	
	
	private Paint getGradientPaint(Color c1, Color c2,int position, int hauteur){
		if(!_gradients.containsKey(c1)){
			_gradients.put(c1, new HashMap<Color, GradientPaint>());
		}
		if(!_gradients.get(c1).containsKey(c1)){
			_gradients.get(c1).put(c2, new GradientPaint(position, getHeight()-hauteur, c1,position+(ProteinPanel.PROTEIN_WIDTH+ProteinPanel.PROTEIN_MARGIN)/2, getHeight()-hauteur,c2)); 
		}
		
		return _gradients.get(c1).get(c2);
	}
}
