/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JLabel;
import org.synteny.Genome;


public class GenomeLabel extends JLabel implements MouseListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = -9122882758982369147L;
	
	/**
	 * @uml.property  name="genome"
	 * @uml.associationEnd  inverse="genomeLabel:org.synteny.Genome"
	 * @uml.association  name="hasGenome"
	 */
	private Genome genome;
	private Collection<GenomeLabelListener> _listeners; 

	public GenomeLabel(Genome g){
		super(g.getSpeciesAbbrv());
		_listeners  = new ArrayList<GenomeLabelListener>();
		setToolTipText(g.getSpeciesName());
		genome = g;
		setPreferredSize(new Dimension(50,ProteinPanel.PROTEIN_HEIGHT+SimpleBlocPanel.BLOC_MARGIN*2+ComparisonPanel.COMPARISON_BOTTOM_MARGIN+ComparisonPanel.COMPARISON_TOP_MARGIN));
		addMouseListener(this);
		setOpaque(true);
		setBackground(Color.white);
	}
	
	/**
	 * Getter of the property <tt>genome</tt>
	 * @return  Returns the genome.
	 * @uml.property  name="genome"
	 */
	public Genome getGenome() {
		return genome;
	}

	/**
	 * Setter of the property <tt>genome</tt>
	 * @param genome  The genome to set.
	 * @uml.property  name="genome"
	 */
	public void setGenome(Genome genome) {
		this.genome = genome;
	}

	public void mouseClicked(MouseEvent e) {
		if(e.getClickCount()==2){
		System.out.println("Genome "+genome.getSpeciesName()+" ("+genome.getSpeciesAbbrv()+")"+" Clicked");
		for(GenomeLabelListener gll : _listeners){
			gll.genomeClicked(genome);
		}
		}
		if(e.getButton() == MouseEvent.BUTTON2 ){
			
		}
	}

	public void mouseEntered(MouseEvent e) {
		setBackground(Color.cyan);
		repaint();
	}

	public void mouseExited(MouseEvent e) {
		setBackground(Color.white);
		repaint();
	}

	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}
	
	public void addGenomeLabelListener(GenomeLabelListener gll){
		_listeners.add(gll);
	}
}
