/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.Color;
import java.awt.GridLayout;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.JLabel;
import javax.swing.JPanel;

import org.synteny.Genome;


public class GenomesPanel extends JPanel implements GenomeLabelListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = -5271201829835523516L;
	/**
	 * @uml.property  name="syntenyPanel"
	 * @uml.associationEnd  inverse="genomesPanel:org.synteny.ihm.SyntenyPanel"
	 * @uml.association  name="hasGenomePanel"
	 */
	private SyntenyPanel syntenyPanel;
	private List<GenomeLabel> labelList;
	/** 
	 * @uml.property name="genomeLabel"
	 * @uml.associationEnd multiplicity="(0 -1)" inverse="genomesPanel:org.synteny.ihm.GenomeLabel"
	 * @uml.association name="hasGenomesLabel"
	 */
	private Collection genomeLabel;


	public GenomesPanel(SyntenyPanel syntenyp, List<ComparisonPanel> comparisons){
		super();
		syntenyPanel = syntenyp;
		labelList = new ArrayList<GenomeLabel>();
		//setLayout(new GridLayout(comparisons.size()+2,1));
		setLayout(new GridLayout(100+2,1));
		setBackground(Color.WHITE);

		for(ComparisonPanel c : comparisons){
			GenomeLabel jl = new GenomeLabel(c.getComparison().getCompGenome());
			jl.addGenomeLabelListener(this);
			add(jl);
			labelList.add(jl);
		}
		JLabel jl = new JLabel(" ");
		add(jl);
	}


	/**
	 * Getter of the property <tt>syntenyPanel</tt>
	 * @return  Returns the syntenyPanel.
	 * @uml.property  name="syntenyPanel"
	 */
	public SyntenyPanel getSyntenyPanel() {
		return syntenyPanel;
	}

	/**
	 * Setter of the property <tt>syntenyPanel</tt>
	 * @param syntenyPanel  The syntenyPanel to set.
	 * @uml.property  name="syntenyPanel"
	 */
	public void setSyntenyPanel(SyntenyPanel syntenyPanel) {
		this.syntenyPanel = syntenyPanel;
	}

	/** 
	 * Getter of the property <tt>genomeLabel</tt>
	 * @return  Returns the genomeLabel.
	 * @uml.property  name="genomeLabel"
	 */
	public Collection getGenomeLabel() {
		return genomeLabel;
	}

	/** 
	 * Setter of the property <tt>genomeLabel</tt>
	 * @param genomeLabel  The genomeLabel to set.
	 * @uml.property  name="genomeLabel"
	 */
	public void setGenomeLabel(Collection genomeLabel) {
		this.genomeLabel = genomeLabel;
	}

	public void updateGenomes(Collection<Genome> genomes){
		removeAll();
		for(GenomeLabel jl : labelList){
			if(genomes.contains(jl.getGenome())){
				add(jl);
			}
		}
		JLabel jl = new JLabel(" ");
		add(jl);
		updateUI();
	}

	public void updateComparisons(List<ComparisonPanel> compList){
		removeAll();
		labelList.clear();
		//setLayout(new GridLayout(compList.size()+2,1));
		for(ComparisonPanel c : compList){
			GenomeLabel jl = new GenomeLabel(c.getComparison().getCompGenome());
			jl.addGenomeLabelListener(this);
			add(jl);
			labelList.add(jl);
		}
		JLabel jl = new JLabel(" ");
		add(jl);
	}


	public void genomeClicked(Genome g) {
		Collection<Genome> selected = syntenyPanel.getSyntenyFrame().getSynteviewToolbarPanel().getSelectReferenceGenomePanel().getSelectComparedGenomesPanel().getSelectedGenomes();
		selected.remove(g);
		selected.add(getSyntenyPanel().getSynteny().getReferenceSpecies());
		syntenyPanel.getSyntenyFrame().referenceGenomeChoosed(g, selected);
	}
}
