/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import org.synteny.Ortholog;
import org.synteny.Paralog;
import org.synteny.ihm.layout.ParalogLayout;

import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JPanel;


public class ParalogPanel extends JPanel{


	/**
	 * 
	 */
	private static final long serialVersionUID = -7364530440091663219L;

	/**
	 * @uml.property  name="paralog"
	 * @uml.associationEnd  inverse="paralogPanel:org.synteny.Paralog"
	 * @uml.association  name="hasParalog"
	 */
	private Paralog paralog;


	/**
	 * @uml.property  name="simpleBlocPanel"
	 * @uml.associationEnd  inverse="paralogPanel:org.synteny.ihm.SimpleBlocPanel"
	 * @uml.association  name="hasParalogPanel"
	 */
	private SimpleBlocPanel simpleBlocPanel;


	
	/**
	 * @uml.property   name="proteinPanel"
	 * @uml.associationEnd   multiplicity="(0 -1)" inverse="paralogPanel:org.synteny.ihm.OrthologPanel"
	 * @uml.association   name="hasProteinPanel"
	 */
	private Collection orthologPanel;

	
	public ParalogPanel(SimpleBlocPanel bp, Paralog p){
		setSimpleBlocPanel(bp);
		paralog = p;
		orthologPanel= new ArrayList<OrthologPanel>();
		ParalogLayout pl = new ParalogLayout(bp);
		setLayout(pl);
		for(Object o: p.getOrtholog()){
			Ortholog or = (Ortholog)o;
			OrthologPanel pp = new OrthologPanel(or);
			pp.setToolTipText("Block size: "+bp.getBloc().getMaximumSize());
			//JLabel jl = new JLabel(or.getCompProtein().getName());
			//jl.setOpaque(false);
			orthologPanel.add(pp);
			add(pp);
		}
		setOpaque(false);
	}

	public void paint(Graphics g){
		g.setColor(Color.lightGray);
		g.fillRoundRect(0, 0, getWidth()-1, getHeight()-1, 10, 10);
		g.setColor(Color.black);
		g.fillRect(2,getHeight()/2-2 , getWidth()-4, 4);
		super.paint(g);
	}
	
	/**
	 * Getter of the property <tt>paralog</tt>
	 * @return  Returns the paralog.
	 * @uml.property  name="paralog"
	 */
	public Paralog getParalog(){
		return paralog;
	}

	/**
	 * Setter of the property <tt>paralog</tt>
	 * @param paralog  The paralog to set.
	 * @uml.property  name="paralog"
	 */
	public void setParalog(Paralog paralog) {
		this.paralog = paralog;
	}

	/**
	 * Getter of the property <tt>orthologPanel</tt>
	 * @return  Returns the orthologPanel.
	 * @uml.property  name="orthologPanel"
	 */
	public Collection getOrthologPanel() {
		return orthologPanel;
	}

	/**
	 * Setter of the property <tt>orthologPanel</tt>
	 * @param orthologPanel  The orthologPanel to set.
	 * @uml.property  name="orthologPanel"
	 */
	public void setOrthologPanel(Collection orthologPanel) {
		this.orthologPanel = orthologPanel;
	}

	/**
	 * Getter of the property <tt>simpleBlocPanel</tt>
	 * @return  Returns the simpleBlocPanel.
	 * @uml.property  name="simpleBlocPanel"
	 */
	public SimpleBlocPanel getSimpleBlocPanel() {
		return simpleBlocPanel;
	}

	/**
	 * Setter of the property <tt>simpleBlocPanel</tt>
	 * @param simpleBlocPanel  The simpleBlocPanel to set.
	 * @uml.property  name="simpleBlocPanel"
	 */
	public void setSimpleBlocPanel(SimpleBlocPanel simpleBlocPanel) {
		this.simpleBlocPanel = simpleBlocPanel;
	}
}
