/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JLabel;
import javax.swing.JPanel;

import org.synteny.Protein;
import org.synteny.factory.IconFactory;


public class ProteinPanel extends JPanel implements MouseListener{
	/**
	 * 
	 */
	private static final long serialVersionUID = -4252830321357369491L;
	public static int PROTEIN_WIDTH = 60;
	public static int PROTEIN_MARGIN = 5;
	public static int PROTEIN_HEIGHT = 24;
	
	Color _color;
	Image _image;
	public ProteinPanel(Protein p){
		protein = p;
		setLayout(new FlowLayout());
		setPreferredSize(new Dimension(PROTEIN_WIDTH,PROTEIN_HEIGHT));
		setOpaque(false);
		syntenyListener = new ArrayList<SyntenyListener>();
		add(new JLabel(protein.getName()),BorderLayout.CENTER);
		addMouseListener(this);
		syntenyListener = new ArrayList<SyntenyListener>();
		
		if(protein.getOrthologs().size()==0){
			_color = Color.white;
			_image = IconFactory.getInstance().getWhiteProteinIcon().getImage();
		} else if(protein.getStrand()){
			_color = Color.cyan;
			_image = IconFactory.getInstance().getYellowProteinIcon().getImage();
		}else{
			_color = Color.yellow;
			_image = IconFactory.getInstance().getBlueProteinIcon().getImage();
		}
	}
	
	public void paint(Graphics g){
		Graphics2D g2 = (Graphics2D)g;
		g2.drawImage(_image, 0, 0, 60, 24, null);
	
		super.paint(g2);
	}
	
	/**
	 * @uml.property  name="protein"
	 * @uml.associationEnd  inverse="proteinPanel:org.synteny.Protein"
	 */
	private Protein protein;

	/**
	 * Getter of the property <tt>protein</tt>
	 * @return  Returns the protein.
	 * @uml.property  name="protein"
	 */
	public Protein getProtein() {
		return protein;
	}

	/**
	 * Setter of the property <tt>protein</tt>
	 * @param protein  The protein to set.
	 * @uml.property  name="protein"
	 */
	public void setProtein(Protein protein) {
		this.protein = protein;
	}

	public void addSyntenyListener(SyntenyListener syntenyListener){
		this.syntenyListener.add(syntenyListener);
	}

	public void mouseClicked(MouseEvent e) {
		for(Object o : syntenyListener){
			SyntenyListener pal = (SyntenyListener)o;
			pal.ProteinClicked(getProtein());
		}
	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	/**
	 * @uml.property  name="syntenyListener"
	 * @uml.associationEnd  multiplicity="(0 -1)" inverse="proteinPanel:org.synteny.ihm.SyntenyListener"
	 */
	private Collection syntenyListener;

	/**
	 * Getter of the property <tt>syntenyListener</tt>
	 * @return  Returns the syntenyListener.
	 * @uml.property  name="syntenyListener"
	 */
	public Collection getSyntenyListener() {
		return syntenyListener;
	}

	/**
	 * Setter of the property <tt>syntenyListener</tt>
	 * @param syntenyListener  The syntenyListener to set.
	 * @uml.property  name="syntenyListener"
	 */
	public void setSyntenyListener(Collection syntenyListener) {
		this.syntenyListener = syntenyListener;
	}
}
