/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.LayoutManager;

import javax.swing.JPanel;

import org.synteny.Genome;
import org.synteny.Protein;
import org.synteny.ihm.layout.ProteomeLayout;
import java.util.Collection;


/**
 * @uml.dependency   supplier="org.synteny.ihm.ProteinPanel"
 */
public class RefProteomePanel extends JPanel {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = -9060948839395271491L;

	/**
	 * @uml.property  name="_refGenome"
	 * @uml.associationEnd  inverse="refProteomePanel:org.synteny.Genome"
	 * @uml.association  name="hasGenome"
	 */
	private Genome _refGenome;
	
	private LayoutManager _proteomeLayout;
	/**
	 * @uml.property  name="syntenyPanel"
	 * @uml.associationEnd  inverse="refProteomePanel:org.synteny.ihm.SyntenyPanel"
	 * @uml.association  name="hasRefGenomePanel"
	 */
	private SyntenyPanel syntenyPanel;
	
	public RefProteomePanel(){
		super();
		_proteomeLayout = new ProteomeLayout();
		setLayout(_proteomeLayout);
	}
	
	public void updateRefGenome(Genome ref){
		_refGenome = ref;
		removeAll();
		for(Object o : _refGenome.getProtein()){
			Protein p = (Protein)o;
			ProteinPanel pp = new ProteinPanel(p);
			pp.setToolTipText("Number of orthologs in synteny groups: "+p.getOrthologs().size());
			pp.addSyntenyListener(getSyntenyPanel());
			add(pp);
		}
		updateUI();
	}
	
	/**
	 * Getter of the property <tt>syntenyPanel</tt>
	 * @return  Returns the syntenyPanel.
	 * @uml.property  name="syntenyPanel"
	 */
	public SyntenyPanel getSyntenyPanel() {
		return syntenyPanel;
	}

	/**
	 * Setter of the property <tt>syntenyPanel</tt>
	 * @param syntenyPanel  The syntenyPanel to set.
	 * @uml.property  name="syntenyPanel"
	 */
	public void setSyntenyPanel(SyntenyPanel syntenyPanel) {
		this.syntenyPanel = syntenyPanel;
	}

	/**
	 * Getter of the property <tt>genome</tt>
	 * @return  Returns the genome.
	 * @uml.property  name="genome"
	 */
	public Genome getGenome() {
		return _refGenome;
	}

	/**
	 * Setter of the property <tt>genome</tt>
	 * @param genome  The genome to set.
	 * @uml.property  name="genome"
	 */
	public void setGenome(Genome genome) {
		this._refGenome = genome;
	}
}
