/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.synteny.Protein;
import org.synteny.ihm.preferences.TabLabel;
import org.synteny.search.ProteinSearcher;
import org.synteny.search.SearchListener;


public class SearchProteinPanel extends JPanel implements Closable, ActionListener,ProteinActionListener,KeyListener,SearchListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = -4455641263767001885L;
	private Collection<CloseListener> _closeListeners;
	/**
	 * @uml.property  name="searchResultPanel"
	 * @uml.associationEnd  inverse="searchProteinPanel:org.synteny.ihm.SearchResultPanel"
	 * @uml.association  name="hasSearchResult"
	 */
	private SearchResultPanel searchResultPanel;
	private JLabel _proteinPid;
	private JLabel _proteinStart;
	private JLabel _proteinStop;
	private JTextArea _proteinSpecies;
	private JLabel _proteinGeneName;
	private JLabel _proteinChromosome;
	private Protein _protein;
	private JTextArea _proteinFunction;
	private JButton _oneSequenceButton;
	
	private JCheckBox _functionCheckBox;
	private JCheckBox _geneNameCheckBox;

	private JButton _searchButton;
	private JTextField _textSearch;
	private BlocSizeAdjustmentWidget _blocSizeAdjustmentWidget;

	public SearchProteinPanel(){
		super();
		_closeListeners = new ArrayList<CloseListener>();
		TabLabel title = new TabLabel("Gene information");
		title.getCloseButton().addActionListener(this);
		setLayout(new BorderLayout());
		add(title,BorderLayout.NORTH);
		setBackground(Color.white);
		JPanel jp = new JPanel();
		jp.setBackground(Color.white);
		jp.setLayout(new BoxLayout(jp,BoxLayout.Y_AXIS));

		Box b = Box.createHorizontalBox();
		b.setBorder(BorderFactory.createTitledBorder("Identifier"));
		b.setAlignmentX(Box.LEFT_ALIGNMENT);
		b.add(getPidLabel());
		getPidLabel().setOpaque(false);
		b.add(Box.createHorizontalGlue());
		jp.add(b);

		jp.add(createInfosSpacer());

		Box b2 = Box.createVerticalBox();
		b2.setMaximumSize(null);
		b2.setAlignmentX(Box.LEFT_ALIGNMENT);
		b2.setBorder(BorderFactory.createTitledBorder("Coordinates"));
		Box b21 = Box.createVerticalBox();
		b21.setMaximumSize(null);
		b21.setAlignmentX(Box.LEFT_ALIGNMENT);
		Box b22 = Box.createHorizontalBox();
		b22.add(new JLabel("Start:"));
		b22.add(getStartLabel());
		b22.add(Box.createHorizontalGlue());
		Box b23 = Box.createHorizontalBox();
		b23.add(new JLabel("Stop: "));
		b23.add(getStopLabel());
		b23.add(Box.createHorizontalGlue());
		b21.add(b22);
		b21.add(Box.createRigidArea(new Dimension(20,20)));
		b21.add(b23);
		b2.add(b21);
		jp.add(b2);

		jp.add(createInfosSpacer());

		Box b3 = Box.createHorizontalBox();
		b3.setAlignmentX(Box.LEFT_ALIGNMENT);
		b3.setBorder(BorderFactory.createTitledBorder("Species"));
		getSpeciesLabel().setColumns(20);
		b3.add(getSpeciesLabel());
		
		b3.add(Box.createHorizontalGlue());
		jp.add(b3);

		jp.add(createInfosSpacer());

		Box b4 = Box.createHorizontalBox();
		b4.setBorder(BorderFactory.createTitledBorder("Gene Name"));
		b4.setAlignmentX(Box.LEFT_ALIGNMENT);
		b4.add(getGeneNameLabel());
		b4.add(Box.createHorizontalGlue());
		jp.add(b4);

		jp.add(createInfosSpacer());

		Box b5 = Box.createHorizontalBox();
		b5.setBorder(BorderFactory.createTitledBorder("Function"));
		b5.setAlignmentX(Box.LEFT_ALIGNMENT);
		b5.add(new JScrollPane(getFunctionLabel()));
		jp.add(b5);

		jp.add(createInfosSpacer());

		Box b6 = Box.createHorizontalBox();
		b6.setBorder(BorderFactory.createTitledBorder("Chromosome"));
		b6.setAlignmentX(Box.LEFT_ALIGNMENT);
		b6.add(getChromosomeLabel());
		b6.add(Box.createHorizontalGlue());
		jp.add(b6);

		jp.add(createInfosSpacer());

		Box b6bis = Box.createHorizontalBox();
		b6bis.setBorder(BorderFactory.createTitledBorder("Sequence information"));
		b6bis.setAlignmentX(Box.LEFT_ALIGNMENT);
		b6bis.add(getOneSequenceButton());
		b6bis.add(Box.createHorizontalGlue());
		jp.add(b6bis);

		jp.add(Box.createVerticalGlue());

		Box searchBlocBox = Box.createVerticalBox();
		searchBlocBox.setName("Search Blocks");
		searchBlocBox.setAlignmentX(Box.LEFT_ALIGNMENT);
		searchBlocBox.add(getBlocSizeAdjustmentWidget());

		Box searchBox = Box.createVerticalBox();
		searchBox.setName("Search Proteins");
		searchBox.setBackground(Color.white);
		searchBox.setAlignmentX(Box.LEFT_ALIGNMENT);
		//searchBox.setBackground(Color.LIGHT_GRAY);
		searchBox.add(createInfosSpacer());
		Box b8 = Box.createVerticalBox();
		b8.setBackground(Color.LIGHT_GRAY);
		b8.setBorder(BorderFactory.createTitledBorder(""));
		//b8.setMaximumSize(null);
		b8.setAlignmentX(Box.LEFT_ALIGNMENT);
		b8.add(getSearchPanel());
		searchBox.add(b8);

		JTabbedPane infos = new JTabbedPane();
		infos.setAlignmentX(Box.LEFT_ALIGNMENT);
		infos.add(searchBox);
		infos.add(searchBlocBox);
		jp.add(infos);
		
		infos.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent arg0) {
				// TODO Auto-generated method stub
				getSearchResultPanel().clear();
			}
		});

		Box last = Box.createVerticalBox();
		last.setAlignmentX(Box.LEFT_ALIGNMENT);
		last.add(getSearchResultPanel());
		jp.add(last);

		add(new JScrollPane(jp),BorderLayout.CENTER);
	}

	private JButton getOneSequenceButton() {
		if(_oneSequenceButton==null){
			_oneSequenceButton = new JButton("get sequence");
			_oneSequenceButton.setActionCommand("OneSequence");
			_oneSequenceButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					System.out.println(_protein.getSequence());
					SequenceDialog.showSequenceDialog(_protein);
				}
			});
		}
		return _oneSequenceButton;
	}

	private JButton getSearchButton() {
		if(_searchButton==null){
			_searchButton = new JButton("Search");
			_searchButton.setActionCommand("search");
			_searchButton.addActionListener(this);
		}
		return _searchButton;
	}

	private JTextField getTextSearch() {
		if(_textSearch==null){
			_textSearch = new JTextField();
			_textSearch.setForeground(Color.black);
			_textSearch.setMaximumSize(new Dimension(3000,30));
			_textSearch.addKeyListener(this);
		}
		return _textSearch;
	}
	
	private JCheckBox getFunctionCheckBox(){
		if(_functionCheckBox==null){
			_functionCheckBox = new JCheckBox();
			_functionCheckBox.setSelected(true);
		}
		
		return _functionCheckBox;
	}
	
	private JCheckBox getGeneNameCheckBox(){
		if(_geneNameCheckBox==null){
			_geneNameCheckBox = new JCheckBox();
			_geneNameCheckBox.setSelected(true);
		}
		return _geneNameCheckBox;
	}
	
	private JPanel getSearchPanel(){
		JPanel jp = new JPanel();
		jp.setLayout(new BoxLayout(jp,BoxLayout.PAGE_AXIS));
		jp.setAlignmentX(Box.LEFT_ALIGNMENT);
		Box functionBox = Box.createHorizontalBox();
		functionBox.add(getFunctionCheckBox());
		functionBox.add(new JLabel("Search function"));
		
		Box geneBox = Box.createHorizontalBox();
		geneBox.add(getGeneNameCheckBox());
		geneBox.add(new JLabel("Search gene name"));
		
		jp.add(functionBox);
		jp.add(geneBox);
	
		Box searchBox = Box.createHorizontalBox();
		searchBox.add(getTextSearch());
		searchBox.add(getSearchButton());
		
		jp.add(searchBox);
		return jp;
	}

	public void addCloseListener(CloseListener closelistener) {
		_closeListeners.add(closelistener);
	}

	public void close() {
	}

	/**
	 * Getter of the property <tt>searchResultPanel</tt>
	 * @return  Returns the searchResultPanel.
	 * @uml.property  name="searchResultPanel"
	 */
	public SearchResultPanel getSearchResultPanel() {
		if(searchResultPanel == null){
			searchResultPanel = new SearchResultPanel();
			searchResultPanel.addSearchListener(this);
			searchResultPanel.setAlignmentX(JPanel.LEFT_ALIGNMENT);
		}
		return searchResultPanel;
	}

	/**
	 * Setter of the property <tt>searchResultPanel</tt>
	 * @param searchResultPanel  The searchResultPanel to set.
	 * @uml.property  name="searchResultPanel"
	 */
	public void setSearchResultPanel(SearchResultPanel searchResultPanel) {
		this.searchResultPanel = searchResultPanel;
	}

	public void actionPerformed(ActionEvent e) {
		if(e.getActionCommand().equals("search")){
			search();
		} else if(e.getActionCommand().equals("searchBloc")){
			searchBloc();
		}else{
			for(CloseListener cl : _closeListeners){
				cl.ComponentClosed(this);
			}
		}
	}

	public JLabel getPidLabel(){
		if(_proteinPid==null){
			_proteinPid = createInfoLabel();
		}

		return _proteinPid;
	}

	public JLabel getStartLabel(){
		if(_proteinStart==null){
			_proteinStart = createInfoLabel();
		}

		return _proteinStart;
	}

	public JLabel getStopLabel(){
		if(_proteinStop==null){
			_proteinStop = createInfoLabel();
		}

		return _proteinStop;
	}

	public JTextArea getSpeciesLabel(){
		if(_proteinSpecies==null){
			_proteinSpecies = createInfoTextField();
		}
		return _proteinSpecies;
	}

	public JLabel getGeneNameLabel(){
		if(_proteinGeneName==null)
			_proteinGeneName = createInfoLabel();

		return _proteinGeneName;
	}

	public JTextArea getFunctionLabel(){
		if(_proteinFunction==null)
			_proteinFunction = createInfoTextField();
		return _proteinFunction;
	}

	public JLabel getChromosomeLabel(){
		if(_proteinChromosome==null)
			_proteinChromosome= createInfoLabel();

		return _proteinChromosome;
	}

	public void ProteinActivated(Protein protein) {
		_protein = protein;
		getSpeciesLabel().setText(protein.getGenome().getSpeciesName());
		getPidLabel().setText(String.valueOf((protein.getIdentifier())));
		getStartLabel().setText(String.valueOf(protein.getGenomeStart()));
		getStopLabel().setText(String.valueOf(protein.getGenoneStop()));
		getFunctionLabel().setText(protein.getFunction());
		getGeneNameLabel().setText(protein.getName());
		getChromosomeLabel().setText(protein.getChromosome());
	}

	public Box createInfosSpacer(){
		Box rigid = Box.createHorizontalBox();
		rigid.add(Box.createRigidArea(new Dimension(20,20)));
		rigid.setAlignmentX(Box.LEFT_ALIGNMENT);
		return rigid;
	}

	public JLabel createInfoLabel(){
		JLabel info = new JLabel(" ");
		//info.setForeground(new Color(130, 147, 162));
		info.setForeground(Color.black);
		return info;
	}

	public JTextArea createInfoTextField(){
		JTextArea info = new JTextArea(" ",5,15);
		//info.setForeground(new Color(130, 147, 162));
		info.setForeground(Color.black);
		info.setLineWrap(true);
		info.setMaximumSize(new Dimension(80,80));
		info.setSize(new Dimension(80,80));
		info.setPreferredSize(new Dimension(80,80));
		info.setEditable(false);
		return info;
	}

	/**
	 * @uml.property  name="proteinSearcher"
	 * @uml.associationEnd  inverse="searchProteinPanel:org.synteny.search.ProteinSearcher"
	 */
	private ProteinSearcher proteinSearcher;


	/**
	 * Getter of the property <tt>proteinSearcher</tt>
	 * @return  Returns the proteinSearcher.
	 * @uml.property  name="proteinSearcher"
	 */
	public ProteinSearcher getProteinSearcher() {
		return proteinSearcher;
	}

	/**
	 * Setter of the property <tt>proteinSearcher</tt>
	 * @param proteinSearcher  The proteinSearcher to set.
	 * @uml.property  name="proteinSearcher"
	 */
	public void setProteinSearcher(ProteinSearcher proteinSearcher) {
		this.proteinSearcher = proteinSearcher;
	}

	public void addSearchListener(SearchListener sl){
		getSearchResultPanel().addSearchListener(sl);
	}

	public void keyPressed(KeyEvent arg0) {
		System.out.println(arg0.getKeyCode());
		if(arg0.getKeyCode() == KeyEvent.VK_ENTER){
			search();
		}
	}

	public void keyReleased(KeyEvent arg0) {
		// TODO Auto-generated method stub

	}

	public void keyTyped(KeyEvent arg0) {
	}

	public void search(){
		Collection<Protein> results;
		try {
			results = getProteinSearcher().searchProtein(getTextSearch().getText(),getFunctionCheckBox().isSelected(),getGeneNameCheckBox().isSelected());
			getSearchResultPanel().setResults(results);
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this, "Synteny data has not been retrieved from the server","Search error",JOptionPane.ERROR_MESSAGE);
		}
	}

	public void searchBloc(){
		Collection<Protein> results;
		try {
			results = getProteinSearcher().searchBlocProtein(getBlocSizeAdjustmentWidget().getBlocWith(),getBlocSizeAdjustmentWidget().getBlocHeight());
			getSearchResultPanel().setResults(results);
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this, "Synteny data has not been retrieved from the server","Search error",JOptionPane.ERROR_MESSAGE);
		}
	}

	public void proteinSearchedFor(Protein protein) {
		ProteinActivated(protein);
	}

	public BlocSizeAdjustmentWidget getBlocSizeAdjustmentWidget(){
		if(_blocSizeAdjustmentWidget == null){
			_blocSizeAdjustmentWidget = new BlocSizeAdjustmentWidget();
			_blocSizeAdjustmentWidget.addActionListener(this);
		}

		return _blocSizeAdjustmentWidget;
	}
}
