/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;

import org.synteny.Protein;
import org.synteny.ihm.preferences.SelectHistogramGenomesCellRenderer;
import org.synteny.search.SearchListener;


public class SearchResultPanel extends JPanel implements MouseListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = -8439243395923268316L;
	Collection <SearchListener> _searchListeners;

	private JTable _table;
	
	public SearchResultPanel(){
		super();
		setLayout(new BorderLayout());
		_table = new JTable(){
			public boolean isCellEditable(int row, int column)
			{
				return false;
			}
		};
		_searchListeners = new ArrayList<SearchListener>();
		_table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		_table.setDefaultRenderer(Object.class, new SelectHistogramGenomesCellRenderer());
		_table.addMouseListener(this);
		//_table.setAutoCreateRowSorter(true);
		setVisible(true);
		
		JScrollPane jsp = new JScrollPane(_table);
		jsp.setPreferredSize(new Dimension(50,300));
		
		add(jsp,BorderLayout.CENTER);
	}

	public void addSearchListener(SearchListener sl){
		_searchListeners.add(sl);
	}

	public void setResults(Collection<Protein> results){
		Vector<Protein> v = new Vector<Protein>();
		v.addAll(results);
		Collections.sort(v);
		
		DefaultTableModel tm = new DefaultTableModel();
		String[] colnames = {"#","Gene name","Function"};
		tm.setColumnIdentifiers(colnames);
		int i=1;
		for(Protein p : results){
			Object[] infos = new Object[3];
			infos[0] = i;
			infos[1] = p;
			infos[2] = p.getFunction();
			tm.addRow(infos);
			i++;
		}
		_table.setModel(tm);
		_table.getTableHeader().resizeAndRepaint();
		System.out.println("Search Protein");
	}

	public void mouseClicked(MouseEvent e) {
		System.out.println(_table.getValueAt(_table.getSelectedRow(),1));
		if (e.getClickCount() == 1) {
			if (_table.getValueAt(_table.getSelectedRow(),1) instanceof Protein) {
				Protein p = (Protein) _table.getValueAt(_table.getSelectedRow(),1);
				for(SearchListener sl : _searchListeners){
					sl.proteinSearchedFor(p);
				}
			}
		}else{
		
		}
	}
	
	public void clear(){
		if(_table.getModel().getRowCount()>0)
			_table.setModel(new DefaultTableModel());
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {
	}

	public void mouseReleased(MouseEvent e) {
	}
}
