/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collection;
import java.util.HashSet;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;

import org.alignment.MultipleAlignerFactory;
import org.alignment.MultipleSequenceAligner;
import org.alignment.view.MultipleAlignmentHtmlView;
import org.synteny.Ortholog;
import org.synteny.Protein;
import org.synteny.factory.IconFactory;


public class SequenceDialog extends JDialog implements ActionListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 7020832229912013442L;

	/**
	 * @uml.property  name="protein"
	 * @uml.associationEnd  inverse="sequenceDialog:org.synteny.Protein"
	 */
	private Protein protein;
	private Collection<Protein> _orthologs;

	private JButton closeButton;
	private JTabbedPane tabbedPane;
	private JPanel oneSequencePanel;
	private JPanel orthoSequencePanel;
	private JTextArea orthoTextArea;
	private JTextArea oneSequenceArea;

	public SequenceDialog(Protein protein){
		super();
		((java.awt.Frame)getOwner()).setIconImage(IconFactory.getInstance().getLogoIcon().getImage());
		setModal(true);
		setLayout(new BorderLayout());
		setSize(new Dimension(400,600));
		setTitle("Sequences viewer");
		setLocationRelativeTo(null);
		this.protein = protein;
		
		_orthologs = new HashSet<Protein>();
		Collection<Protein> refProts = new HashSet<Protein>();
		for(Ortholog o : protein.getOrthologs()){
			refProts.add(o.getRefProtein());
		}
		for(Protein p : refProts){
			_orthologs.add(p);
			for(Ortholog o : p.getOrthologs()){
				_orthologs.add(o.getCompProtein());
			}
		}

		add(getTabbedPane(),BorderLayout.CENTER);
		add(getCloseButton(),BorderLayout.SOUTH);
		setPreferredSize(new Dimension(100,100));
	}

	/**
	 * Getter of the property <tt>protein</tt>
	 * @return  Returns the protein.
	 * @uml.property  name="protein"
	 */
	public Protein getProtein() {
		return protein;
	}

	public JButton getCloseButton(){
		if(closeButton == null){
			closeButton = new JButton("close");
			closeButton.setToolTipText("Close Dialog");
			closeButton.addActionListener(this);
		}
		return closeButton;
	}

	/**
	 * Setter of the property <tt>protein</tt>
	 * @param protein  The protein to set.
	 * @uml.property  name="protein"
	 */
	public void setProtein(Protein protein) {
		this.protein = protein;
	}

	public JTabbedPane getTabbedPane(){
		if(tabbedPane == null){
			tabbedPane = new JTabbedPane();
			tabbedPane.addTab("Protein Sequence (Fasta)", getOneSequenceTextPane());
			tabbedPane.addTab("Orthologs Sequence (Fasta)", getOrthoSequenceTextPane());		
			}
		return tabbedPane;
	}

	public JPanel getOneSequenceTextPane(){
		if(oneSequencePanel == null){
			oneSequencePanel = new JPanel();
			oneSequencePanel.setLayout(new BorderLayout());
			
			JToolBar jtb = new JToolBar();
			jtb.setFloatable(false);
			JButton jb1 = new JButton(IconFactory.getInstance().getSaveIcon());
			jb1.setToolTipText("Save sequence");
			jtb.add(jb1);
			
		
			jb1.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					JFileChooser jf = new JFileChooser();
					if(jf.showSaveDialog(SequenceDialog.this)==JFileChooser.APPROVE_OPTION){
						File f = jf.getSelectedFile();
						try {
							PrintWriter pw= new PrintWriter(new BufferedWriter(new FileWriter(f)));
							pw.println(getOneSequenceArea().getText());
							pw.close();
						} catch (IOException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						}
					}
				}
			});
			
		
			oneSequencePanel.add(jtb,BorderLayout.NORTH);
			
			JTextArea jta = getOneSequenceArea();
			String s = protein.toFasta();
			jta.setText(s);
			oneSequencePanel.add(new JScrollPane(jta),BorderLayout.CENTER);
		}
		return oneSequencePanel;
	}

	public JPanel getOrthoSequenceTextPane(){
		if(orthoSequencePanel == null){
			orthoSequencePanel = new JPanel();
			orthoSequencePanel.setLayout(new BorderLayout());

			JToolBar jtb = new JToolBar();
			jtb.setFloatable(false);
			JButton jb1 = new JButton(IconFactory.getInstance().getSaveIcon());
			jb1.setToolTipText("Save multiple sequences");
			jtb.add(jb1);
			
			
			/*JButton jb2 = new JButton(IconFactory.getInstance().getAlignIcon());
			jb2.setToolTipText("Perform multiple alignment");
			jtb.add(jb2);
			
			jb2.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent arg0) {
					MultipleSequenceAligner msa = MultipleAlignerFactory.createMultipleSequenceAligner();
					msa.setProtein(_orthologs);
					msa.alignSequences();
					MultipleAlignmentHtmlView.showMultipleAlignmentHtmlView(msa.getTextAlignement());
				}
			});*/
			
			jb1.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					JFileChooser jf = new JFileChooser();
					if(jf.showSaveDialog(SequenceDialog.this)==JFileChooser.APPROVE_OPTION){
						File f = jf.getSelectedFile();
						try {
							PrintWriter pw= new PrintWriter(new BufferedWriter(new FileWriter(f)));
							pw.println(getOrthoJTextArea().getText());
							pw.close();
						} catch (IOException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						}
					}
				}
			});
			
			orthoSequencePanel.add(jtb,BorderLayout.NORTH);
			JTextArea jta = getOrthoJTextArea();
			StringBuffer sb = new StringBuffer();
			for(Protein p : _orthologs){
				sb.append(p.toFasta());
			}
			jta.setText(sb.toString());
			orthoSequencePanel.add(new JScrollPane(jta));
		}
		return orthoSequencePanel;
	}

	public JTextArea getOrthoJTextArea(){
		if(orthoTextArea==null){
			orthoTextArea = new JTextArea();
			orthoTextArea.setFont(new Font("Monospaced",Font.PLAIN,12));
		}
		return orthoTextArea;
	}
	
	public JTextArea getOneSequenceArea(){
		if(oneSequenceArea==null){
			oneSequenceArea = new JTextArea();
			oneSequenceArea.setFont(new Font("Monospaced",Font.PLAIN,12));
		}
		return oneSequenceArea;
	}

	/**
	 */
	public static void showSequenceDialog(Protein protein){
		SequenceDialog sd = new SequenceDialog(protein);
		sd.setVisible(true);
	}

	public void actionPerformed(ActionEvent e) {
		if(e.getActionCommand().equals("close")){
			dispose();
		}
	}

}
