/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Iterator;

import javax.swing.JPanel;

import org.synteny.Bloc;
import org.synteny.Ortholog;
import org.synteny.Paralog;
import org.synteny.ihm.layout.BlocLayout;


public class SimpleBlocPanel extends JPanel implements MouseListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 5411430304292702188L;
	public static final int BLOC_MARGIN = 1;
	
	/**
	 * @uml.property  name="comparisonPanel"
	 * @uml.associationEnd  inverse="blocPanel:org.synteny.ihm.ComparisonPanel"
	 * @uml.association  name="hasBlocPanels"
	 */
	private ComparisonPanel comparisonPanel;

	/**
	 * @uml.property  name="bloc"
	 * @uml.associationEnd  inverse="blocPanel:org.synteny.Bloc"
	 * @uml.association  name="hasBloc"
	 */
	private Bloc bloc;

	/** 
	 * @uml.property name="paralogPanel"
	 * @uml.associationEnd inverse="simpleBlocPanel:org.synteny.ihm.ParalogPanel"
	 * @uml.association name="hasParalogPanel"
	 */
	private ParalogPanel paralogPanel;

	/**
	 * @uml.property   name="paralogDialog"
	 * @uml.associationEnd   inverse="blocPanel:org.synteny.ihm.ComplexBlocDialog"
	 * @uml.association   name="hasParalogDialog"
	 */
	private ComplexBlocDialog paralogDialog;

	
	/**
	 */
	public SimpleBlocPanel(Bloc bloc){
		super();
		this.bloc = bloc;
		setLayout(new BlocLayout());
		Iterator<Paralog> it = bloc.getParalog().iterator();
		setToolTipText("Synteny block size: "+bloc.getMaximumSize());
		if(it.hasNext()){
			Paralog p = it.next();
			paralogPanel = new ParalogPanel(this,p);
		}
		setOpaque(false);
		add(paralogPanel,BorderLayout.CENTER);
		//addMouseListener(this);
	}
	
	/**
	 * Getter of the property <tt>comparisonPanel</tt>
	 * @return  Returns the comparisonPanel.
	 * @uml.property  name="comparisonPanel"
	 */
	public ComparisonPanel getComparisonPanel() {
		return comparisonPanel;
	}

	/**
	 * Setter of the property <tt>comparisonPanel</tt>
	 * @param comparisonPanel  The comparisonPanel to set.
	 * @uml.property  name="comparisonPanel"
	 */
	public void setComparisonPanel(ComparisonPanel comparisonPanel) {
		this.comparisonPanel = comparisonPanel;
	}

	/**
	 */
	public int getGenomePosition(){
		int minPosition = bloc.getComparison().getRefGenome().getSize();
		for(Object o : bloc.getParalog()){
			Paralog p = (Paralog)o;
			Ortholog ort = (Ortholog)(p.getOrtholog().get(0));
			int pos = ort.getRefProtein().getGenomePosition();
			if(pos<minPosition){
				minPosition = pos;
			}
		}
		return minPosition;
	}
	
	public void mouseClicked(MouseEvent e) {
		if(paralogDialog==null){
			paralogDialog = new ComplexBlocDialog(this);
		}
		System.out.println("Espece: "+bloc.getComparison().getCompGenome().getSpeciesAbbrv());
		System.out.println("Taille max:"+bloc.getMaximumSize());
		paralogDialog.setVisible(true);
		for(Object o : bloc.getParalog()){
			Paralog p= (Paralog)o;
			for(Object o2 : p.getOrtholog()){
				Ortholog or = (Ortholog)o2;
				System.out.print("Position: "+or.getRefProtein().getGenomePosition()+"--"+or.getCompProtein().getIdentifier()+"/"+or.getRefProtein().getIdentifier()+"-");
			}
			System.out.println();
		}
	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}


	/**
	 * Getter of the property <tt>bloc</tt>
	 * @return  Returns the bloc.
	 * @uml.property  name="bloc"
	 */
	public Bloc getBloc() {
		return bloc;
	}

	/**
	 * Setter of the property <tt>bloc</tt>
	 * @param bloc  The bloc to set.
	 * @uml.property  name="bloc"
	 */
	public void setBloc(Bloc bloc) {
		this.bloc = bloc;
	}


	/** 
	 * Getter of the property <tt>paralogDialog</tt>
	 * @return  Returns the paralogDialog.
	 * @uml.property  name="paralogDialog"
	 */
	public ComplexBlocDialog getParalogDialog() {
		return paralogDialog;
	}

	/** 
	 * Setter of the property <tt>paralogDialog</tt>
	 * @param paralogDialog  The paralogDialog to set.
	 * @uml.property  name="paralogDialog"
	 */
	public void setParalogDialog(ComplexBlocDialog paralogDialog) {
		this.paralogDialog = paralogDialog;
	}


	/** 
	 * Getter of the property <tt>paralogPanel</tt>
	 * @return  Returns the paralogPanel.
	 * @uml.property  name="paralogPanel"
	 */
	public ParalogPanel getParalogPanel() {
		return paralogPanel;
	}

	/** 
	 * Setter of the property <tt>paralogPanel</tt>
	 * @param paralogPanel  The paralogPanel to set.
	 * @uml.property  name="paralogPanel"
	 */
	public void setParalogPanel(ParalogPanel paralogPanel) {
		this.paralogPanel = paralogPanel;
	}
	
	public void paint(Graphics g){
		super.paint(g);
		if(bloc.getNbParalogs()>1){
			g.setColor(Color.black);
			g.fillOval(0, 0, 15, 15);
			g.setColor(Color.green);
			g.fillRect(1, 7, 13, 2);
			g.fillRect(7, 1, 2, 13);
		}
	}
}
