/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.synteny.Genome;


public class SizeDistributionDialog extends JDialog implements ActionListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = -1856883509488752441L;
	/**
	 * @uml.property  name="blocSizeHistogram"
	 * @uml.associationEnd  inverse="sizeDistributionDialog:org.synteny.ihm.BlocSizeHistogram"
	 * @uml.association  name="hasHistogram"
	 */
	private BlocSizeHistogram blocSizeHistogram;
	private Map<Integer,Integer> _data;
	private Genome _genome1,_genome2;
	
	
	private SizeDistributionDialog(Map<Integer, Integer> map, Genome sp1,Genome sp2){
		super();
		blocSizeHistogram = new BlocSizeHistogram(map,sp1.getSpeciesName(),sp2.getSpeciesName());
		_data = map;
		_genome1 = sp1;
		_genome2 = sp2;
		setModal(false);
		setLayout(new BorderLayout());
		
		JButton jb = new JButton("quit");
		jb.addActionListener(this);
		JButton save = new JButton("saveData");
		save.addActionListener(this);
		
		JPanel but = new JPanel();
		but.setLayout(new FlowLayout());
		but.add(jb);
		but.add(save);
		add(but,BorderLayout.SOUTH);
		add(getHistogramInfos(),BorderLayout.NORTH);
		
		
		add(blocSizeHistogram,BorderLayout.CENTER);
		pack();
	}
	
	public static void showDialog(Map<Integer,Integer> map,Genome sp1,Genome sp2){
		SizeDistributionDialog sdd = new SizeDistributionDialog(map,sp1,sp2);
		sdd.setVisible(true);
	}

	public void actionPerformed(ActionEvent e) {
		if(e.getActionCommand().equals("quit")){
			dispose();
		}else if(e.getActionCommand().equals("saveData")){
			saveData();
		}
	}

	/**
	 * Getter of the property <tt>blocSizeHistogram</tt>
	 * @return  Returns the blocSizeHistogram.
	 * @uml.property  name="blocSizeHistogram"
	 */
	public BlocSizeHistogram getBlocSizeHistogram() {
		return blocSizeHistogram;
	}

	/**
	 * Setter of the property <tt>blocSizeHistogram</tt>
	 * @param blocSizeHistogram  The blocSizeHistogram to set.
	 * @uml.property  name="blocSizeHistogram"
	 */
	public void setBlocSizeHistogram(BlocSizeHistogram blocSizeHistogram) {
		this.blocSizeHistogram = blocSizeHistogram;
	}
	
	public void saveData(){
		JFileChooser jf = new JFileChooser();
		if(jf.showSaveDialog(this)==JFileChooser.APPROVE_OPTION){
			File f = jf.getSelectedFile();
			PrintWriter fichier;
			try {
				fichier = new PrintWriter(new BufferedWriter(new FileWriter(f)));
				for(Integer i : _data.keySet()){
					fichier.println(i+","+_data.get(i));
				}
				fichier.close();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		}
	}
	
	public double getAverage(){
		double total = 0;
		double sum = 0;
		
		for(Integer i : _data.keySet()){
			total += _data.get(i);
			sum += (i* _data.get(i));
		}
		
		return (sum/total);
	}
	
	public int getMaximum(){
		int max = -1;
		for(Integer i : _data.keySet()){
			if(i>max) max = i;
		}
		return max;
	}
	
	public JLabel getHistogramInfos(){
		JLabel infos = new JLabel();
		
		infos.setText("Infos: Maxi="+getMaximum()+" Average="+new DecimalFormat("0.00").format(getAverage())+" reference="+_genome1.getSize()+" proteins,  compared="+_genome2.getSize()+" proteins");
		return infos;
	}
}
