/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JSlider;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;

import org.synteny.Genome;
import org.synteny.Synteny;
import org.synteny.factory.IconFactory;
import org.synteny.task.SelectReferenceGenomeTask;
import org.synteny.task.TaskProgressDialog;

public class SyntenyFrame extends JFrame implements GenomeSelectionListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = -4746683845787809710L;

	private JSplitPane jsplitpane;

	private SelectReferenceGenomeTask _newRefGenomeTask = null;
	private TaskProgressDialog _taskProgressDialog;


	public SyntenyFrame(Synteny s){
		super();
		setTitle("SynteView");
		setIconImage(IconFactory.getInstance().getLogoIcon().getImage());
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		jsplitpane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);

		syntenyPanel = new SyntenyPanel(s);
		syntenyPanel.setSyntenyFrame(this);
		setLayout(new BorderLayout());

		jsplitpane.setRightComponent(syntenyPanel);

		synteviewToolbarPanel = new SynteviewToolbarPanel(this);
		jsplitpane.setLeftComponent(synteviewToolbarPanel);
		add(jsplitpane,BorderLayout.CENTER);

		JToolBar jtb = new JToolBar(JToolBar.HORIZONTAL);
		jtb.setLayout(new BoxLayout(jtb,BoxLayout.X_AXIS));
		jtb.add(Box.createHorizontalGlue());
		jtb.setFloatable(false);
		final JSlider jsl = new JSlider(1,100);

		_newRefGenomeTask = new SelectReferenceGenomeTask();
		_taskProgressDialog = new TaskProgressDialog(_newRefGenomeTask);
		
		//jtb.add(jsl);
		jtb.add(_taskProgressDialog);
		jtb.add(Box.createHorizontalStrut(2));
		add(jtb,BorderLayout.NORTH);
		setPreferredSize(new Dimension(800,600));
		pack();
		jsplitpane.setDividerLocation(1.0);
		setLocationRelativeTo(null);
		setVisible(true);
	}

	/**
	 * @uml.property  name="synteviewToolbarPanel"
	 * @uml.associationEnd  inverse="syntenyFrame:org.synteny.ihm.SynteviewToolbarPanel"
	 * @uml.association  name="hasToolBarPanel"
	 */
	private SynteviewToolbarPanel synteviewToolbarPanel;


	/**
	 * Getter of the property <tt>synteviewToolbarPanel</tt>
	 * @return  Returns the synteviewToolbarPanel.
	 * @uml.property  name="synteviewToolbarPanel"
	 */
	public SynteviewToolbarPanel getSynteviewToolbarPanel() {
		return synteviewToolbarPanel;
	}


	/**
	 * Setter of the property <tt>synteviewToolbarPanel</tt>
	 * @param synteviewToolbarPanel  The synteviewToolbarPanel to set.
	 * @uml.property  name="synteviewToolbarPanel"
	 */
	public void setSynteviewToolbarPanel(
			SynteviewToolbarPanel synteviewToolbarPanel) {
		this.synteviewToolbarPanel = synteviewToolbarPanel;
	}


	/**
	 * @uml.property  name="syntenyPanel"
	 * @uml.associationEnd  inverse="syntenyFrame:org.synteny.ihm.SyntenyPanel"
	 * @uml.association  name="hasSyntenyPanel"
	 */
	private SyntenyPanel syntenyPanel;


	/**
	 * Getter of the property <tt>syntenyPanel</tt>
	 * @return  Returns the syntenyPanel.
	 * @uml.property  name="syntenyPanel"
	 */
	public SyntenyPanel getSyntenyPanel() {
		return syntenyPanel;
	}


	/**
	 * Setter of the property <tt>syntenyPanel</tt>
	 * @param syntenyPanel  The syntenyPanel to set.
	 * @uml.property  name="syntenyPanel"
	 */
	public void setSyntenyPanel(SyntenyPanel syntenyPanel) {
		this.syntenyPanel = syntenyPanel;
	}


	public void updateLeftToolbarSize() {
		jsplitpane.setDividerLocation(-1);
	}
	

	public void updateLeftToolbarSize(int i) {
		
		jsplitpane.setDividerLocation(i);
	}

	public void comparedGenomesChoosed(Collection compGenomes) {
		getSyntenyPanel().updateGenomes(compGenomes);
	}

	public void referenceGenomeChoosed(Genome genome,Collection<Genome> compGenomes) {
		_newRefGenomeTask.setNewReferenceGenome(genome,compGenomes);
		getSynteviewToolbarPanel().getSelectComparisonPanel().changeGenomeList(compGenomes);
		getSynteviewToolbarPanel().getSelectReferenceGenomePanel().getSelectComparedGenomesPanel().setComparedGenomes(new ArrayList<Genome>(compGenomes));
		getSynteviewToolbarPanel().getSelectReferenceGenomePanel().getRefGenomePanel().setSelectedGenome(genome);		
		getSynteviewToolbarPanel().getSearchProteinPanel().getBlocSizeAdjustmentWidget().setNbGenomes(compGenomes.size());
		_newRefGenomeTask.setSynteny(getSyntenyPanel().getSynteny());
		_newRefGenomeTask.execute();
	}
}
