/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.crypto.Cipher;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.synteny.Comparison;
import org.synteny.Genome;
import org.synteny.Protein;
import org.synteny.Synteny;
import org.synteny.ihm.layout.SyntenyLayout;
import org.synteny.search.ProteinSearcherImpl;
import org.synteny.search.SearchListener;

public class SyntenyPanel extends JPanel implements SyntenyListener, SearchListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 8692674162128853222L;
	/**
	 * @uml.property  name="synteny"
	 * @uml.associationEnd  inverse="syntenyPanel:org.synteny.Synteny"
	 * @uml.association  name="hasSynteny"
	 */
	private Synteny synteny;

	private JScrollPane _genomes;
	private JScrollPane _syntenyScroll;
	private JScrollPane _refGenomeScroll;
	private JScrollPane _rateScroll;
	/** 
	 * @uml.property name="conservationRatePanel"
	 * @uml.associationEnd inverse="syntenyPanel:org.synteny.ihm.ConservationRatePanel"
	 */
	private ConservationRatePanel conservationRatePanel;


	/**
	 * @uml.property  name="syntenyFrame"
	 * @uml.associationEnd  inverse="syntenyPanel:org.synteny.ihm.SyntenyFrame"
	 * @uml.association  name="hasSyntenyPanel"
	 */
	private SyntenyFrame syntenyFrame;


	/**
	 * @uml.property  name="genomesPanel"
	 * @uml.associationEnd  inverse="syntenyPanel:org.synteny.ihm.GenomesPanel"
	 * @uml.association  name="hasGenomePanel"
	 */
	private GenomesPanel genomesPanel;


	/**
	 * @uml.property  name="comparisonPanel"
	 * @uml.associationEnd  multiplicity="(0 -1)" inverse="syntenyPanel:org.synteny.ihm.ComparisonPanel"
	 * @uml.association  name="hasComparisonPanel"
	 */
	private List comparisonPanel;

	private JPanel syntenyJPanel;
	private JPanel refGenomePanel;

	/**
	 * @uml.property  name="refProteomePanel"
	 * @uml.associationEnd  inverse="syntenyPanel:org.synteny.ihm.RefProteomePanel"
	 * @uml.association  name="hasRefGenomePanel"
	 */
	private RefProteomePanel refProteomePanel;



	public SyntenyPanel(Synteny s){
		super();
		synteny = s;
		s.addSyntenyListener(this);
		initialize();
	}

	public void initialize(){
		comparisonPanel = new ArrayList<ComparisonPanel>();
		setLayout(new BorderLayout());
		syntenyJPanel = new JPanel();
		syntenyJPanel.setBackground(Color.WHITE);
		syntenyJPanel.setLayout(new SyntenyLayout(syntenyJPanel));

		refGenomePanel = new JPanel();
		refGenomePanel.setLayout(new BorderLayout());
		refProteomePanel = new RefProteomePanel();
		refProteomePanel.setSyntenyPanel(this);
		_refGenomeScroll = new JScrollPane(refProteomePanel,JScrollPane.VERTICAL_SCROLLBAR_NEVER,JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		initRefGenomePanel();
		add(refGenomePanel,BorderLayout.NORTH);
		_syntenyScroll = new JScrollPane(syntenyJPanel,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		_syntenyScroll.getHorizontalScrollBar().setBlockIncrement(ProteinPanel.PROTEIN_WIDTH);
		add(_syntenyScroll,BorderLayout.CENTER);
		_syntenyScroll.getVerticalScrollBar().addAdjustmentListener(new AdjustmentListener(){
			public void adjustmentValueChanged(AdjustmentEvent e) {
				_genomes.getVerticalScrollBar().setValue(e.getValue());
			}
		});
		_syntenyScroll.getHorizontalScrollBar().addAdjustmentListener(new AdjustmentListener(){
			public void adjustmentValueChanged(AdjustmentEvent e) {
				_refGenomeScroll.getHorizontalScrollBar().setValue(e.getValue());
				if(_rateScroll!=null)
					_rateScroll.getHorizontalScrollBar().setValue(e.getValue());
			}
		});

		genomesPanel= new GenomesPanel(this,comparisonPanel);
		_genomes = new JScrollPane(getGenomesPanel(),JScrollPane.VERTICAL_SCROLLBAR_NEVER,JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		add(_genomes,BorderLayout.WEST);

		_rateScroll = new JScrollPane(getConservationRatePanel(), JScrollPane.VERTICAL_SCROLLBAR_NEVER,JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		_rateScroll.setPreferredSize(new Dimension(getConservationRatePanel().getPreferredSize().width,0));
		add(_rateScroll,BorderLayout.SOUTH);
		setOpaque(true);
		setBackground(Color.WHITE);
	}


	/**
	 * Getter of the property <tt>synteny</tt>
	 * @return  Returns the synteny.
	 * @uml.property  name="synteny"
	 */
	public Synteny getSynteny() {
		return synteny;
	}

	/**
	 * Setter of the property <tt>synteny</tt>
	 * @param synteny  The synteny to set.
	 * @uml.property  name="synteny"
	 */
	public void setSynteny(Synteny synteny) {
		this.synteny = synteny;
	}


	/**
	 * Getter of the property <tt>comparisonPanel</tt>
	 * @return  Returns the comparisonPanel.
	 * @uml.property  name="comparisonPanel"
	 */
	public Collection getComparisonPanel() {
		return comparisonPanel;
	}

	/**
	 * Setter of the property <tt>comparisonPanel</tt>
	 * @param comparisonPanel  The comparisonPanel to set.
	 * @uml.property  name="comparisonPanel"
	 */
	public void setComparisonPanel(List comparisonPanel) {
		this.comparisonPanel = comparisonPanel;
	}


	/**
	 * Getter of the property <tt>syntenyFrame</tt>
	 * @return  Returns the syntenyFrame.
	 * @uml.property  name="syntenyFrame"
	 */
	public SyntenyFrame getSyntenyFrame() {
		return syntenyFrame;
	}

	/**
	 * Setter of the property <tt>syntenyFrame</tt>
	 * @param syntenyFrame  The syntenyFrame to set.
	 * @uml.property  name="syntenyFrame"
	 */
	public void setSyntenyFrame(SyntenyFrame syntenyFrame) {
		this.syntenyFrame = syntenyFrame;
	}

	private void initRefGenomePanel(){
		refGenomePanel.removeAll();
		if(synteny.getComparison().size()>0){
			refProteomePanel.updateRefGenome(synteny.getReferenceSpecies());

			if(getSynteny().getComparison().size()>0){
				GenomeLabel rg = new GenomeLabel(getSynteny().getReferenceSpecies());
				refGenomePanel.add(rg,BorderLayout.WEST);
			}
		}
		refGenomePanel.add(_refGenomeScroll,BorderLayout.CENTER);
	}

	public void referenceSpeciesChanged() {
		initRefGenomePanel();
		comparisonPanel.clear();
		syntenyJPanel.removeAll();
		syntenyJPanel.setLayout(new SyntenyLayout(syntenyJPanel));
		syntenyJPanel.setPreferredSize(new Dimension(synteny.getReferenceSpecies().getSize()*(OrthologPanel.PROTEIN_WIDTH+OrthologPanel.PROTEIN_MARGIN),
				synteny.getComparison().size()*(ProteinPanel.PROTEIN_HEIGHT+SimpleBlocPanel.BLOC_MARGIN*2+ComparisonPanel.COMPARISON_BOTTOM_MARGIN+ComparisonPanel.COMPARISON_TOP_MARGIN)));
		for(Object o : synteny.getComparison()){
			Comparison c = (Comparison) o;
			ComparisonPanel cp = new ComparisonPanel(c);
			cp.setSyntenyPanel(this);
			comparisonPanel.add(cp);
			syntenyJPanel.add(cp);
		}
		syntenyJPanel.updateUI();

		genomesPanel.updateComparisons(comparisonPanel);
		if(getSynteny().getReferenceSpecies()!=null){
			getConservationRatePanel().computeRates();
			_rateScroll.setPreferredSize(new Dimension(getConservationRatePanel().getPreferredSize().width,60));
		}
		getSyntenyFrame().getSynteviewToolbarPanel().getSearchProteinPanel().setProteinSearcher(new ProteinSearcherImpl(synteny.getReferenceSpecies()));
		updateUI();
	}

	/**
	 * Getter of the property <tt>genomesPanel</tt>
	 * @return  Returns the genomesPanel.
	 * @uml.property  name="genomesPanel"
	 */
	public GenomesPanel getGenomesPanel() {
		return genomesPanel;
	}

	/**
	 * Setter of the property <tt>genomesPanel</tt>
	 * @param genomesPanel  The genomesPanel to set.
	 * @uml.property  name="genomesPanel"
	 */
	public void setGenomesPanel(GenomesPanel genomesPanel) {
		this.genomesPanel = genomesPanel;
	}

	public List<Genome> getVisibleComparedGenomes(){
		List<Genome> list = new ArrayList<Genome>();
		for(Object o : getComparisonPanel()){
			ComparisonPanel cp = (ComparisonPanel)o;
			if(cp.isVisible()){
				list.add(cp.getComparison().getCompGenome());
			}
		}
		return list;
	}

	public void updateGenomes(Collection<Genome> genomes){
		System.out.println("Remove ALl panels");
		syntenyJPanel.removeAll();
		System.out.println("Update GenomesPanel");
		genomesPanel.updateGenomes(genomes);
		//System.out.println("Init Ref genome panel");
		//initRefGenomePanel();
		System.out.println("Ajout des panels");
		for(Object o : getComparisonPanel()){
			ComparisonPanel cp = (ComparisonPanel)o;
			if(genomes.contains(cp.getComparison().getCompGenome())){
				syntenyJPanel.add(cp);
				cp.setVisible(true);
			}else{
				cp.setVisible(false);
			}
		}

		/*for(int i=0;i<getComponentCount();i++){
			Component jc = syntenyJPanel.getComponents()[i];
			ComparisonPanel cp = (ComparisonPanel)jc;
			cp.setVisible(genomes.contains(cp.getComparison().getCompGenome()));
		}*/

		System.out.println("Updateui");
		syntenyJPanel.updateUI();
		System.out.println("Conservation rate");
		getConservationRatePanel().computeRates();
		updateUI();
	}

	/**
	 * Getter of the property <tt>refProteomePanel</tt>
	 * @return  Returns the refProteomePanel.
	 * @uml.property  name="refProteomePanel"
	 */
	public RefProteomePanel getRefProteomePanel() {
		return refProteomePanel;
	}

	/**
	 * Setter of the property <tt>refProteomePanel</tt>
	 * @param refProteomePanel  The refProteomePanel to set.
	 * @uml.property  name="refProteomePanel"
	 */
	public void setRefProteomePanel(RefProteomePanel refProteomePanel) {
		this.refProteomePanel = refProteomePanel;
	}

	/** 
	 * Getter of the property <tt>conservationRatePanel</tt>
	 * @return  Returns the conservationRatePanel.
	 * @uml.property  name="conservationRatePanel"
	 */
	public ConservationRatePanel getConservationRatePanel() {
		if (conservationRatePanel == null) {
			conservationRatePanel = new ConservationRatePanel(this);
		}
		return conservationRatePanel;
	}

	/** 
	 * Setter of the property <tt>conservationRatePanel</tt>
	 * @param conservationRatePanel  The conservationRatePanel to set.
	 * @uml.property  name="conservationRatePanel"
	 */
	public void setConservationRatePanel(
			ConservationRatePanel conservationRatePanel) {
				this.conservationRatePanel = conservationRatePanel;
			}

	public void speciesBeingAnalysed(String speciesName, int nbGenomes) {
	}

	public void ProteinClicked(Protein p) {
		System.out.println(p.getIdentifier());
		if(!getSyntenyFrame().getSynteviewToolbarPanel().searchTabDisplayed())
			getSyntenyFrame().getSynteviewToolbarPanel().displaySearchTab();
		getSyntenyFrame().getSynteviewToolbarPanel().getSearchProteinPanel().ProteinActivated(p);
	}

	public void proteinSearchedFor(Protein protein) {
		System.out.println(protein);
		int minScroll = _syntenyScroll.getHorizontalScrollBar().getMinimum();
		int maxScroll = _syntenyScroll.getHorizontalScrollBar().getMaximum();
		int maxGenome = protein.getGenome().getSize();
		int current = protein.getGenomePosition();
		int scrolPos = (int)(current*1.0*(maxScroll-minScroll)/maxGenome);
		
		System.out.println("Scroll:\n Min: "+minScroll+"\n Max:"+maxScroll+"\n Genome: \n max:"+maxGenome+"\n current:"+current+"\n ScrollFinal:"+scrolPos);
		_syntenyScroll.getHorizontalScrollBar().setValue(scrolPos);
	}

}
