/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collection;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToolBar;

import org.exceptions.SyntenyNotInitializedException;
import org.synteny.Comparison;
import org.synteny.Genome;
import org.synteny.factory.IconFactory;
import org.synteny.ihm.preferences.SelectDisplayedGenomesPanel;
import org.synteny.ihm.preferences.SelectHistogramGenomePanel;
import org.synteny.ihm.preferences.SelectReferenceGenomePanel;
import org.synteny.ihm.preferences.Settings;
import org.synteny.ihm.preferences.SettingsDialog;
import org.synteny.saver.SyntenySaver;


public class SynteviewToolbarPanel extends JPanel implements CloseListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1089581291318749688L;

	/**
	 * @uml.property  name="syntenyFrame"
	 * @uml.associationEnd  inverse="synteviewToolbarPanel:org.synteny.ihm.SyntenyFrame"
	 * @uml.association  name="hasToolBarPanel"
	 */
	private SyntenyFrame syntenyFrame;

	private JPanel _nullPanel;
	private JPanel _currentPanel;
	private JToolBar _jtoolbar;
	private JButton _refGButton;
	private JButton _compGButton;
	private JButton _sizeDistrib;
	private JButton _searchButton;
	private JButton _paramButton;
	private JButton _aboutButton;
	private JButton _saveButton;

	/**
	 * @uml.property  name="selectReferenceGenomePanel"
	 * @uml.associationEnd  inverse="synteviewToolbarPanel:org.synteny.ihm.preferences.SelectReferenceGenomePanel"
	 * @uml.association  name="hasRefGenomePanel"
	 */
	private SelectReferenceGenomePanel selectReferenceGenomePanel;

	/**
	 * @uml.property  name="selectComparisonPanel"
	 * @uml.associationEnd  inverse="synteviewToolbarPanel:org.synteny.ihm.preferences.SelectReferenceGenomePanel"
	 * @uml.association  name="hasSelectSizeDistribComparison"
	 */
	private SelectHistogramGenomePanel selectComparisonPanel;

	/**
	 * @uml.property  name="searchProteinPanel"
	 * @uml.associationEnd  inverse="synteviewToolbarPanel:org.synteny.ihm.SearchProteinPanel"
	 * @uml.association  name="hasSearchPanel"
	 */
	private SearchProteinPanel searchProteinPanel;

	private AboutPanel aboutPanel;


	public SynteviewToolbarPanel(SyntenyFrame sf){
		super();
		syntenyFrame = sf;
		_nullPanel = new JPanel();
		_nullPanel.setPreferredSize(new Dimension(1,getHeight()));
		selectReferenceGenomePanel = new SelectReferenceGenomePanel(getSyntenyFrame().getSyntenyPanel().getSynteny().getSpeciesTree(), "Select synteny species");
		selectReferenceGenomePanel.addGenomeSelectionListener(sf);
		selectReferenceGenomePanel.addCloseListener(this);

		selectComparisonPanel = new SelectHistogramGenomePanel("Select species for histogram");
		selectComparisonPanel.addGenomeSelectionListener(new GenomeSelectionListener(){
			public void comparedGenomesChoosed(Collection compGenomes){
			}
			public void referenceGenomeChoosed(Genome genome,Collection<Genome> compGenomes){
				showBlocksSizeDistribution(genome);
			}
		});
		selectComparisonPanel.addCloseListener(this);

		searchProteinPanel = new SearchProteinPanel();
		searchProteinPanel.addSearchListener(getSyntenyFrame().getSyntenyPanel());
		searchProteinPanel.addCloseListener(this);

		aboutPanel = new AboutPanel();
		aboutPanel.addCloseListener(this);

		_jtoolbar = new JToolBar(JToolBar.VERTICAL);
		_jtoolbar.add(getRefGenomeButton());
		_jtoolbar.add(getSizeDistribButton());
		_jtoolbar.add(getSearchButton());
		_jtoolbar.add(getParamButton());
		_jtoolbar.add(getAboutButton());
		_jtoolbar.add(getSaveButton());
		_jtoolbar.setFloatable(false);
		setLayout(new BorderLayout());
		_currentPanel = aboutPanel;
		add(_jtoolbar,BorderLayout.WEST);
		add(_currentPanel,BorderLayout.CENTER);
	}


	private JButton getRefGenomeButton() {
		if(_refGButton==null){
			_refGButton = new JButton();
			_refGButton.setToolTipText("Select the reference and compared Species");
			_refGButton.setIcon(IconFactory.getInstance().getSpeciesImage());
			_refGButton.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					remove(_currentPanel);
					if(_currentPanel!=selectReferenceGenomePanel){
						add(selectReferenceGenomePanel);
						_currentPanel = selectReferenceGenomePanel;
					}else{
						_currentPanel = _nullPanel;
					}
					getSyntenyFrame().updateLeftToolbarSize();
					updateUI();
				}
			});
		}
		return _refGButton;
	}

	private JButton getSizeDistribButton(){
		if(_sizeDistrib==null){
			_sizeDistrib = new JButton();
			_sizeDistrib.setToolTipText("Display histograms of synteny blocks distribution");
			_sizeDistrib.setIcon(IconFactory.getInstance().getHistogramImage());
			_sizeDistrib.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent arg0) {
					remove(_currentPanel);
					if(_currentPanel!=selectComparisonPanel){
						add(selectComparisonPanel);
						_currentPanel = selectComparisonPanel;
					}else{
						_currentPanel = _nullPanel;
					}
					getSyntenyFrame().updateLeftToolbarSize();
					updateUI();
				}
			});
		}
		return _sizeDistrib;
	}

	private JButton getSearchButton(){
		if(_searchButton==null){
			_searchButton= new JButton();
			_searchButton.setToolTipText("Display infos & Search for genes");
			_searchButton.setIcon(IconFactory.getInstance().getSearchImage());
			_searchButton.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent arg0) {
					remove(_currentPanel);
					if(_currentPanel!=searchProteinPanel){
						add(searchProteinPanel);
						_currentPanel = searchProteinPanel;
					}else{
						_currentPanel = _nullPanel;
					}
					getSyntenyFrame().updateLeftToolbarSize();
					updateUI();
				}
			});
		}
		return _searchButton;
	}

	public JButton getParamButton(){
		if(_paramButton==null){
			_paramButton= new JButton();
			_paramButton.setToolTipText("Settings");
			_paramButton.setIcon(IconFactory.getInstance().getEditIcon());
			_paramButton.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent arg0) {
					SettingsDialog.showDialog(Settings.getInstance());
				}

			});
		}
		return _paramButton;
	}


	public JButton getAboutButton(){
		if(_aboutButton==null){
			_aboutButton= new JButton();
			_aboutButton.setToolTipText("About");
			_aboutButton.setIcon(IconFactory.getInstance().getAboutIcon());
			_aboutButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent arg0) {
					changePanel(aboutPanel);
				}
			});
		}
		return _aboutButton;
	}

	public JButton getSaveButton(){
		if(_saveButton == null){
			_saveButton = new JButton();
			_saveButton.setToolTipText("Save Synteny data as CSV");
			_saveButton.setIcon(IconFactory.getInstance().getSaveIcon());
			_saveButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					SyntenySaver saver = new SyntenySaver(getSyntenyFrame().getSyntenyPanel().getSynteny());
					try {
						String csv = saver.saveToCSV();
						JFileChooser jf = new JFileChooser();
						if(jf.showSaveDialog(SynteviewToolbarPanel.this)==JFileChooser.APPROVE_OPTION){
							File f = jf.getSelectedFile();
							try {
								PrintWriter pw= new PrintWriter(new BufferedWriter(new FileWriter(f)));
								pw.println(csv);
								pw.close();
							} catch (IOException e1) {
								e1.printStackTrace();
							}
						}
					} catch (SyntenyNotInitializedException e2) {
						JOptionPane.showMessageDialog(SynteviewToolbarPanel.this, "Synteny data has not been retrieved from the server","Search error",JOptionPane.ERROR_MESSAGE);
					}
				}
			});
		}
		return _saveButton;
	}

	/**
	 * Getter of the property <tt>syntenyFrame</tt>
	 * @return  Returns the syntenyFrame.
	 * @uml.property  name="syntenyFrame"
	 */
	public SyntenyFrame getSyntenyFrame() {
		return syntenyFrame;
	}

	/**
	 * Setter of the property <tt>syntenyFrame</tt>
	 * @param syntenyFrame  The syntenyFrame to set.
	 * @uml.property  name="syntenyFrame"
	 */
	public void setSyntenyFrame(SyntenyFrame syntenyFrame) {
		this.syntenyFrame = syntenyFrame;
	}


	/**
	 * Getter of the property <tt>selectReferenceGenomePanel</tt>
	 * @return  Returns the selectReferenceGenomePanel.
	 * @uml.property  name="selectReferenceGenomePanel"
	 */
	public SelectReferenceGenomePanel getSelectReferenceGenomePanel() {
		return selectReferenceGenomePanel;
	}

	/**
	 * Setter of the property <tt>selectReferenceGenomePanel</tt>
	 * @param selectReferenceGenomePanel  The selectReferenceGenomePanel to set.
	 * @uml.property  name="selectReferenceGenomePanel"
	 */
	public void setSelectReferenceGenomePanel(
			SelectReferenceGenomePanel selectReferenceGenomePanel) {
		this.selectReferenceGenomePanel = selectReferenceGenomePanel;
	}



	public void showBlocksSizeDistribution(Genome g){
		Comparison c = g.getCompComparison();
		SizeDistributionDialog.showDialog(c.getBlocSizeDistribution(),c.getRefGenome(),g);
	}

	/**
	 * Getter of the property <tt>selectComparisonPanel</tt>
	 * @return  Returns the selectComparisonPanel.
	 * @uml.property  name="selectComparisonPanel"
	 */
	public SelectHistogramGenomePanel getSelectComparisonPanel() {
		return selectComparisonPanel;
	}

	/**
	 * Setter of the property <tt>selectComparisonPanel</tt>
	 * @param selectComparisonPanel  The selectComparisonPanel to set.
	 * @uml.property  name="selectComparisonPanel"
	 */
	public void setSelectComparisonPanel(
			SelectHistogramGenomePanel selectComparisonPanel) {
		this.selectComparisonPanel = selectComparisonPanel;
	}

	/**
	 * Getter of the property <tt>searchProteinPanel</tt>
	 * @return  Returns the searchProteinPanel.
	 * @uml.property  name="searchProteinPanel"
	 */
	public SearchProteinPanel getSearchProteinPanel() {
		return searchProteinPanel;
	}

	/**
	 * Setter of the property <tt>searchProteinPanel</tt>
	 * @param searchProteinPanel  The searchProteinPanel to set.
	 * @uml.property  name="searchProteinPanel"
	 */
	public void setSearchProteinPanel(SearchProteinPanel searchProteinPanel) {
		this.searchProteinPanel = searchProteinPanel;
	}

	public void displaySearchTab(){
		remove(_currentPanel);
		_currentPanel = searchProteinPanel;
		add(_currentPanel);
		getSyntenyFrame().updateLeftToolbarSize();
		updateUI();
	}

	public boolean searchTabDisplayed(){
		return (_currentPanel==searchProteinPanel);
	}

	/**
	 * @uml.property   name="selectComparedGenomesPanel1"
	 * @uml.associationEnd   inverse="synteviewToolbarPanel:org.synteny.ihm.preferences.SelectComparedGenomesPanel"
	 */
	private org.synteny.ihm.preferences.SelectComparedGenomesPanel selectComparedGenomesPanel1;



	/**
	 * Getter of the property <tt>selectComparedGenomesPanel1</tt>
	 * @return  Returns the selectComparedGenomesPanel1.
	 * @uml.property  name="selectComparedGenomesPanel1"
	 */
	public org.synteny.ihm.preferences.SelectComparedGenomesPanel getSelectComparedGenomesPanel1() {
		return selectComparedGenomesPanel1;
	}

	/**
	 * Setter of the property <tt>selectComparedGenomesPanel1</tt>
	 * @param selectComparedGenomesPanel1  The selectComparedGenomesPanel1 to set.
	 * @uml.property  name="selectComparedGenomesPanel1"
	 */
	public void setSelectComparedGenomesPanel1(
			org.synteny.ihm.preferences.SelectComparedGenomesPanel selectComparedGenomesPanel1) {
		this.selectComparedGenomesPanel1 = selectComparedGenomesPanel1;
	}

	public void setListOfSpeciesToCompareForHistogram(Collection<Genome> coll){

	}

	public void changePanel(JPanel jp){
		remove(_currentPanel);
		if(_currentPanel!= jp){
			add(jp);
			_currentPanel = jp;
			getSyntenyFrame().updateLeftToolbarSize();
		}else{
			_currentPanel = _nullPanel;
			getSyntenyFrame().updateLeftToolbarSize(SynteviewToolbarPanel.this._jtoolbar.getWidth());
		}
		updateUI();
	}


	public void ComponentClosed(Closable closable) {
		remove((JPanel)closable);
		_currentPanel = _nullPanel;
		getSyntenyFrame().updateLeftToolbarSize(SynteviewToolbarPanel.this._jtoolbar.getWidth());
	}
}
