/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JFrame;
import javax.swing.JPanel;

public class WaitingCircle extends JPanel implements Runnable{

	private static final long serialVersionUID = -2528214039680482952L;
	private int HEIGHT = 100;
	private int WIDTH = 100;

	private int CIRCLEHEIGHT = (int)(HEIGHT*0.3);
	private int CIRCLEWIDTH = (int)(WIDTH*0.3);

	private Color LIGHT = Color.LIGHT_GRAY;
	private Color DARK = Color.BLACK;

	private Color _bgcolor = Color.WHITE;

	private Color [] _colors = new Color[8];

	private int _vitesse = 150;

	private int _way = WaitingCircle.CLOCKWISE;

	public static final int CLOCKWISE = 0;
	public static final int ANTICLOCKWISE = 1;
	private Thread _thread;
	private boolean _pause = false;

	/**
	 * Draw a component that rotate on himself
	 * @param width
	 * @param height
	 * @param way CLOCKWISE or ANTICLOCKWISE
	 */
	public WaitingCircle(int width, int height, int way) {
		super();
		this._way = way;
		//this.setBackgroundColor(_bgcolor);
		setOpaque(false);
		this.setSize(width, height);
		setColors(this.LIGHT, this.DARK);
		_thread = new Thread(this);
		_thread.start();
	}

	public void paint(Graphics g) {
		//g.setColor(this._bgcolor);
		//g.fillRect(0, 0, this.WIDTH + this.CIRCLEWIDTH + 1, this.HEIGHT + this.CIRCLEHEIGHT +1);
		double x=0;
		for(int i=0;i<8;i++) {
			x = i*Math.PI/4;
			g.setColor(_colors[i]);
			g.fillOval(((int)(Math.cos(x)*this.WIDTH/2)) + this.WIDTH/2, ((int)(Math.sin(x)*this.WIDTH/2)) + this.WIDTH/2, this.CIRCLEWIDTH, this.CIRCLEHEIGHT);                       }
	}

	private void changeColor() {
		if(_way == WaitingCircle.ANTICLOCKWISE) {
			Color t = _colors[0];
			for(int i=0;i<7;i++) {
				_colors[i] = _colors[i+1];
			}
			_colors[7] = t;
		} else if(_way == WaitingCircle.CLOCKWISE) {
			Color t = _colors[7];
			for(int i=7;i>0;i--) {
				_colors[i] = _colors[i-1];
			}
			_colors[0] = t;
		}
	}

	public void setPauseColor(){
		for(int i=0;i<8;i++){
			_colors[i] = this.LIGHT;
		}
	}

	public void run() {
		Thread thisThread = Thread.currentThread();
		while(thisThread == _thread) {
			try {
				Thread.sleep(_vitesse);
				if(!_pause) {
					changeColor();
					repaint();
				}
			} catch (InterruptedException e) {}
		}
	}
	/**
	 * Stop the rotation
	 */
	 public void stop() {
		this._thread = null;
	}
	public void pause() {
		setPauseColor();
		_pause = true;
		repaint();
	}
	public void restart() {
		setColors(this.LIGHT, this.DARK);
		_pause = false;
		repaint();
	}
	public Color getBackgroundcolor() {
		return _bgcolor;
	}

	public void setBackgroundColor(Color bgcolor) {
		_bgcolor = bgcolor;
		//this.setBackground(_bgcolor);
		repaint();
	}

	public int getSpeed() {
		return _vitesse;
	}

	public void setSpeed(int vitesse) {
		_vitesse = vitesse;
	}
	/**
	 * Define the color of the 8 circles
	 * @param light
	 * @param dark
	 */
	public void setColors(Color light, Color dark) {
		this.LIGHT = light;
		this.DARK = dark;
		int blue = (DARK.getBlue() - LIGHT.getBlue())/8;
		int green = (DARK.getGreen() - LIGHT.getGreen())/8;
		int red = (DARK.getRed() - LIGHT.getRed())/8;
		/*for(int i=0;i<8;i++) {
			_colors[i] = new Color(LIGHT.getRed()-red*i,LIGHT.getGreen()-green*i,LIGHT.getBlue()+blue*i);
		}*/
		for(int i=0;i<8;i++) {
			System.out.println((DARK.getRed()+red*i)+" "+(DARK.getGreen()+green*(i))+" "+(DARK.getBlue()+blue*(i)));
			_colors[i] = new Color(LIGHT.getRed()+red*i,LIGHT.getGreen()+green*(i),LIGHT.getBlue()+blue*(i));
		}
		repaint();
	}

	public void setSize(int width, int height) {
		this.WIDTH = (int)(width/1.3);
		this.HEIGHT = (int)(height/1.3);
		CIRCLEHEIGHT = (int)(HEIGHT*0.3);
		CIRCLEWIDTH = (int)(WIDTH*0.3);
		this.setPreferredSize(new Dimension(this.WIDTH + CIRCLEWIDTH, this.HEIGHT + CIRCLEHEIGHT));
		this.repaint();
	}
	public static void main(String [] argv) {
		JFrame f = new JFrame("W a i t i n g   c i r c l e");
		WaitingCircle wc = new WaitingCircle(20,20, WaitingCircle.CLOCKWISE);
		wc.setSpeed(150);
		f.add(wc);
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.pack();
		f.setVisible(true);
	}
}
