/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm.preferences;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.DefaultListModel;
import javax.swing.JList;

import org.synteny.Genome;
import org.synteny.SpeciesTree;

public class CompGenomesPanel extends JList implements KeyListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 5297195131857689161L;
	private DefaultListModel _listModel ;

	/**
	 * @uml.property   name="refGenomeCellRenderer"
	 * @uml.associationEnd   inverse="compGenomesPanel:org.synteny.ihm.preferences.RefGenomeCellRenderer"
	 * @uml.association   name="hasCellRenderer"
	 */
	private RefGenomeCellRenderer refGenomeCellRenderer;

	private Collection<SpeciesListListener> _listListeners;
	
	public CompGenomesPanel(SpeciesTree sptree){
		super();
		_listModel = new DefaultListModel();
		_listListeners = new ArrayList<SpeciesListListener>();
		setModel(_listModel);
		refGenomeCellRenderer = new RefGenomeCellRenderer();
		setCellRenderer(refGenomeCellRenderer);
		setDragEnabled(true);
		setTransferHandler(new ListGenomeTransfertHandler(sptree));
		addKeyListener(this);
	}

	/** 
	 * Getter of the property <tt>refGenomeCellRenderer</tt>
	 * @return  Returns the refGenomeCellRenderer.
	 * @uml.property  name="refGenomeCellRenderer"
	 */
	public RefGenomeCellRenderer getRefGenomeCellRenderer() {
		return refGenomeCellRenderer;
	}
	/** 
	 * Setter of the property <tt>refGenomeCellRenderer</tt>
	 * @param refGenomeCellRenderer  The refGenomeCellRenderer to set.
	 * @uml.property  name="refGenomeCellRenderer"
	 */
	public void setRefGenomeCellRenderer(
			RefGenomeCellRenderer refGenomeCellRenderer) {
		this.refGenomeCellRenderer = refGenomeCellRenderer;
	}

	public Collection<Genome> getGenomes(){
		Collection<Genome> genomes = new ArrayList<Genome>();
		for(int i=0;i<getModel().getSize();i++){
			genomes.add((Genome)(getModel().getElementAt(i)));
		}
		return genomes;
	}
	
	public void keyPressed(KeyEvent arg0) {
		System.out.println(arg0.getKeyCode()+" "+KeyEvent.VK_DELETE);
		if(arg0.getKeyCode()== KeyEvent.VK_DELETE){
			Object[] gentab = getSelectedValues();
			for(int i=0;i<gentab.length;i++){
				((DefaultListModel)(getModel())).removeElement(gentab[i]);
			}
		}
		
		for(Object o: getSpeciesListListener()){
			SpeciesListListener sll = (SpeciesListListener)o;
			sll.listChanged(getModel());
		}
	}

	public void keyReleased(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void keyTyped(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	
	public void addSpeciesListListener(SpeciesListListener sll){
		_listListeners.add(sll);
	}
	
	public Collection<SpeciesListListener> getSpeciesListListener(){
		return _listListeners;
	}
	
	public void addGenome(Genome g){
		((DefaultListModel)(getModel())).addElement(g);
	}
}
