/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm.preferences;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;


public class ConnectionSettingsPanel extends JPanel implements ActionListener, Saveable{ 
	/**
	 * 
	 */
	private static final long serialVersionUID = 8535743726578856660L;

	private String[] _layerStrings= {"Database","Web Service"};
	
	private JLayeredPane _layeredPane;
	private JComboBox _chooseConnection;

	/**
	 * @uml.property  name="httpConnectionSettingsPanel"
	 * @uml.associationEnd  inverse="connectionSettingsPanel:org.synteny.ihm.preferences.HttpConnectionSettingsPanel"
	 */
	private HttpConnectionSettingsPanel httpConnectionSettingsPanel;
	/**
	 * @uml.property  name="databaseConnectionSettingsPanel"
	 * @uml.associationEnd  inverse="connectionSettingsPanel:org.synteny.ihm.preferences.DatabaseConnectionSettingsPanel"
	 */
	private DatabaseConnectionSettingsPanel databaseConnectionSettingsPanel;

	

	public ConnectionSettingsPanel(Settings settings){
		super();
		setName("Connection Settings");
		setLayout(new BorderLayout());
		
		this.settings = settings; 
		JLabel jl = new JLabel("Choose Connection type");
		_chooseConnection = new JComboBox(_layerStrings);
		_chooseConnection.addActionListener(this);
		
		JPanel jp = new JPanel();
		jp.setLayout(new BorderLayout());
		jp.add(jl,BorderLayout.WEST);
		jp.add(_chooseConnection,BorderLayout.CENTER);
		
		add(jp,BorderLayout.NORTH);
		
		System.out.println(settings.getConnectionMode());
		if(settings.getConnectionMode().equals("db")){
			add(getDatabaseConnectionSettingsPanel(),BorderLayout.CENTER);
			_chooseConnection.setSelectedIndex(0);
		}else{
			add(getHttpConnectionSettingsPanel(),BorderLayout.CENTER);
			_chooseConnection.setSelectedIndex(1);
		}
	}


	/**
	 * Getter of the property <tt>databaseConnectionSettingsPanel</tt>
	 * @return  Returns the databaseConnectionSettingsPanel.
	 * @uml.property  name="databaseConnectionSettingsPanel"
	 */
	public DatabaseConnectionSettingsPanel getDatabaseConnectionSettingsPanel() {
		if (databaseConnectionSettingsPanel == null) {
			databaseConnectionSettingsPanel = new DatabaseConnectionSettingsPanel(
					getSettings());
		}
		return databaseConnectionSettingsPanel;
	}

	/**
	 * Setter of the property <tt>databaseConnectionSettingsPanel</tt>
	 * @param databaseConnectionSettingsPanel  The databaseConnectionSettingsPanel to set.
	 * @uml.property  name="databaseConnectionSettingsPanel"
	 */
	public void setDatabaseConnectionSettingsPanel(
			DatabaseConnectionSettingsPanel databaseConnectionSettingsPanel) {
				this.databaseConnectionSettingsPanel = databaseConnectionSettingsPanel;
			}


	/**
	 * Getter of the property <tt>httpConnectionSettingsPanel</tt>
	 * @return  Returns the httpConnectionSettingsPanel.
	 * @uml.property  name="httpConnectionSettingsPanel"
	 */
	public HttpConnectionSettingsPanel getHttpConnectionSettingsPanel() {
		if (httpConnectionSettingsPanel == null) {
			httpConnectionSettingsPanel = new HttpConnectionSettingsPanel(
					getSettings());
		}
		return httpConnectionSettingsPanel;
	}

	/**
	 * Setter of the property <tt>httpConnectionSettingsPanel</tt>
	 * @param httpConnectionSettingsPanel  The httpConnectionSettingsPanel to set.
	 * @uml.property  name="httpConnectionSettingsPanel"
	 */
	public void setHttpConnectionSettingsPanel(
			HttpConnectionSettingsPanel httpConnectionSettingsPanel) {
				this.httpConnectionSettingsPanel = httpConnectionSettingsPanel;
			}

	/* Quand la combobox est activée */
	public void actionPerformed(ActionEvent e) {
		if(_chooseConnection.getSelectedIndex()==0){
			remove(getHttpConnectionSettingsPanel());
			add(getDatabaseConnectionSettingsPanel(),BorderLayout.CENTER);
			getDatabaseConnectionSettingsPanel().repaint();
		}else{
			remove(getDatabaseConnectionSettingsPanel());
			add(getHttpConnectionSettingsPanel(),BorderLayout.CENTER);
			getHttpConnectionSettingsPanel().repaint();
		}
		updateUI();
	}

	/**
	 * @uml.property  name="settings"
	 * @uml.associationEnd  inverse="connectionSettingsPanel:org.synteny.ihm.preferences.Settings"
	 */
	private Settings settings;


	/**
	 * Getter of the property <tt>settings</tt>
	 * @return  Returns the settings.
	 * @uml.property  name="settings"
	 */
	public Settings getSettings() {
		return settings;
	}
	
	public void save(){
		if(_chooseConnection.getSelectedIndex()==0){
			getSettings().setConnectionMode("db");
		}else{
			getSettings().setConnectionMode("http");
		}
		getDatabaseConnectionSettingsPanel().save();
		getHttpConnectionSettingsPanel().save();
	}


	/**
	 * Setter of the property <tt>settings</tt>
	 * @param settings  The settings to set.
	 * @uml.property  name="settings"
	 */
	public void setSettings(Settings settings) {
		this.settings = settings;
	}
}
