/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm.preferences;

import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;


public class DatabaseConnectionSettingsPanel extends JPanel implements Saveable {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 3465654432280324073L;
	/**
	 * @uml.property   name="querySettingsPanel"
	 * @uml.associationEnd   inverse="databaseConnectionSettingsPanel:org.synteny.ihm.preferences.QuerySettingsPanel"
	 */
	private QuerySettingsPanel querySettingsPanel;
	/**
	 * @uml.property  name="loginSettingsPanel"
	 * @uml.associationEnd  inverse="databaseConnectionSettingsPanel:org.synteny.ihm.preferences.LoginSettingsPanel"
	 */
	private LoginSettingsPanel loginSettingsPanel;
	/**
	 * @uml.property  name="settings"
	 * @uml.associationEnd  inverse="databaseConnectionSettingsPanel:org.synteny.ihm.preferences.Settings"
	 */
	private Settings settings;
	
	
	public DatabaseConnectionSettingsPanel(Settings settings){
		super();
		this.settings = settings;
		setLayout(new BorderLayout());
		JTabbedPane jt = new JTabbedPane();
		jt.add(getLoginSettingsPanel());
		jt.add(getQuerySettingsPanel());
		add(jt,BorderLayout.CENTER);
	}

	/**
	 * Getter of the property <tt>querySettingsPanel</tt>
	 * @return  Returns the querySettingsPanel.
	 * @uml.property  name="querySettingsPanel"
	 */
	public QuerySettingsPanel getQuerySettingsPanel() {
		if (querySettingsPanel == null) {
			querySettingsPanel = new QuerySettingsPanel(getSettings());
		}
	
		return querySettingsPanel;
	}

	/**
	 * Setter of the property <tt>querySettingsPanel</tt>
	 * @param querySettingsPanel  The querySettingsPanel to set.
	 * @uml.property  name="querySettingsPanel"
	 */
	public void setQuerySettingsPanel(QuerySettingsPanel querySettingsPanel) {
		this.querySettingsPanel = querySettingsPanel;
	}

	/**
	 * Getter of the property <tt>loginSettingsPanel</tt>
	 * @return  Returns the loginSettingsPanel.
	 * @uml.property  name="loginSettingsPanel"
	 */
	public LoginSettingsPanel getLoginSettingsPanel() {
		if (loginSettingsPanel == null) {
			loginSettingsPanel = new LoginSettingsPanel(getSettings());
		}
		return loginSettingsPanel;
	}

	/**
	 * Setter of the property <tt>loginSettingsPanel</tt>
	 * @param loginSettingsPanel  The loginSettingsPanel to set.
	 * @uml.property  name="loginSettingsPanel"
	 */
	public void setLoginSettingsPanel(LoginSettingsPanel loginSettingsPanel) {
		this.loginSettingsPanel = loginSettingsPanel;
	}



	/**
	 * Getter of the property <tt>settings</tt>
	 * @return  Returns the settings.
	 * @uml.property  name="settings"
	 */
	public Settings getSettings() {
		return settings;
	}

	/**
	 * Setter of the property <tt>settings</tt>
	 * @param settings  The settings to set.
	 * @uml.property  name="settings"
	 */
	public void setSettings(Settings settings) {
		this.settings = settings;
	}
	
	public void save(){
		getLoginSettingsPanel().save();
		getQuerySettingsPanel().save();
	}
}
