/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm.preferences;

import java.awt.BorderLayout;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;


public class HttpConnectionSettingsPanel extends JPanel implements Saveable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 9184210988242786444L;
	/**
	 * @uml.property  name="settings"
	 * @uml.associationEnd  inverse="httpConnectionSettingsPanel:org.synteny.ihm.preferences.Settings"
	 */
	private Settings settings;

	
	private JTextField _syntenyURL;  
	private JTextField _speciesTreeURL;
	private JTextField _genomeURL;
	private JTextField _proteinInfoURL;

	public HttpConnectionSettingsPanel(Settings settings){
		super();
		setLayout(new BorderLayout());

		this.settings = settings;
		_syntenyURL = new JTextField();
		_syntenyURL.setText(settings.getSyntenyURL());
		_speciesTreeURL = new JTextField();;
		_speciesTreeURL.setText(settings.getSpeciesTreeURL());
		_genomeURL = new JTextField();;
		_genomeURL.setText(settings.getGenomeURL());
		_proteinInfoURL = new JTextField();;
		_proteinInfoURL.setText(settings.getProteinInfosURL());
		
		JPanel jp = new JPanel();
		jp.setLayout(new BoxLayout(jp,BoxLayout.Y_AXIS));
		jp.add(new JLabel("Synteny URL"));
		jp.add(_syntenyURL);
		
		jp.add(new JLabel("Species tree URL"));
		jp.add(_speciesTreeURL);
		
		jp.add(new JLabel("Genomes URL"));
		jp.add(_genomeURL);
		
		jp.add(new JLabel("Protein informations URL"));
		jp.add(_proteinInfoURL);

		add(new JScrollPane(jp),BorderLayout.CENTER);
	}

	public void save() {
		getSettings().setSyntenyURL(_syntenyURL.getText());
		getSettings().setSpeciesTreeeURL(_speciesTreeURL.getText());
		getSettings().setGenomeURL(_genomeURL.getText());
		getSettings().setProteinInfosURL(_proteinInfoURL.getText());
	}

	/**
	 * Getter of the property <tt>settings</tt>
	 * @return  Returns the settings.
	 * @uml.property  name="settings"
	 */
	public Settings getSettings() {
		return settings;
	}

	/**
	 * Setter of the property <tt>settings</tt>
	 * @param settings  The settings to set.
	 * @uml.property  name="settings"
	 */
	public void setSettings(Settings settings) {
		this.settings = settings;
	}
}
