/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm.preferences;

import java.util.ArrayList;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.ListModel;

import org.synteny.Genome;
import org.synteny.SpeciesTree;

public class ListGenomeTransfertHandler extends GenomeTranferHandler {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8806140726600326642L;
	private int[] indices = null;
	private int addIndex = -1; //Location where items were added
	private int addCount = 0;  //Number of items added.
	private SpeciesTree _spTree;

	public ListGenomeTransfertHandler(SpeciesTree spt){
		super();
		_spTree = spt;
	}

	//Bundle up the selected items in the list
	//as a single string, for export.
	protected String exportGenome(JComponent c) {
		JList list = (JList)c;
		indices = list.getSelectedIndices();
		Object[] values = list.getSelectedValues();
		StringBuffer buff = new StringBuffer();

		for (int i = 0; i < values.length && i<=100; i++) {
			Object val = values[i];
			Genome g = (Genome)val;
			buff.append(g == null ? "" : g.getSpeciesAbbrv());
			if (i != values.length - 1) {
				buff.append("\n");
			}
		}
		return buff.toString();
	}

	//Take the incoming string and wherever there is a
	//newline, break it into a separate item in the list.
	protected void importGenome(JComponent c, String genomeAbbrv) {
		JList target = (JList)c;
		DefaultListModel listModel = (DefaultListModel)target.getModel();
		int index = target.getSelectedIndex();
		//Prevent the user from dropping data back on itself.
		//For example, if the user is moving items #4,#5,#6 and #7 and
		//attempts to insert the items after item #5, this would
		//be problematic when removing the original items.
		//So this is not allowed.
		if (indices != null && index >= indices[0] - 1 &&
				index <= indices[indices.length - 1]) {
			indices = null;
			return;
		}

		int max = listModel.getSize();
		if (index < 0) {
			index = max;
		} else {
			index++;
			if (index > max) {
				index = max;
			}
		}
		addIndex = index;
		String[] values = genomeAbbrv.split("\n");
		addCount = values.length;
		for (int i = 0; i < values.length; i++) {
			if(listModel.getSize()<100){
				listModel.add(index++, _spTree.getGenome(values[i]));
			}
		}
	}

	//If the remove argument is true, the drop has been
	//successful and it's time to remove the selected items 
	//from the list. If the remove argument is false, it
	//was a Copy operation and the original list is left
	//intact.
	protected void cleanup(JComponent c, boolean remove) {
		if (remove && indices != null) {
			JList source = (JList)c;
			ListModel model  = (ListModel)source.getModel();
			List<Object> genomes = new ArrayList<Object>();
			for(int i=0;i<model.getSize();i++){
				genomes.add(model.getElementAt(i));
			}
			//If we are moving items around in the same list, we
			//need to adjust the indices accordingly, since those
			//after the insertion point have moved.
			if (addCount > 0) {
				for (int i = 0; i < indices.length && i<100; i++) {
					if (indices[i] > addIndex) {
						indices[i] += addCount;
					}
				}
			}
			for (int i = indices.length - 1; i >= 0; i--) {
				if(i>100) i=100;
				genomes.remove(indices[i]);
			}
			source.setListData(genomes.toArray());
		}
		indices = null;
		addCount = 0;
		addIndex = -1;
		
		((RefGenomePanel)c).transferFinished();
	}
}
