/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm.preferences;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;


public class QuerySettingsPanel extends JPanel implements Saveable {

	/**
	 * 
	 */
	private static final long serialVersionUID = -8606266381502031074L;
	JTextArea _comparisonQuery;
	JTextArea _speciesTreeQuery;
	JTextArea _genomeQuery;
	JTextArea _proteinQuery;
	
	public QuerySettingsPanel(Settings settings){
		super();
		setName("Queries on database");
		setLayout(new BorderLayout());

		this.settings = settings; 
		
		JPanel jp = new JPanel();
		jp.setLayout(new BoxLayout(jp,BoxLayout.Y_AXIS));
	
		JLabel comparison = new JLabel("Synteny retrieval Query");
		_comparisonQuery = createTextArea();
		_comparisonQuery.setText(getSettings().getSyntenyQuery());
		jp.add(comparison);
		jp.add(new JScrollPane(_comparisonQuery));

		JLabel speciestree = new JLabel("Species tree retrieval Query");
		_speciesTreeQuery = createTextArea();
		_speciesTreeQuery.setText(getSettings().getSpeciesTreeQuery());
		jp.add(speciestree);
		jp.add(new JScrollPane(_speciesTreeQuery));
		
		JLabel genome = new JLabel("Genome retrieval Query");
		_genomeQuery = createTextArea();
		_genomeQuery.setText(getSettings().getGenomeQuery());
		jp.add(genome);
		jp.add(new JScrollPane(_genomeQuery));
		
		JLabel protein = new JLabel("Protein infos retrieval Query");
		_proteinQuery = createTextArea();
		_proteinQuery.setText(getSettings().getProteinInfosQuery());
		jp.add(protein);
		jp.add(new JScrollPane(_proteinQuery));
		
		JScrollPane jsp = new JScrollPane(jp);
		jsp.setPreferredSize(new Dimension(450,100));
		add(jsp,BorderLayout.CENTER);
		
	}
	
	private JTextArea createTextArea(){
		JTextArea jt = new JTextArea();
		jt.setLineWrap(true);
		jt.setAutoscrolls(true);
		jt.setColumns(30);
		jt.setRows(10);
		return jt;
	}

	/**
	 * @uml.property  name="settings"
	 * @uml.associationEnd  inverse="querySettingsPanel:org.synteny.ihm.preferences.Settings"
	 */
	private Settings settings;

	/**
	 * Getter of the property <tt>settings</tt>
	 * @return  Returns the settings.
	 * @uml.property  name="settings"
	 */
	public Settings getSettings() {
		return settings;
	}

	/**
	 * Setter of the property <tt>settings</tt>
	 * @param settings  The settings to set.
	 * @uml.property  name="settings"
	 */
	public void setSettings(Settings settings) {
		this.settings = settings;
	}

	public void save() {
		getSettings().setSyntenyQuery(_comparisonQuery.getText());
		getSettings().setSpeciesTreeQuery(_speciesTreeQuery.getText());
		getSettings().setGenomeQuery(_genomeQuery.getText());
		getSettings().setProteinInfosQuery(_proteinQuery.getText());
	}
	
}
