/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm.preferences;


import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import javax.swing.DefaultListModel;
//import javax.swing.DropMode;
import javax.swing.JList;

import org.synteny.Genome;
import org.synteny.SpeciesTree;

public class RefGenomePanel extends JList implements SpeciesTreeListener, MouseListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 2893182072089054657L;
	/** 
	 * @uml.property name="refGenomeCellRenderer"
	 * @uml.associationEnd inverse="refGenomePanel:org.synteny.ihm.preferences.RefGenomeCellRenderer"
	 * @uml.association name="hasCellRenderer"
	 */
	private RefGenomeCellRenderer refGenomeCellRenderer;

	/**
	 * @uml.property  name="speciesListListener"
	 * @uml.associationEnd  multiplicity="(0 -1)" inverse="refGenomePanel:org.synteny.ihm.preferences.SpeciesListListener"
	 */
	private Collection<SpeciesListListener> speciesListListener;

	private DefaultListModel _listModel;

	private Collection<Genome> _genomesAuthorized;
	/**
	 */
	public RefGenomePanel(SpeciesTree spt){
		super();
		_listModel = new DefaultListModel();
		addMouseListener(this);
		setModel(_listModel);
		speciesListListener = new ArrayList<SpeciesListListener>();
		refGenomeCellRenderer = new RefGenomeCellRenderer();
		_genomesAuthorized = new ArrayList<Genome>();
		setCellRenderer(refGenomeCellRenderer);
		setDragEnabled(true);
		setTransferHandler(new ListGenomeTransfertHandler(spt));
	}

	public void genomeAdded(SpeciesTreeEvent e, Genome g) {
		/*if(e.getType()==SpeciesTreeEvent.double_click){
			_listModel.removeAllElements();
			if(_genomesAuthorized.size()==0 || _genomesAuthorized.contains(g)){
				_listModel.addElement(g);
			}
		}*/
	}

	public void genomesAdded(SpeciesTreeEvent e, Collection<Genome> c) {
		if(e.getType()==SpeciesTreeEvent.simple_click){
			List<Genome> l = new ArrayList<Genome>(c);
			Collections.sort(l);
			_listModel.removeAllElements();
			for(Genome g : l){
				System.out.println(g);
				if(_genomesAuthorized.size()==0 || _genomesAuthorized.contains(g)){
					_listModel.addElement(g);
				}
			}
		}
	}

	public void setSelectedGenome(Genome g) {
		_listModel.removeAllElements();
		if(_genomesAuthorized.size()==0 || _genomesAuthorized.contains(g)){
			_listModel.addElement(g);
			setSelectedIndex(0);
		}
	}

	public void mouseClicked(MouseEvent e) {
		if (e.getClickCount() == 2) {
			Genome g = (Genome)(getSelectedValue());
			System.out.println("Double clicked on Item " + g);
			for(SpeciesListListener sll : speciesListListener){
				sll.genomeDoubleClicked(g);
			}
		}
	}

	public void transferFinished() {
		for(SpeciesListListener sll : speciesListListener){
			sll.genomesListTransfered();
		}
	}


	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	/** 
	 * Getter of the property <tt>refGenomeCellRenderer</tt>
	 * @return  Returns the refGenomeCellRenderer.
	 * @uml.property  name="refGenomeCellRenderer"
	 */
	public RefGenomeCellRenderer getRefGenomeCellRenderer() {
		return refGenomeCellRenderer;
	}

	/** 
	 * Setter of the property <tt>refGenomeCellRenderer</tt>
	 * @param refGenomeCellRenderer  The refGenomeCellRenderer to set.
	 * @uml.property  name="refGenomeCellRenderer"
	 */
	public void setRefGenomeCellRenderer(
			RefGenomeCellRenderer refGenomeCellRenderer) {
		this.refGenomeCellRenderer = refGenomeCellRenderer;
	}

	/**
	 * Getter of the property <tt>speciesListListener</tt>
	 * @return  Returns the speciesListListener.
	 * @uml.property  name="speciesListListener"
	 */
	public Collection getSpeciesListListener() {
		return speciesListListener;
	}

	/**
	 * Setter of the property <tt>speciesListListener</tt>
	 * @param speciesListListener  The speciesListListener to set.
	 * @uml.property  name="speciesListListener"
	 */
	public void setSpeciesListListener(Collection speciesListListener) {
		this.speciesListListener = speciesListListener;
	}

	public void addSpeciesListListener(SpeciesListListener sll){
		speciesListListener.add(sll);
	}

	public void setGenomesAuthorized(Collection<Genome> authorized){
		_genomesAuthorized = authorized;
	}

	public void removeGenome(Genome g){
		ArrayList<Genome> v = new ArrayList<Genome>();
		for(int i=0;i<getModel().getSize();i++){
			if(!getModel().getElementAt(i).equals(g)){
				v.add((Genome)(getModel().getElementAt(i)));
			}
		}
		setListData(v.toArray());
	}
}
