/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm.preferences;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ListModel;
import javax.swing.SwingConstants;

import org.synteny.Genome;
import org.synteny.SpeciesTree;
import org.synteny.ihm.Closable;
import org.synteny.ihm.CloseListener;
import org.synteny.ihm.GenomeSelectionListener;

public class SelectComparedGenomesPanel extends JPanel implements Closable, ActionListener, SpeciesTreeListener, SpeciesListListener {

		/**
		 * 
		 */
		private static final long serialVersionUID = 231144498495203430L;

		/**
		 * @uml.property   name="speciesTreeView"
		 * @uml.associationEnd   inverse="selectComparedGenomesPanel:org.synteny.ihm.preferences.SpeciesTreeView"
		 * @uml.association   name="hasSpeciesTreeView"
		 */
		private SpeciesTreeView speciesTreeView;

		/**
		 * @uml.property   name="compGenomesPanel"
		 * @uml.associationEnd   inverse="selectComparedGenomesPanel:org.synteny.ihm.preferences.CompGenomesPanel"
		 * @uml.association   name="hasCompGenomesPanel"
		 */
		private CompGenomesPanel compGenomesPanel;


		/**
		 * @uml.property   name="genomeSelectionListener"
		 * @uml.associationEnd   multiplicity="(0 -1)" inverse="selectComparedGenomesPanel:org.synteny.ihm.GenomeSelectionListener"
		 * @uml.association   name="hasSelectionListener"
		 */
		private Collection genomeSelectionListener;

		private Collection<CloseListener> closelisteners;

		/**
		 * @uml.property  name="currentSelectionGenomePanel"
		 * @uml.associationEnd  inverse="selectComparedGenomesPanel:org.synteny.ihm.preferences.RefGenomePanel"
		 * @uml.association  name="hasCurrentSelectionPanel"
		 */
		private RefGenomePanel currentSelectionGenomePanel;

		private JPanel butPanel;
		
		private SpeciesTree _speciesTree;
		
		private JLabel comparedPanelTitle;
		
		/**
		 */
		public SelectComparedGenomesPanel(SpeciesTree sptree, String paneltitle,boolean titlepane){
			super();
			genomeSelectionListener = new ArrayList<GenomeSelectionListener>();
			setLayout(new BorderLayout());
			closelisteners = new ArrayList<CloseListener>();
			JPanel jp = new JPanel();
			jp.setLayout(new GridLayout(1,2));
			JSplitPane jp2 =new JSplitPane(JSplitPane.VERTICAL_SPLIT);
			JPanel jp3 = new JPanel();
			jp3.setLayout(new BorderLayout());
			speciesTreeView = new SpeciesTreeView(sptree);
			compGenomesPanel = new CompGenomesPanel(sptree);
			compGenomesPanel.addSpeciesListListener(this);
			currentSelectionGenomePanel = new RefGenomePanel(sptree);
			currentSelectionGenomePanel.addSpeciesListListener(this);
			
			_speciesTree = sptree;
			
			//speciesTreeView.addSpeciesTreeListener(compGenomesPanel);
			speciesTreeView.addSpeciesTreeListener(this);
			
			jp2.setTopComponent(new JScrollPane(getSpeciesTreeView()));
			jp2.getTopComponent().setPreferredSize(new Dimension(20,200));
			JScrollPane current = new JScrollPane(getCurrentSelectionGenomePanel());
			JPanel jpjp = new JPanel();
			jpjp.setLayout(new BorderLayout());
			jpjp.add(current, BorderLayout.CENTER);
			//current.setPreferredSize(new Dimension(20,150));
			JLabel jljl2 = new JLabel("Species in the current taxon");
			jljl2.setFont(new Font("Arial",Font.BOLD,13));
			jljl2.setHorizontalAlignment(SwingConstants.CENTER);
			jpjp.add(jljl2,BorderLayout.NORTH);
			jp2.setBottomComponent(jpjp);
			jp.add(jp2);
			
			
			jp3.add(new JScrollPane(getCompGenomesPanel()),BorderLayout.CENTER);
			comparedPanelTitle = new JLabel("Currently selected species (0)");
			comparedPanelTitle.setFont(new Font("Arial",Font.BOLD,13));
			comparedPanelTitle.setHorizontalAlignment(SwingConstants.CENTER);
			jp3.add(comparedPanelTitle,BorderLayout.NORTH);
			jp3.add(getButPanel(),BorderLayout.SOUTH);
			jp.add(jp3);
			add(jp,BorderLayout.CENTER);
			if(titlepane){
				TabLabel title = new TabLabel(paneltitle);
				title.getCloseButton().addActionListener(this);
				add(title,BorderLayout.NORTH);
			}
		}

		
		private JPanel getButPanel(){
			if(butPanel == null){
				butPanel = new JPanel();
				butPanel.setLayout(new FlowLayout());
				JButton clearAll = new JButton("delete selected");
				clearAll.addActionListener(new ActionListener(){

					public void actionPerformed(ActionEvent e) {
						Object[] gentab = getCompGenomesPanel().getSelectedValues();
						for(int i=0;i<gentab.length;i++){
							((DefaultListModel)(getCompGenomesPanel().getModel())).removeElement(gentab[i]);
						}
						informComparedGenomesListeners();
						speciesTreeView.informListeners();
						updateCompPanelTitle();
					}
				});
				butPanel.add(clearAll);
			}
			return butPanel;
		}

		void informComparedGenomesListeners() {
			Collection<Genome> gen = new ArrayList<Genome>();
			for(int i=0;i<getCompGenomesPanel().getModel().getSize();i++){
				gen.add((Genome)(getCompGenomesPanel().getModel().getElementAt(i)));
			}

			for(Object o : genomeSelectionListener){
				GenomeSelectionListener gl = (GenomeSelectionListener)o;
				gl.comparedGenomesChoosed(gen);
			}
		}
		
		/**
		 * Getter of the property <tt>speciesTreeView</tt>
		 * @return  Returns the speciesTreeView.
		 * @uml.property  name="speciesTreeView"
		 */
		public SpeciesTreeView getSpeciesTreeView() {
			return speciesTreeView;
		}

		/**
		 * Setter of the property <tt>speciesTreeView</tt>
		 * @param speciesTreeView  The speciesTreeView to set.
		 * @uml.property  name="speciesTreeView"
		 */
		public void setSpeciesTreeView(SpeciesTreeView speciesTreeView) {
			this.speciesTreeView = speciesTreeView;
		}


		/** 
		 * Getter of the property <tt>compGenomesPanel</tt>
		 * @return  Returns the compGenomesPanel.
		 * @uml.property  name="compGenomesPanel"
		 */
		public CompGenomesPanel getCompGenomesPanel() {
			return compGenomesPanel;
		}

		/** 
		 * Setter of the property <tt>compGenomesPanel</tt>
		 * @param compGenomesPanel  The compGenomesPanel to set.
		 * @uml.property  name="compGenomesPanel"
		 */
		public void setCompGenomesPanel(CompGenomesPanel compGenomesPanel) {
			this.compGenomesPanel = compGenomesPanel;
		}

		public void addGenomeSelectionListener(GenomeSelectionListener gsl){
			genomeSelectionListener.add(gsl);
		}

		/** 
		 * Getter of the property <tt>genomeSelectionListener</tt>
		 * @return  Returns the genomeSelectionListener.
		 * @uml.property  name="genomeSelectionListener"
		 */
		public Collection getGenomeSelectionListener() {
			return genomeSelectionListener;
		}

		/** 
		 * Setter of the property <tt>genomeSelectionListener</tt>
		 * @param genomeSelectionListener  The genomeSelectionListener to set.
		 * @uml.property  name="genomeSelectionListener"
		 */
		public void setGenomeSelectionListener(Collection genomeSelectionListener) {
			this.genomeSelectionListener = genomeSelectionListener;
		}

		public void close() {

		}

		public void addCloseListener(CloseListener closelistener) {
			closelisteners.add(closelistener);
		}

		public void actionPerformed(ActionEvent arg0) {
			for(CloseListener cl : closelisteners){
				cl.ComponentClosed(this);
			}
		}

		/**
		 * Getter of the property <tt>currentSelectionGenomePanel</tt>
		 * @return  Returns the currentSelectionGenomePanel.
		 * @uml.property  name="currentSelectionGenomePanel"
		 */
		public RefGenomePanel getCurrentSelectionGenomePanel() {
			return currentSelectionGenomePanel;
		}

		/**
		 * Setter of the property <tt>currentSelectionGenomePanel</tt>
		 * @param currentSelectionGenomePanel  The currentSelectionGenomePanel to set.
		 * @uml.property  name="currentSelectionGenomePanel"
		 */
		public void setCurrentSelectionGenomePanel(
				RefGenomePanel currentSelectionGenomePanel) {
					this.currentSelectionGenomePanel = currentSelectionGenomePanel;
				}

		public void genomeDoubleClicked(Genome genome) {
			currentSelectionGenomePanel.removeGenome(genome);
			compGenomesPanel.addGenome(genome);
			System.out.println(genome);
			updateCompPanelTitle();
		}

		public void GenomeSimpleClicked(Genome genome) {
		}

		public void genomeAdded(SpeciesTreeEvent e, Genome g) {
			// TODO Auto-generated method stub

		}

		public void genomesAdded(SpeciesTreeEvent e, Collection<Genome> c) {
			if(e.getType()==SpeciesTreeEvent.simple_click){
				List<Genome> l = new ArrayList<Genome>(c);
				Vector<Genome> v = new Vector<Genome>();
				HashSet<Genome> h = new HashSet<Genome>();
				ListModel dlmComp = ((ListModel)(compGenomesPanel).getModel());
				Collections.sort(l);
				System.out.println("A Ajouter: "+l.size());
				System.out.println("Comp : "+dlmComp.getSize());
				for(int i=0;i<dlmComp.getSize();i++){
					h.add((Genome)dlmComp.getElementAt(i));
				}
				
				for(Genome g : l){
					//System.out.println(g);
					if(!h.contains(g)){
						v.add(g);
					}
				}
				currentSelectionGenomePanel.setListData(v.toArray());
			}
		}

		public void setComparedGenomes(List<Genome> list){
			DefaultListModel dlmComp = ((DefaultListModel)(compGenomesPanel).getModel());
			dlmComp.removeAllElements();
			for(Genome g : list){
				dlmComp.addElement(g);
			}
			informComparedGenomesListeners();
			speciesTreeView.informListeners();
			updateCompPanelTitle();
		}


		public void genomesListTransfered() {
			informComparedGenomesListeners();
			updateCompPanelTitle();
		}
		
		public void updateCompPanelTitle(){
			comparedPanelTitle.setText("Currently selected species ("+compGenomesPanel.getModel().getSize()+")");
		}
		
		public Collection<Genome> getSelectedGenomes(){
			Collection<Genome> c = new ArrayList<Genome>();
			for(int i=0;i<this.getCompGenomesPanel().getModel().getSize();i++){
				c.add((Genome)(this.getCompGenomesPanel().getModel().getElementAt(i)));
			}
			return c;
		}


		public void listChanged(ListModel tm) {
			System.out.println("Delete clocked");
			informComparedGenomesListeners();
			speciesTreeView.informListeners();
			updateCompPanelTitle();
		}
}
