/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm.preferences;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import org.synteny.Genome;
import org.synteny.ihm.Closable;
import org.synteny.ihm.CloseListener;
import org.synteny.ihm.GenomeSelectionListener;

public class SelectHistogramGenomePanel extends JPanel implements MouseListener,Closable,ActionListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 466983478123152951L;
	Collection<CloseListener> _closeListeners;
	Collection<GenomeSelectionListener> _listeners;
	Collection<Genome> _genomes;
	JTable _genomeTable;

	public SelectHistogramGenomePanel(String titleString){
		super();
		setLayout(new BorderLayout());
		_closeListeners = new ArrayList<CloseListener>();
		_listeners = new ArrayList<GenomeSelectionListener>();
		_genomeTable = new JTable(){
			public boolean isCellEditable(int row, int column)
			{
				return false;
			}
		};
		_genomeTable.setDefaultRenderer(Object.class, new SelectHistogramGenomesCellRenderer());
		//_genomeTable.setAutoCreateRowSorter(true);
		_genomeTable.addMouseListener(this);

		TabLabel title = new TabLabel(titleString);
		title.getCloseButton().addActionListener(this);
		add(title,BorderLayout.NORTH);
		add(new JScrollPane(_genomeTable),BorderLayout.CENTER);
	}

	public void addCloseListener(CloseListener closelistener) {
		_closeListeners.add(closelistener);
	}

	public void close() {

	}

	public void actionPerformed(ActionEvent arg0) {
		for(CloseListener cl : _closeListeners){
			cl.ComponentClosed(this);
		}
	}

	public void addGenomeSelectionListener(GenomeSelectionListener gsl){
		_listeners.add(gsl);
	}

	public void changeGenomeList(Collection<Genome> genomeList){
		DefaultTableModel tm = new DefaultTableModel();
		System.out.println(genomeList.size());
		String[] colnames = {"Abreviation","Name","Taxonomy"};
		tm.setColumnIdentifiers(colnames);
		for(Genome g : genomeList){
			Object[] infos = new Object[3];
			infos[0] = g.getSpeciesAbbrv();
			infos[1] = g;
			infos[2] = g.getSpeciesTree().getTaxonomy().toString();
			tm.addRow(infos);
		}
		_genomes = genomeList;
		_genomeTable.setModel(tm);
	}

	public void mouseClicked(MouseEvent arg0) {
		for(GenomeSelectionListener gl : _listeners){
			gl.referenceGenomeChoosed((Genome)(_genomeTable.getValueAt(_genomeTable.getSelectedRow(),1)),_genomes);
		}
	}

	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}
}
