/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm.preferences;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.ListModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.synteny.Genome;
import org.synteny.SpeciesTree;
import org.synteny.factory.IconFactory;
import org.synteny.ihm.Closable;
import org.synteny.ihm.CloseListener;
import org.synteny.ihm.GenomeSelectionListener;


public class SelectReferenceGenomePanel extends JPanel implements Closable, ActionListener, SpeciesListListener,ListSelectionListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 6707839409796235442L;

	/**
	 * @uml.property   name="speciesTreeView"
	 * @uml.associationEnd   inverse="selectReferenceGenomePanel:org.synteny.ihm.preferences.SpeciesTreeView"
	 * @uml.association   name="hasSpeciesTreeView"
	 */
	private SpeciesTreeView speciesTreeView;

	private Collection<GenomeSelectionListener> _listeners;

	private Collection<CloseListener> _closeListeners;

	
	private TabLabel _titleLabel;
	/**
	 * @uml.property  name="refGenomePanel"
	 * @uml.associationEnd  inverse="selectReferenceGenomePanel:org.synteny.ihm.preferences.RefGenomePanel"
	 * @uml.association  name="hasRefGenomePanel"
	 */
	private RefGenomePanel refGenomePanel;
	
	/**
	 * @uml.property  name="genomeSelectionListener"
	 * @uml.associationEnd  multiplicity="(0 -1)" inverse="selectReferenceGenomePanel:org.synteny.ihm.GenomeSelectionListener"
	 * @uml.association  name="hasSelectionListener"
	 */
	private Collection genomeSelectionListener;
	
	/**
	 * @uml.property  name="selectComparedGenomesPanel"
	 * @uml.associationEnd  inverse="selectReferenceGenomePanel:org.synteny.ihm.preferences.SelectComparedGenomesPanel"
	 */
	private SelectComparedGenomesPanel selectComparedGenomesPanel;

	
	private JButton executeButton;
	private String _panelTitle;
	
	/**
	 */
	public SelectReferenceGenomePanel(SpeciesTree sptree, String paneltitle){
		super();
		_closeListeners = new ArrayList<CloseListener>();
		_listeners = new ArrayList<GenomeSelectionListener>();
		_panelTitle = paneltitle;
		setLayout(new BorderLayout());
		JTabbedPane jtabpane = new JTabbedPane();
		JPanel jp = new JPanel();
		jp.setLayout(new GridLayout(1,2));
		_titleLabel = new TabLabel(paneltitle);
		_titleLabel.getCloseButton().addActionListener(this);
		speciesTreeView = new SpeciesTreeView(sptree);
		refGenomePanel = new RefGenomePanel(sptree);
		refGenomePanel.addListSelectionListener(this);
		//refGenomePanel.addSpeciesListListener(this);
		speciesTreeView.addSpeciesTreeListener(refGenomePanel);
		JScrollPane jsp1 = new JScrollPane(getSpeciesTreeView());
		jsp1.setPreferredSize(new Dimension(200,200));
		JScrollPane jsp2 = new JScrollPane(getRefGenomePanel());
		jsp2.setPreferredSize(new Dimension(200,200));
		jp.add(jsp1);
		jp.add(jsp2);
		jp.setName("Reference");
		
		
		selectComparedGenomesPanel = new SelectComparedGenomesPanel(sptree,"",false);
		selectComparedGenomesPanel.setName("Compared");
		
		jtabpane.add(jp);
		jtabpane.add(selectComparedGenomesPanel);
		
		add(getExecuteButton(),BorderLayout.SOUTH);
		
		add(jtabpane,BorderLayout.CENTER);
		add(_titleLabel,BorderLayout.NORTH);
	}

	/**
	 * Getter of the property <tt>speciesTreeView</tt>
	 * @return  Returns the speciesTreeView.
	 * @uml.property  name="speciesTreeView"
	 */
	public SpeciesTreeView getSpeciesTreeView() {
		return speciesTreeView;
	}

	/**
	 * Setter of the property <tt>speciesTreeView</tt>
	 * @param speciesTreeView  The speciesTreeView to set.
	 * @uml.property  name="speciesTreeView"
	 */
	public void setSpeciesTreeView(SpeciesTreeView speciesTreeView) {
		this.speciesTreeView = speciesTreeView;
	}

	public void addGenomeSelectionListener(GenomeSelectionListener gsl){
		_listeners.add(gsl);
	}
	
	public void refGenomeChoosed(Genome g){
		for(GenomeSelectionListener gsl : _listeners){
				gsl.referenceGenomeChoosed(g,getSelectComparedGenomesPanel().getCompGenomesPanel().getGenomes());
		}
	}

	/**
	 * Getter of the property <tt>genomeSelectionListener</tt>
	 * @return  Returns the genomeSelectionListener.
	 * @uml.property  name="genomeSelectionListener"
	 */
	public Collection getGenomeSelectionListener() {
		return genomeSelectionListener;
	}

	/**
	 * Setter of the property <tt>genomeSelectionListener</tt>
	 * @param genomeSelectionListener  The genomeSelectionListener to set.
	 * @uml.property  name="genomeSelectionListener"
	 */
	public void setGenomeSelectionListener(Collection genomeSelectionListener) {
		this.genomeSelectionListener = genomeSelectionListener;
	}

	public void close() {
	}

	public void addCloseListener(CloseListener closelistener) {
		_closeListeners.add(closelistener);
	}

	public void actionPerformed(ActionEvent e) {
		for(CloseListener cl : _closeListeners){
			cl.ComponentClosed(this);
		}
	}

	public void genomeDoubleClicked(Genome genome) {
		refGenomeChoosed(genome);
	}

	public void GenomeSimpleClicked(Genome genome) {
	}

	/**
	 * Getter of the property <tt>refGenomePanel</tt>
	 * @return  Returns the refGenomePanel.
	 * @uml.property  name="refGenomePanel"
	 */
	public RefGenomePanel getRefGenomePanel() {
		return refGenomePanel;
	}

	/**
	 * Setter of the property <tt>refGenomePanel</tt>
	 * @param refGenomePanel  The refGenomePanel to set.
	 * @uml.property  name="refGenomePanel"
	 */
	public void setRefGenomePanel(RefGenomePanel refGenomePanel) {
		this.refGenomePanel = refGenomePanel;
	}

	public void genomesListTransfered() {
	}


	/**
	 * Getter of the property <tt>selectComparedGenomesPanel</tt>
	 * @return  Returns the selectComparedGenomesPanel.
	 * @uml.property  name="selectComparedGenomesPanel"
	 */
	public SelectComparedGenomesPanel getSelectComparedGenomesPanel() {
		return selectComparedGenomesPanel;
	}

	/**
	 * Setter of the property <tt>selectComparedGenomesPanel</tt>
	 * @param selectComparedGenomesPanel  The selectComparedGenomesPanel to set.
	 * @uml.property  name="selectComparedGenomesPanel"
	 */
	public void setSelectComparedGenomesPanel(
			SelectComparedGenomesPanel selectComparedGenomesPanel) {
		this.selectComparedGenomesPanel = selectComparedGenomesPanel;
	}
	
	public JButton getExecuteButton(){
		if(executeButton == null){
			executeButton = new JButton("Start data retrieval process");
			//executeButton.setBackground(Color.RED);
			executeButton.setIcon(IconFactory.getInstance().getWheelIcon());
			executeButton.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent arg0) {
					if(refGenomePanel.getSelectedValue()!=null && selectComparedGenomesPanel.getSelectedGenomes().size()>0){
						refGenomeChoosed((Genome)refGenomePanel.getSelectedValue());
					}else{
						JOptionPane.showMessageDialog(SelectReferenceGenomePanel.this, "The reference species is not choosed, or no compared species have been selected","Retrieval error",JOptionPane.ERROR_MESSAGE);
					}
				}
				
			});
		}
		return executeButton;
	}

	public void valueChanged(ListSelectionEvent e) {
		if(refGenomePanel.getSelectedValue()==null){
			_titleLabel.setText(_panelTitle+" (none selected) ");
		}else{
		_titleLabel.setText("<html>Selected Synteny species: <U><I>"+refGenomePanel.getSelectedValue()+"</I></U></html>");
		}
	}

	public void listChanged(ListModel tm) {
		// TODO Auto-generated method stub
		
	}
}
