/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm.preferences;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.InvalidPropertiesFormatException;
import java.util.Properties;

public class Settings implements Saveable {
	
	private static Settings _settings;
	private Properties _preferences;
	private File _propFile;
	
	public static int MUSCLE = 0;
	
	
	private Settings(){
		_preferences = new Properties();
		_propFile = new File(System.getProperty("user.home")+"/.synteviewProp");
		load();
	}
	
	public static Settings getInstance(){
		if(_settings==null){
			_settings = new Settings();
		}
		return _settings;
	}
	
	public void load(){
		FileInputStream fi;
		try {
			fi = new FileInputStream(_propFile);
			_preferences.loadFromXML(fi);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvalidPropertiesFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}
	
	public void save(){
		FileOutputStream fo;
		try {
			fo = new FileOutputStream(_propFile);
			_preferences.storeToXML(fo, "SynteView Settings File");
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public String getDBHost(){
		if(_preferences.containsKey("dbhost")){
			return _preferences.getProperty("dbhost");
		}
		return "host.name.com";
	}
	
	public void setDBHost(String host){
		_preferences.setProperty("dbhost", host);
	}
	
	public String getDBUser(){
		if(_preferences.containsKey("dbuser")){
			return _preferences.getProperty("dbuser");
		}
		return "user";
	}
	public void setDBUser(String user){
		_preferences.setProperty("dbuser", user);
	}
	
	public String getDBPassword(){
		if(_preferences.containsKey("dbpass")){
			return _preferences.getProperty("dbpass");
		}
		return "password";
	}
	public void setDBPassword(String pass){
		_preferences.setProperty("pdbpass", pass);
	}
	
	public String getDBName(){
		if(_preferences.containsKey("dbname")){
			return _preferences.getProperty("dbname");
		}
		return "name";
	}
	public void setDBName(String name){
		_preferences.setProperty("dbname", name);
	}
	
	public String getDBPort(){
		if(_preferences.containsKey("dbport")){
			return _preferences.getProperty("dbport");
		}
		return "0000";
	}
	public void setDBPort(String port){
		_preferences.setProperty("dbport", port);
	}
	
	public String getSyntenyQuery(){
		if(_preferences.containsKey("syntenyquery")){
			return _preferences.getProperty("syntenyquery");
		}
		return ("(select bloc.blocid, p1.species_abrv as s1, p2.species_abrv as s2, bloc.pid1 as p1, p1.strand as strand2, p1.start as start1,  p1.stop as stop1,  p1.genename, bloc.pid2 as p2,  p2.strand as strand2,  p2.start as start2,  p2.stop as stop2, p2.genename  from bloc,  protein as p1,  protein as p2  where bloc.pid1 = p1.pid and  bloc.pid2 = p2.pid and  (bloc.abrv1='%spabrv') and (bloc.abrv2 in (%sptab)) union  select bloc.blocid,  p2.species_abrv,  p1.species_abrv, bloc.pid2,  p2.strand as strand1,  p2.start as start1,  p2.stop as stop1,  p2.genename,  bloc.pid1,  p1.strand as strand2,  p1.start as start2,  p1.stop as stop2,  p1.genename  from bloc,  protein as p1,  protein as p2  where bloc.pid1 = p1.pid and  bloc.pid2 = p2.pid and  (bloc.abrv2='%spabrv') and (bloc.abrv1 in (%sptab)))  order by blocid,start1;");
	}
	public void setSyntenyQuery(String query){
		_preferences.setProperty("syntenyquery", query);
	}
	
	public String getSpeciesTreeQuery(){
		if(_preferences.containsKey("speciestreequery")){
			return _preferences.getProperty("speciestreequery");
		}
		return  "SELECT DISTINCT abrv,taxonomy,species,sum(nbprotein) FROM genome group by abrv,taxonomy,species";
	}
	public void setSpeciesTreeQuery(String query){
		_preferences.setProperty("speciestreequery", query);
	}
	
	public String getGenomeQuery(){
		if(_preferences.containsKey("genomequery")){
			return _preferences.getProperty("genomequery");
		}
		return "SELECT pid,genename,strand,start,chromosome,function FROM protein where species_abrv= '%spabrv' order by chromosome,start";
	}
	public void setGenomeQuery(String query){
		_preferences.setProperty("genomequery", query);
	}
	
	public String getProteinInfosQuery(){
		if(_preferences.containsKey("proteininfoquery")){
			return _preferences.getProperty("proteininfoquery");
		}
		return "select * from ((SELECT pid,protein.chromosome, species, function, start, stop, genename, length,distpam,sequence FROM genome,protein left outer join bestortho on ((bestortho.pid_1=-1 and protein.pid=bestortho.pid_2) ) where genome.abrv = protein.species_abrv and pid = %pid) union (SELECT pid,protein.chromosome, species, function, start, stop, genename, length,distpam,sequence FROM genome,protein left outer join bestortho on ((bestortho.pid_2=-1 and protein.pid = bestortho.pid_1) ) where genome.abrv = protein.species_abrv and pid = %pid)) as req order by distpam";
	}
	public void setProteinInfosQuery(String query){
		_preferences.setProperty("proteininfoquery", query);
	}
	
	/**
	 * @return http ou db
	 */
	public String getConnectionMode(){
		if(_preferences.containsKey("connectionmode")){
			return _preferences.getProperty("connectionmode");
		}
		return "http";
	}
	public void setConnectionMode(String mode){
		_preferences.setProperty("connectionmode", mode);
	}

	public String getSyntenyURL() {
		if(_preferences.containsKey("syntenyurl")){
			return _preferences.getProperty("syntenyurl");
		}
		return "http://www.synteview.u-psud.fr/synteGet/bloc2.php?species=%species&comp=%comp";
	}
	public void setSyntenyURL(String url){
		_preferences.setProperty("syntenyurl", url);
	}

	public String getSpeciesTreeURL() {
		if(_preferences.containsKey("speciestreeurl")){
			return _preferences.getProperty("speciestreeurl");
		}
		return "http://www.synteview.u-psud.fr/synteGet/speciesTab2.php";
	}
	public void setSpeciesTreeeURL(String url){
		_preferences.setProperty("speciestreeurl", url);
	}

	public String getGenomeURL() {
		if(_preferences.containsKey("genomeurl")){
			return _preferences.getProperty("genomeurl");
		}
		return "http://www.synteview.u-psud.fr/synteGet/genome2.php?species=%species";
	}
	public void setGenomeURL(String url){
		_preferences.setProperty("genomeurl", url);
	}

	public String getProteinInfosURL() {
		if(_preferences.containsKey("proteininfourl")){
			return _preferences.getProperty("proteininfourl");
		}
		return "http://www.synteview.u-psud.fr/synteGet/pidInfo2.php?pid=%pid";
	}
	public void setProteinInfosURL(String url){
		_preferences.setProperty("proteininfourl", url);
	}
	
	public int getMultipleAlignmentProgram(){
		return Settings.MUSCLE;
	}

	public String getMultipleAlignmentPath(){
		if(_preferences.getProperty("multiplealignmentpath")!=null){
			return _preferences.getProperty("multiplealignmentpath");
		}
		return "/users/bioinfo/lemoine/Desktop/muscle/muscle";
	}
	
	public void setMultipleAlignmentPath(String path){
		System.out.println(path);
		_preferences.setProperty("multiplealignmentpath", path);
	}
}
