/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm.preferences;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.synteny.factory.IconFactory;



public class SettingsDialog extends JDialog implements ActionListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = -7504471003931750403L;

	/**
	 * @uml.property  name="settingsPanel"
	 * @uml.associationEnd  inverse="settingsDialog:org.synteny.ihm.preferences.SettingsPanel"
	 */
	private SettingsPanel settingsPanel;

	JButton _ok,_cancel;
	boolean _good = false; 
	
	private SettingsDialog(Settings settings){
		super();
		((java.awt.Frame)getOwner()).setIconImage(IconFactory.getInstance().getLogoIcon().getImage());
		setModal(true);
		setLayout(new BorderLayout());
		setTitle("Settings dialog");
		_ok = new JButton("Ok");
		_ok.setActionCommand("good");
		_ok.addActionListener(this);
		_cancel = new JButton("Cancel");
		_cancel.setActionCommand("bad");
		_cancel.addActionListener(this);
		settingsPanel = new  SettingsPanel(settings);
		
		JPanel bottom = new JPanel();
		bottom.setLayout(new FlowLayout());
		bottom.add(_ok);
		bottom.add(_cancel);
		
		add(settingsPanel,BorderLayout.CENTER);
		add(bottom,BorderLayout.SOUTH);
		
		//setPreferredSize(new Dimension(300,300));
		pack();
		setLocationRelativeTo(null);
	}
	
	public boolean getOk(){
		return _good;
	}
	
	/**
	 * Getter of the property <tt>settingsPanel</tt>
	 * @return  Returns the settingsPanel.
	 * @uml.property  name="settingsPanel"
	 */
	public SettingsPanel getSettingsPanel() {
		return settingsPanel;
	}

	/**
	 * Setter of the property <tt>settingsPanel</tt>
	 * @param settingsPanel  The settingsPanel to set.
	 * @uml.property  name="settingsPanel"
	 */
	public void setSettingsPanel(SettingsPanel settingsPanel) {
		this.settingsPanel = settingsPanel;
	}

	public void actionPerformed(ActionEvent e) {
		_good = e.getActionCommand().equals("good");
		dispose();
	}
	
	public void save(){
		getSettingsPanel().save();
	}
	
	public static void showDialog(Settings settings){
		SettingsDialog sd = new SettingsDialog(settings);
		sd.setVisible(true);
		if(sd.getOk()){
			sd.save();
		}
	}
	
	public static void main(String[] args) {
		showDialog(Settings.getInstance());
	}

}
