/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm.preferences;

import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;


public class SettingsPanel extends JPanel implements Saveable{

	/**
	 * 
	 */
	private static final long serialVersionUID = 4214033907440671274L;
	/**
	 * @uml.property  name="settings"
	 * @uml.associationEnd  inverse="settingsPanel:org.synteny.ihm.preferences.Settings"
	 */
	private Settings settings;

	/**
	 * @uml.property  name="connectionSettingsPanel"
	 * @uml.associationEnd  inverse="settingsPanel:org.synteny.ihm.preferences.ConnectionSettingsPanel"
	 */
	private ConnectionSettingsPanel connectionSettingsPanel;


	public SettingsPanel(Settings settings){
		super();
		this.settings = settings;
		setLayout(new BorderLayout());
		JTabbedPane jtb = new JTabbedPane();
		jtb.add(getConnectionSettingsPanel());
		//jtb.add(getAlignementSettingsPanel());
		add(jtb,BorderLayout.CENTER);
	}


	/**
	 * Getter of the property <tt>settings</tt>
	 * @return  Returns the settings.
	 * @uml.property  name="settings"
	 */
	public Settings getSettings() {
		return settings;
	}

	/**
	 * Setter of the property <tt>settings</tt>
	 * @param settings  The settings to set.
	 * @uml.property  name="settings"
	 */
	public void setSettings(Settings settings) {
		this.settings = settings;
	}


	/**
	 * Getter of the property <tt>connectionSettingsPanel</tt>
	 * @return  Returns the connectionSettingsPanel.
	 * @uml.property  name="connectionSettingsPanel"
	 */
	public ConnectionSettingsPanel getConnectionSettingsPanel() {
		if (connectionSettingsPanel == null) {
			connectionSettingsPanel = new ConnectionSettingsPanel(getSettings());
		}
		return connectionSettingsPanel;
	}

	/**
	 * Setter of the property <tt>connectionSettingsPanel</tt>
	 * @param connectionSettingsPanel  The connectionSettingsPanel to set.
	 * @uml.property  name="connectionSettingsPanel"
	 */
	public void setConnectionSettingsPanel(
			ConnectionSettingsPanel connectionSettingsPanel) {
		this.connectionSettingsPanel = connectionSettingsPanel;
	}

	public void save(){
		getConnectionSettingsPanel().save();
		getAlignementSettingsPanel().save();
		getSettings().save();
	}

	/**
	 * @uml.property  name="alignementSettingsPanel"
	 * @uml.associationEnd  inverse="settingsPanel:org.synteny.ihm.preferences.AlignementSettingsPanel"
	 */
	private AlignementSettingsPanel alignementSettingsPanel;

	/**
	 * Getter of the property <tt>alignementSettingsPanel</tt>
	 * @return  Returns the alignementSettingsPanel.
	 * @uml.property  name="alignementSettingsPanel"
	 */
	public AlignementSettingsPanel getAlignementSettingsPanel() {
		if(alignementSettingsPanel == null){
			alignementSettingsPanel = new AlignementSettingsPanel();
		}
		return alignementSettingsPanel;
	}


	/**
	 * Setter of the property <tt>alignementSettingsPanel</tt>
	 * @param alignementSettingsPanel  The alignementSettingsPanel to set.
	 * @uml.property  name="alignementSettingsPanel"
	 */
	public void setAlignementSettingsPanel(
			AlignementSettingsPanel alignementSettingsPanel) {
		this.alignementSettingsPanel = alignementSettingsPanel;
	}


}
