/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.ihm.preferences;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JTree;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import org.synteny.Genome;
import org.synteny.SpeciesTree;

/**
 * @uml.dependency   supplier="org.synteny.SpeciesTree"
 */
public class SpeciesTreeView extends JTree implements MouseListener,SpeciesTreeEvent{

	/**
	 * 
	 */
	private static final long serialVersionUID = -4506033213200628556L;

	/**
	 * @uml.property   name="speciesTreeListener"
	 * @uml.associationEnd   multiplicity="(0 -1)" inverse="speciesTreeView:org.synteny.ihm.preferences.SpeciesTreeListener"
	 * @uml.association   name="hasSpeciesTreeListener"
	 */
	private Collection<SpeciesTreeListener> _speciesTreeListener;
	
	private int clickStatus=0;

	public SpeciesTreeView(SpeciesTree spt){
		super(spt);
		//addTreeSelectionListener(this);
		addMouseListener(this);
		_speciesTreeListener = new ArrayList<SpeciesTreeListener>();
	}

	public Collection<Genome> getSelectedGenomes(){
		Collection<Genome> coll = new ArrayList<Genome>();
		if(getSelectionPaths()!=null){
			for(TreePath p : getSelectionPaths()){
				for(Genome g : ((SpeciesTree)(p.getLastPathComponent())).getGenomes()){
					if(!coll.contains(g))
						coll.add(g);
				}
			}
		}
		return coll;
	}


	public SpeciesTree getSelectedSpeciesTree(){
		if(getSelectionPaths()!=null){
			for(TreePath p : getSelectionPaths()){
				return ((SpeciesTree)(p.getLastPathComponent()));
			}
		}
		return null;
	}

	public void mouseClicked(MouseEvent arg0) {
		if(arg0.getClickCount()==2){
			clickStatus = double_click;
			informListeners();
		}else{
			if(arg0.getClickCount()==1){
				clickStatus = simple_click;
				informListeners();
			}
		}
	}

	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	public void mousePressed(MouseEvent e) {
	}

	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	public void addSpeciesTreeListener(SpeciesTreeListener stl){
		_speciesTreeListener.add(stl);
	}

	/**
	 * Getter of the property <tt>speciesTreeListener</tt>
	 * @return  Returns the speciesTreeListener.
	 * @uml.property  name="speciesTreeListener"
	 */
	public Collection getSpeciesTreeListener() {
		return _speciesTreeListener;
	}

	/**
	 * Setter of the property <tt>speciesTreeListener</tt>
	 * @param speciesTreeListener  The speciesTreeListener to set.
	 * @uml.property  name="speciesTreeListener"
	 */
	public void setSpeciesTreeListener(Collection speciesTreeListener) {
		_speciesTreeListener = speciesTreeListener;
	}

	public int getType() {
		return clickStatus;
	}

	public void informListeners(){
		/*if(getSelectedGenomes().size()==1){
			for(SpeciesTreeListener stl : _speciesTreeListener){
				stl.genomeAdded(this,(Genome)(getSelectedGenomes().toArray()[0]));
			}
		}*/

		for(SpeciesTreeListener stl : _speciesTreeListener){
			stl.genomesAdded(this,getSelectedGenomes());
		}
	}
	
	public void setSpeciesTree(SpeciesTree spt){
		this.setModel(new DefaultTreeModel(spt));
	}
}
