/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.saver;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.exceptions.SyntenyNotInitializedException;
import org.synteny.Bloc;
import org.synteny.Comparison;
import org.synteny.Genome;
import org.synteny.Ortholog;
import org.synteny.Paralog;
import org.synteny.Protein;
import org.synteny.Synteny;


public class SyntenySaver {

	private Synteny _synteny;
	private StringBuffer _buffer;
	
	public SyntenySaver(Synteny synteny){
		_synteny = synteny;
		_buffer = new StringBuffer();
	}
	
	public String saveToCSV() throws SyntenyNotInitializedException{
		if(getSynteny().getReferenceSpecies()==null)
			throw new SyntenyNotInitializedException();
		
		StringBuffer buffer = new StringBuffer();
		System.out.println("CSV Référence");
		buffer.append(getCSVRefGenome()+"\n");
		for(Object obj : getSynteny().getComparison()){
			System.out.println("CSV Comp");
			Comparison comp = (Comparison)obj;
			buffer.append(getCSVComparison(comp)+"\n");
		}
		System.out.println("Fin CSV Comp");
		return buffer.toString();
	}
	
	public Synteny getSynteny(){
		return _synteny;
	}
	
	
	public String getCSVRefGenome(){
		_buffer.setLength(0);
		Genome ref = getSynteny().getReferenceSpecies();
		_buffer.append(ref.getSpeciesName());
		for(Object obj : ref.getProtein()){
			Protein prot = (Protein)obj;
			_buffer.append(";"+prot.getName());
		}
		return _buffer.toString();
	}
	
	private String getCSVComparison(Comparison comp){
		int position = 0;
		_buffer.setLength(0);
		_buffer.append(comp.getCompGenome().getSpeciesName());
		List<Bloc> blocList = new ArrayList<Bloc>(comp.getBloc());
		Collections.sort(blocList);
		for(Bloc bloc : blocList){
			for(Object obj : bloc.getParalog()){
				Paralog para = (Paralog)obj;
				for(Object obj2 : para.getOrtholog()){
					Ortholog ortho = (Ortholog)obj2;
					for(int i=position;i<(ortho.getRefProtein().getGenomePosition()-1);i++){
						position++;
						_buffer.append(";");
					}
					position++;
					_buffer.append(";"+ortho.getCompProtein().getName());
				}
			}			
		}
		return _buffer.toString();
	}
}
