/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.search;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.synteny.Protein;
import org.synteny.factory.DatabaseManager;
import org.synteny.factory.DatabaseManagerImpl;
import org.synteny.factory.DatabaseQueryResult;
import org.synteny.ihm.preferences.Settings;


public class DatabaseInfoSearch implements ProteinInfoSearch {

	public DatabaseInfoSearch(){
		databaseManager = new DatabaseManagerImpl();
	}

	public Map search(Protein protein) {
		Map m = null;
		try {
			m = getProteinInfo(protein.getIdentifier());
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return m;
	}

	private Map getProteinInfo(int pid) throws Exception{
		Vector line;
		HashMap<String, String> infos = new HashMap<String, String>();
		DatabaseQueryResult dbqr = sendQuery(pid);

		line = (Vector<Vector>)(dbqr.getDataVector()).elementAt(0);
		infos.put("pid", String.valueOf((Integer)(line.elementAt(0))));
		infos.put("chromosome", (String)(line.elementAt(1)));
		infos.put("genome", (String)(line.elementAt(2)));
		infos.put("function", (String)(line.elementAt(3)));
		infos.put("start", String.valueOf((Integer)(line.elementAt(4))));
		infos.put("stop", String.valueOf((Integer)(line.elementAt(5))));
		infos.put("name", (String)(line.elementAt(6)));
		infos.put("length", String.valueOf((Integer)(line.elementAt(7))));
		infos.put("sequence", String.valueOf((line.elementAt(9))));
		return infos;
	}

	public DatabaseQueryResult sendQuery(int pid) throws Exception{
		String query = Settings.getInstance().getProteinInfosQuery().replace("%pid", String.valueOf(pid));
		DatabaseQueryResult qr = databaseManager.sendQuery(query);
		return qr;
	}

	/**
	 * @uml.property  name="databaseManager"
	 * @uml.associationEnd  inverse="databaseInfoSearch:org.synteny.factory.DatabaseManager"
	 */
	private DatabaseManager databaseManager;

	/**
	 * Getter of the property <tt>databaseManager</tt>
	 * @return  Returns the databaseManager.
	 * @uml.property  name="databaseManager"
	 */
	public DatabaseManager getDatabaseManager() {
		return databaseManager;
	}

	/**
	 * Setter of the property <tt>databaseManager</tt>
	 * @param databaseManager  The databaseManager to set.
	 * @uml.property  name="databaseManager"
	 */
	public void setDatabaseManager(DatabaseManager databaseManager) {
		this.databaseManager = databaseManager;
	}
}
