/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.search;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.synteny.Protein;
import org.synteny.ihm.preferences.Settings;


public class HttpInfoSearch implements ProteinInfoSearch {

	URL _proteinURL;
	HttpURLConnection _server;

	public HttpInfoSearch(){
	}

	public Map search(Protein protein) {
		Map m = null;
		try {
			_proteinURL = new URL(Settings.getInstance().getProteinInfosURL().replace("%pid", String.valueOf(protein.getIdentifier())));
			connect();
			m = getProteinInfo();
			disconnect();
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return m;
	}

	private void connect() throws Exception
	{
		try
		{
			_server = (HttpURLConnection) _proteinURL.openConnection();
			_server.setDoInput(true);
			_server.setDoOutput(true);
			_server.setRequestMethod("GET");
			_server.setRequestProperty("Content-type",
			"application/x-www-form-urlencoded");
			_server.connect();
		}
		catch (Exception e)
		{
			throw new Exception("Connection failed");
		}
	}

	private Map getProteinInfo() throws Exception{
		String line;
		HashMap<String, String> infos = new HashMap<String, String>();
		try
		{
			BufferedReader s = new BufferedReader(
					new InputStreamReader(
							_server.getInputStream()));
			if( s.readLine().contains("!synteview data!")){
			}
			line = s.readLine();
			if(!line.substring(0,1).equals("!")){
				String[] split =line.split("\t");
				infos.put("pid", split[0]);
				infos.put("chromosome", split[1]);
				infos.put("genome", split[2]);
				infos.put("function", split[3]);
				infos.put("start", split[4]);
				infos.put("stop", split[5]);
				infos.put("name", split[6]);
				infos.put("length", split[7]);
				infos.put("sequence", split[9]);
			}
			s.close();
			return infos;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new Exception("Unable to read input stream "+_proteinURL);
		}
	}

	public void disconnect()
	{
		_server.disconnect();
	}
}
