/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.search;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import org.exceptions.SyntenyNotInitializedException;
import org.synteny.Bloc;
import org.synteny.Genome;
import org.synteny.Ortholog;
import org.synteny.Paralog;
import org.synteny.Protein;

public class ProteinSearcherImpl implements ProteinSearcher {

	public Genome getGenome() {
		return _genome;
	}

	public Collection searchProtein(String pattern, boolean function,
			boolean genename) throws SyntenyNotInitializedException {
		System.out.println("Search for proteins like : " + pattern);
		if (getGenome() == null) {
			return new ArrayList<Protein>();
		}

		Collection<Protein> matchingProtein = new HashSet<Protein>();
		for (Object o : getGenome().getProtein()) {
			boolean trouve = false;
			Protein protein = (Protein) o;
			Iterator<Ortholog> it = protein.getOrthologs().iterator();
			while (it.hasNext() && !trouve) {
				Ortholog ortho = it.next();
				if (genename && ortho.getCompProtein().geneNameMatch(pattern)) {
					matchingProtein.add(protein);
					trouve = true;
				}
			}
			if ((function && protein.functionMatch(pattern))
					|| (genename && protein.geneNameMatch(pattern))) {
				System.out
						.println("Protein match : " + protein.getIdentifier());
				matchingProtein.add(protein);
			}
		}
		return matchingProtein;
	}

	public Collection searchBlocProtein(int blocWidth, int blocHeight)
			throws SyntenyNotInitializedException {
		System.out.println("Searching blocks of width>=" + blocWidth
				+ " and height>=" + blocHeight);
		if (getGenome() == null) {
			return new ArrayList<Protein>();
		}

		Collection<Protein> resultProtein = new HashSet<Protein>();
		Collection<Protein> matchingProtein = new HashSet<Protein>();

		List<Protein> proteins = new ArrayList<Protein>(getGenome()
				.getProtein());
		Collections.sort(proteins);

		for (Object o : proteins) {
			int trouve = 0;
			Protein protein = (Protein) o;
			if (!matchingProtein.contains(protein)
					&& protein.getOrthologs().size() >= blocHeight) {
				Iterator<Ortholog> it = protein.getOrthologs().iterator();
				Collection<Protein> tempProtein = new HashSet<Protein>();
				Collection<Protein> tempMatchProtein = new HashSet<Protein>();
				while (it.hasNext() && trouve < blocHeight) {
					Ortholog ortho = it.next();
					Iterator<Bloc> blocIt = ortho.getParalog().getBloc()
							.iterator();
					while (blocIt.hasNext()) {
						Bloc b = blocIt.next();
						if (b.getMaximumSize() >= blocWidth) {
							tempProtein.add(protein);
							tempMatchProtein.add(protein);
							for (Object o1 : b.getParalog()) {
								Paralog p = (Paralog) o1;
								for (Object o2 : p.getOrtholog()) {
									Ortholog ort = (Ortholog) o2;
									tempMatchProtein.add(ort.getRefProtein());
								}
							}
							trouve++;
						}
					}
				}
				if (trouve > 0 && trouve >= (blocHeight - 1)) {
					matchingProtein.addAll(tempMatchProtein);
					resultProtein.addAll(tempProtein);
				}
			}
		}
		return resultProtein;
	}

	public void setGenome(Genome genome) {
		_genome = genome;
	}

	/**
	 * The genome in wich the search will be performed
	 * 
	 * @uml.property name="_genome"
	 */
	private Genome _genome;

	/**
	 * Getter of the property <tt>_genome</tt>
	 * 
	 * @return Returns the _genome.
	 * @uml.property name="_genome"
	 */
	public Genome get_genome() {
		return _genome;
	}

	/**
	 * Setter of the property <tt>_genome</tt>
	 * 
	 * @param _genome
	 *            The _genome to set.
	 * @uml.property name="_genome"
	 */
	public void set_genome(Genome _genome) {
		this._genome = _genome;
	}

	/**
	 */
	public ProteinSearcherImpl() {
	}

	/**
	 */
	public ProteinSearcherImpl(Genome genome) {
		_genome = genome;
	}
}
