/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.task;

import java.util.ArrayList;
import java.util.Collection;

import org.synteny.Genome;
import org.synteny.Protein;
import org.synteny.Synteny;


public class SelectReferenceGenomeTask implements Task {

	Collection<TaskListener> _listeners;
	Genome _newReferenceGenome;
	Synteny _synteny;
	Collection<Genome> _compGenomes;
	Thread _thread;
	int _currentAdvance;
	boolean _started;

	public SelectReferenceGenomeTask(){
		_listeners = new ArrayList<TaskListener>();
		_thread = new Thread(this);
		_currentAdvance = 0;
		_thread.start();
		_compGenomes = new ArrayList<Genome>();
		_started = false;
	}

	public void addTaskListener(TaskListener taskListener) {
		_listeners.add(taskListener);
	}

	public void execute() {
		_started = true;
		_currentAdvance = 0;
	}

	public Collection getTaskListener() {
		return _listeners;
	}

	public void setTaskListener(Collection taskListener) {
		_listeners = taskListener;
	}

	public void run() {
		while(true){
			try{
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				if(_started){
					_synteny.setReferenceSpecies(_newReferenceGenome,_compGenomes);
				}
			}catch(Exception e){
				_started= false;
			}
		}
	}

	public void referenceSpeciesChanged() {
		for(TaskListener tl : _listeners){
			tl.taskAvanced(_synteny.getComparison().size(),_synteny.getComparison().size(), "done");
		}
		_started = false;
	}

	public void speciesBeingAnalysed(String speciesName, int nbGenomes) {
		for(TaskListener tl : _listeners){
			tl.taskAvanced(_currentAdvance, nbGenomes, speciesName);
			_currentAdvance++;
		}
	}

	public void setNewReferenceGenome(Genome genome,Collection<Genome> compGenomes) {
		_newReferenceGenome = genome;
		_compGenomes.clear();
		_compGenomes.addAll(compGenomes);
	}

	public void setSynteny(Synteny synteny) {
		_synteny = synteny;
		if(!_synteny.getSyntenyListener().contains(this)){
			_synteny.addSyntenyListener(this);
		}
	}

	public void ProteinClicked(Protein p) {
		// TODO Auto-generated method stub

	}
}
