/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.task;

import java.awt.Dimension;

import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

import org.synteny.ihm.WaitingCircle;

public class TaskProgressDialog extends JPanel implements TaskListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = -6966741764947473217L;

	/**
	 * @uml.property  name="task"
	 * @uml.associationEnd  inverse="taskProgressDialog:org.synteny.task.Task"
	 */
	private Task task;

	JProgressBar _progress;
	WaitingCircle _wait;
	
	public TaskProgressDialog(Task task){
		setLayout(new BoxLayout(this,BoxLayout.X_AXIS));
		setTask(task);
		task.addTaskListener(this);
		initGui();
	}
	
	private void initGui(){
		_progress = new JProgressBar();
		_progress.setStringPainted(true);
		_wait = new WaitingCircle(20,20,WaitingCircle.CLOCKWISE);
		_wait.setBackgroundColor(this.getBackground());
		_wait.pause();
		setMaximumSize(new Dimension(700,20));
		add(_progress);
		add(_wait);
		setVisible(true);
	}
	
	public void taskAvanced(int current, int total,String s) {
		_progress.setMaximum(total);
		_progress.setValue(current);
		_progress.setString(s);
		
		if(total==current){
			_wait.pause();
		}else{
			_wait.restart();
		}
	}

	/**
	 * Getter of the property <tt>task</tt>
	 * @return  Returns the task.
	 * @uml.property  name="task"
	 */
	public Task getTask() {
		return task;
	}

	/**
	 * Setter of the property <tt>task</tt>
	 * @param task  The task to set.
	 * @uml.property  name="task"
	 */
	public void setTask(Task task) {
		this.task = task;
	}
}
