/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny;

import java.util.Collection;
import java.util.List;


public interface Genome extends Comparable<Genome>{

	/**
	 * @return  Returns the refComparison.
	 * @uml.property  name="refComparison"
	 * @uml.associationEnd  multiplicity="(0 -1)" inverse="refGenome:org.synteny.Comparison"
	 * @uml.association  name="referenceGenome"
	 */
	public Collection getRefComparison();

	/**
	 * Setter of the property <tt>refComparison</tt>
	 * @param refComparison  The refComparison to set.
	 * @uml.property  name="refComparison"
	 */
	public void setRefComparison(Collection refComparison);

	/**
	 * @return  Returns the compComparison.
	 * @uml.property  name="compComparison"
	 * @uml.associationEnd  inverse="compGenome:org.synteny.Comparison"
	 * @uml.association  name="comparisonCom"
	 */
	public Comparison getCompComparison();

	/**
	 * Setter of the property <tt>compComparison</tt>
	 * @param compComparison  The compComparison to set.
	 * @uml.property  name="compComparison"
	 */
	public void setCompComparison(Comparison compComparison);

	/**
	 * @return  Returns the protein.
	 * @uml.property  name="protein"
	 * @uml.associationEnd  multiplicity="(0 -1)" inverse="genome:org.synteny.Protein"
	 * @uml.association  name="hasForGenome"
	 */
	public List getProtein();

	/**
	 * Setter of the property <tt>protein</tt>
	 * @param protein  The protein to set.
	 * @uml.property  name="protein"
	 */
	public void setProtein(List protein);

	/**
	 * @return  Returns the speciesTree.
	 * @uml.property  name="speciesTree"
	 * @uml.associationEnd  inverse="genome:org.synteny.SpeciesTree"
	 * @uml.association  name="hasForTaxonomy"
	 */
	public SpeciesTree getSpeciesTree();

	/**
	 * Setter of the property <tt>speciesTree</tt>
	 * @param speciesTree  The speciesTree to set.
	 * @uml.property  name="speciesTree"
	 */
	public void setSpeciesTree(SpeciesTree speciesTree);


	/**
	 */
	public abstract String getSpeciesName();


	/**
	 */
	public abstract String getSpeciesAbbrv();


	/**
	 */
	public abstract void addProtein(Protein protein);


	/**
	 */
	public abstract int getSize();


	/**
	 */
	public abstract void setSize(int size);




	public abstract double taxonomicSimilarity(Genome genome);


	/**
	 */
	public abstract void addRefComparison(Comparison comparison);


	/**
	 */
	public abstract boolean containsProtein(int pid);


	/**
	 */
	public abstract Protein getProtein(int pid);

	public int getGenomePositionOfProtein(ProteinImpl proteinImpl);

	public void clearProtList();









}
