/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny;

import java.util.Collection;
import java.util.List;
import org.synteny.search.ProteinInfoSearch;


public interface Protein extends Comparable<Protein>{

	/**
	 * @return  Returns the genome.
	 * @uml.property  name="genome"
	 * @uml.associationEnd  inverse="protein:org.synteny.Genome"
	 * @uml.association  name="hasForGenome"
	 */
	public Genome getGenome();

	/**
	 * Setter of the property <tt>genome</tt>
	 * @param genome  The genome to set.
	 * @uml.property  name="genome"
	 */
	public void setGenome(Genome genome);


	/**
	 */
	public abstract int getIdentifier();


	/**
	 */
	public abstract String getFunction();


	/**
	 */
	public abstract boolean getStrand();


	/**
	 */
	public abstract void setStrand(boolean strand);


	/**
	 */
	public abstract String getName();


	/**
	 */
	public abstract void setName(String name);


	/**
	 */
	public abstract int getGenomePosition();

	public abstract int getGenomeStart();

	public abstract int getGenoneStop();

	public abstract void setGenomeStartStop(int start,int stop);

	public String getSequence();
	/**
	 */
	public abstract String getChromosome();


	/**
	 */
	public abstract void setChromosome(String chromosome);

	public int matchInOtherComparisonNumber(List<Genome> genomesList);
	
	public void addMatchInComparison(Genome g);

	public abstract void delete();

	public void setFunction(String function);

	/**
	 * @return  Returns the proteinInfoSearch.
	 * @uml.property  name="proteinInfoSearch"
	 * @uml.associationEnd  inverse="protein:org.synteny.search.ProteinInfoSearch"
	 */
	public ProteinInfoSearch getProteinInfoSearch();

	/**
	 * Setter of the property <tt>proteinInfoSearch</tt>
	 * @param proteinInfoSearch  The proteinInfoSearch to set.
	 * @uml.property  name="proteinInfoSearch"
	 */
	public void setProteinInfoSearch(ProteinInfoSearch proteinInfoSearch);
	
	public Collection<Ortholog> getOrthologs();
	public void addOrtholog(Ortholog ortho);
	
	public String toFasta();

	/**
	 * @param pattern Le pattern que doit matcher la protéine (Nom protéine + Fonction) : Requiert potentiellement connexino à la base de données
	 * @return
	 */
	public boolean functionMatch(String pattern);
	
	/**
	 * @param pattern Le pattern que doit matcher la protéine (juste Nom de la protéine): Pas de connexion supplémentaire à la base de données
	 * @return
	 */
	public boolean geneNameMatch(String pattern);
}
